/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2001.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



/**
 * A plugin that implements a simple clock.
 */
registry.register( new (function() extends ti.chimera.Plugin( main, "Clock" ) {
  
  // *** preferences ***
  var blink = true;
  var milTime  = false;    // true for 12 or false for 24 hr
  var onColor  = java.awt.Color.black;
  var offColor = new java.awt.Color( 193, 179, 120 );
  var bgColor  = new java.awt.Color( 209, 194, 135 );
  // *******************
  
  /**
   * Timer to handle updates every 1 sec.
   */
  var timer = null;
  
  
  /**
   * Initialize/cleanup
   */
  addResource( new (function() extends ti.chimera.Resource(true) {
    
    public function install()
    {
      timer = new swing.Timer( 500, null );
      timer.start();
    }
    
    public function uninstall()
    {
      timer.stop();
      timer = null;
    }
    
  })() );
  
  
  /**
   * The tool-bar that implements the clock.
   */
  function ClockToolBar() extends swing.JToolBar()
  {
    var lcd = new ti.swing.LCDCharacterDisplay( 1, 8, onColor, offColor, bgColor );
    var p = new swing.JPanel();  // lcd in panel, to add extra space around it
    p.setBorder( swing.BorderFactory.createLoweredBevelBorder() );       
    p.add(lcd);
    add(p);
    
    // implement flashing ':' by alternating seperator char:
    var sep = [ ":", (blink?" ":":") ];
    var idx = 0;
    
    timer.addActionListener( new pkg.util.ActionAdapter( "Foo", "Bar", null, function(evt) {
      
      var d = new java.util.Date();
      
      var hours = d.getHours();
      
      if( !milTime )
      {
        hours = hours % 12;
        
        if( hours == 0 )
          hours = 12;
      }
      
      var hr = hours.castToString();
      var mn = d.getMinutes().castToString();
      var sc = d.getSeconds().castToString();
      
      if( hr.length() < 2 ) hr = " " + hr;
      if( mn.length() < 2 ) mn = "0" + mn;
      if( sc.length() < 2 ) sc = "0" + sc;
      
      //if( (mn == 30) && (d.getHours() > 15) )
      //  lcd.updateScreen( [" BEER:30"] );
      //else
        lcd.updateScreen( [hr + sep[idx] + mn + sep[idx] + sc] );
      
      idx = (idx + 1) % sep.length();
      
    } ) );
    
  }
  
  var toolBarResource = new ToolBarResource( function() { return new ClockToolBar(); }, false );
  
  /**
   * Register a service to allow other plugins to make use of the object
   * browser.
   */
  registerServiceFactory( function() {
    
    return new (function() extends ti.chimera.Service("clock") {
      
      /**
       * Display a clock, if not already shown.
       */
      public function show()
      {
        addResource(toolBarResource);
      }
      
      /**
       * Hide the clock, if shown.
       */
      public function hide()
      {
        removeResource(toolBarResource);
      }
      
      public function setEnabled(enabled)
      {
        if(enabled)
          show();
        else
          hide();
      }
      
    })();
    
  } );
  
})() );



/**
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

