/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2001.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * A plugin that lets the user browse the osdoc comments in their src file.
 */
registry.register( new (function() extends ti.chimera.Plugin( main, "Doc Browser" ) {
  
  /**
   * Keep track of the instances of the views.
   */
  var viewTable;
  
  /**
   * Initialize/cleanup
   */
  addResource( new (function() extends ti.chimera.Resource(true) {
    
    public function install()
    {
      viewTable = new java.util.Hashtable();
    }
    
    public function uninstall()
    {
      viewTable = null;
    }
    
  })() );
  
  /**
   * 
   * @param name         the name of the output view
   */
  function DocBrowserView(name)
    extends ti.chimera.View( main, name, null )
  {
    var docTextPane = new pkg.HTMLEditorPane();
    var scrollPane  = new swing.JScrollPane(docTextPane);
    
    docTextPane.setEditable(false);
    
    /**
     * Get the component that makes up this view.
     */
    public function getComponent()
    {
      return scrollPane;
    }
    
    /**
     * Set the contents of this view
     * 
     * @param text   a HTML document (string) to display
     */
    public function setText(text)
    {
      docTextPane.setText(text);
    }
    
    addCloseRunnable( function() {
      viewTable.remove(name);
    } );
  }
  
  /**
   * Register the service provided by this plugin
   */
  registerServiceFactory( function() {
    
    return new (function() extends ti.chimera.Service("doc browser") {
      
      /**
       * Show the html in the view with the specified name.
       * 
       * @param name     the name of the view to display
       * @param str      the html to show
       */
      public function showHtml( name, str )
      {
        var view = viewTable.get(name);
        
        if( view == null )
        {
          view = new DocBrowserView(name);
          viewTable.put( name, view );
          addResource( new ViewResource(view) );
        }
        
        view.setText(str);
      }
      
      /**
       * Show the API docs from the specified script file
       * 
       * @param file     the file to show API of
       */
      public function showApiDocs(file)
      {
        try
          showHtml( pkg.util.getFile(file).getName(), pkg.osdoc.parseToDocument(file) );
        catch(oscript.parser.ParseException e)
          services["prompt"].showErrorMessage("parse error: <br><pre>" + truncate(e.getMessage()) + "</pre>");
      }
      
      /**
       * truncate error msg at a few lines
       */
      private function truncate(msg)
      {
        var idx = 0;
        for( var i=0; (i<5) && (idx >= 0); i++ )
          idx = msg.indexOf( "\n", idx+1 );
        if( idx > 0 )
          msg = msg.substring( 0, idx );
        return msg;
      }
      
    })();
    
  } );
  
})() );



/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

