/******************************************************************************
 *        Copyright Texas Instruments 2002-2004. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * A bookmark object tracks relevant info for a bookmark, such as
 * the editor view, and position within document.
 */
public function Bookmark()
{
  public static const var bookmarkGlyph = 
    pkg.util.getImageIcon("Bookmarks16.gif");
  
  public const var view = getCurrentView();
  private const var frame = view.getFrame();
  public const var position = frame.getBuffer().getDocument().createPosition(
    frame.getBuffer().getCaretPosition()
  );
  
  private const var documentListener = new (function() extends swing.event.DocumentListener() {
    public function insertUpdate(evt)  { updateGutterIcon(); }
    public var removeUpdate  = insertUpdate;
    public var changedUpdate = insertUpdate;
  })();
  
  public var next;
  public var prev;
  
  if( currentBookmark == null )
  {
    next = this;
    prev = this;
  }
  else
  {
    next = currentBookmark.next;
    prev = currentBookmark;
    
    prev.next = this;
    next.prev = this;
  }
  
  /**
   * Bring the view associated with this bookmark to the foreground,
   * and jump to the position of the bookmark within the document.
   */
  public function jumpToBookmark()
  {
    view.toForeground();
    var frame = view.getFrame();
    if( frame != null )
      frame.scrollToOffset( position.getOffset() );
  }
  
  /** Remove this bookmark from the circular-list of bookmarks. */
  public function remove()
  {
    if( next.next == next )
    {
      currentBookmark = null;
    }
    else
    {
      prev.next = next;
      next.prev = prev;
    }
    
    if( line != null )
      frame.removeGutterIcon( line, bookmarkGlyph );
    
    frame.getBuffer().getDocument().removeDocumentListener(documentListener);
  }
  
  /** the last line that the icon was displayed on */
  private var line = null;
  
  private function updateGutterIcon()
  {
    var newLine = frame.getOffsetLine( position.getOffset() );
    
    if( line == newLine )
      return;
    
    frame.addGutterIcon( newLine, bookmarkGlyph );
    
    if( line != null )
      frame.removeGutterIcon( line, bookmarkGlyph );
    
    line = newLine;
  }
  
  updateGutterIcon();
  
  frame.getBuffer().getDocument().addDocumentListener(documentListener);
}
