/******************************************************************************
 *        Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * The editor plugin provides a text editor.
 */
registry.register( new (function() extends ti.chimera.Plugin( main, "Editor" ) {
  
  pkg.registry.setDefaultPersistentValue( 
    "/Dialogs/Editor/bounds",
    new java.awt.Rectangle(330,0,525,675) 
  );
  
  /****************************************************************************
   * Handle actions (menubar, toolbar, etc) that are context sensitive, ie. 
   * only enabled when the in-focus view is an editor view.
   */
  
  var am = new pkg.util.ActionManager();
  var viewActionList = new java.util.LinkedList();
  
  var toolBar = am.getToolBar();
  addResource( new ToolBarResource( toolBar, true ) );
  
  function addGlobalAction( contextSensitive, menuBarPath, name, tip, smallIcon, bigIcon, accel, fxn )
  {
    am.addGlobalAction( contextSensitive, menuBarPath, name, tip, smallIcon, bigIcon, accel, fxn );
    
    if( name != null )
      viewActionList.add( am.createAction( false, name, tip, smallIcon, fxn ) );
  }
  
  if( ! registry.exists("/Preferences/Editor") )
  {
    registry.link( 
      new ti.chimera.registry.PersistentNode( new java.awt.Font( "Monospaced", java.awt.Font.PLAIN, 12 ), 
                                               new ti.chimera.pref.FontNodeContract( ti.chimera.pref.FontNodeContract.MONOSPACE ),
                                               "the font for editor windows" ),
      "/Preferences/Editor/Font" );
    
    registry.link( 
      new ti.chimera.registry.PersistentNode( java.awt.Color.white, 
                                               new ti.chimera.pref.ColorNodeContract(),
                                               "the background color for editor windows" ),
      "/Preferences/Editor/Background Color" );
    
    registry.link( 
      new ti.chimera.registry.PersistentNode( java.awt.Color.black, 
                                               new ti.chimera.pref.ColorNodeContract(),
                                               "the foreground color for editor windows" ),
      "/Preferences/Editor/Foreground Color" );
  }
  if( ! registry.exists("/Editor") )
  {
    registry.link( 
      new ti.chimera.registry.PersistentNode( new java.util.HashMap(), 
                                               new ti.chimera.registry.TypeNodeContract( java.util.Map ),
                                               "table of cursor positions, maping file to [line, column]" ),
      "/Editor/Cursor Position Table" );
  }
  
  /***************************************************************************/
  // open/save actions:
  {
    addGlobalAction( false, "/File/New/Script File...",  "New",                 "Open new file",
                     "New16.gif",                        "New16.gif",
                     "N",  function(evt) { services["editor"].newFile(null); } );
    addGlobalAction( true,  "/File/Save",                "Save (Cntl-S)",       "Save this file",
                     "Save16.gif",                       "Save16.gif",
                     "S",  function(evt) { pkg.editor.EditorFrame.getCurrentFrame().save(); } );
    addGlobalAction( true,  "/File/Save As",             "Save As",             "Save this file",
                     "SaveAs16.gif",                     "SaveAs16.gif",
                     null, function(evt) {
                       var frame = pkg.editor.EditorFrame.getCurrentFrame();
                       frame.saveAs( chooseFile( frame.getFile(), true ) );
                     } );
    addGlobalAction( true,  null,                        "Reload",              "Reload this file",
                     "Refresh16.gif",                    "Refresh16.gif",
                     null, function(evt) { pkg.editor.EditorFrame.getCurrentFrame().refresh(); } );
    toolBar.addSeparator();
  }
  // cut/copy/paste actions:
  {
    addGlobalAction( true,  "/Edit/Cut",                 "Cut",                 "Cut the selected text",
                     "Cut16.gif",                        "Cut16.gif", 
                     "X",  function(evt) { pkg.editor.EditorFrame.getCurrentFrame().cut(); } );
    addGlobalAction( true,  "/Edit/Copy",                "Copy",                "Copy the selected text",
                     "Copy16.gif",                       "Copy16.gif", 
                     "C",  function(evt) { pkg.editor.EditorFrame.getCurrentFrame().copy(); } );
    addGlobalAction( true,  "/Edit/Paste",               "Paste",               "Paste the selected text",
                     "Paste16.gif",                      "Paste16.gif", 
                     "V",  function(evt) { pkg.editor.EditorFrame.getCurrentFrame().paste(); } );
    windowManager.addMenuBarItem( "/Edit", null );
    toolBar.addSeparator();
  }
  // editing actions:
  {
    addGlobalAction( true,  "/Edit/Increase Indent",     "Increase Indent",     "Increase indent of selected text",
                     "shift_right.gif",                  "shift_right.gif",
                     null, function(evt) { pkg.editor.EditorFrame.getCurrentFrame().getMode().performAction("increase-indent-level"); } );
    addGlobalAction( true,  "/Edit/Decrease Indent",     "Decrease Indent",     "Decrease indent of selected text",
                     "shift_left.gif",                   "shift_left.gif",
                     null, function(evt) { pkg.editor.EditorFrame.getCurrentFrame().getMode().performAction("decrease-indent-level"); } );
    windowManager.addMenuBarItem( "/Edit", null );
    addGlobalAction( true,  "/Edit/Comment Region",      "Comment Region",      "Comment out selected region",
                     "comment.gif",                      "comment.gif",
                     null, function(evt) { pkg.editor.EditorFrame.getCurrentFrame().getMode().performAction("comment-region"); } );
    addGlobalAction( true,  "/Edit/Uncomment Region",    "Uncomment Region",    "Uncomment selected region",
                     "uncomment.gif",                    "uncomment.gif",
                     null, function(evt) { pkg.editor.EditorFrame.getCurrentFrame().getMode().performAction("uncomment-region"); } );
    windowManager.addMenuBarItem( "/Edit", null );
    toolBar.addSeparator();
  }
  // formatting actions:   XXX get better icons!
  {
    am.addGlobalAction( true,  null, "Bold",   "Insert <b></b>",       "Bold16.gif",   "Bold16.gif",
                        "B",  function(evt) { pkg.editor.cMode.insertTags("<b>","</b>"); } );
    am.addGlobalAction( true,  null, "Italic", "Insert <i></i>",       "Italic16.gif", "Italic16.gif", 
                        "I",  function(evt) { pkg.editor.cMode.insertTags("<i>","</i>"); } );
    am.addGlobalAction( true,  null, "Code",   "Insert <code></code>", "Normal16.gif", "Normal16.gif",
                        "T",  function(evt) { pkg.editor.cMode.insertTags("<code>","</code>"); } );
    am.addGlobalAction( true,  null, "Link",   "Insert {@link }",      "Underline16.gif", "Underline16.gif",
                        null, function(evt) { pkg.editor.cMode.insertTags("{@link ","}"); } );
    //windowManager.addMenuBarItem( "/Edit", null );
    toolBar.addSeparator();
  }
  // undo/redo actions:
  {
    addGlobalAction( true,  "/Edit/Undo",                "Undo (Cntl-Z)",       "Undo last edit",
                     "Undo16.gif",                       "Undo16.gif",
                     "Z",  function(evt) { pkg.editor.EditorFrame.getCurrentFrame().undo(); } );
    addGlobalAction( true,  "/Edit/Redo",                "Redo (Cntl-Shift-Z)", "Redo last undo",
                     "Redo16.gif",                       "Redo16.gif",
                     null, function(evt) { pkg.editor.EditorFrame.getCurrentFrame().redo(); } );
    windowManager.addMenuBarItem( "/Edit", null );
    toolBar.addSeparator();
  }
  // bookmark actions:
  {
    addGlobalAction( true,  "/Edit/Toggle Bookmark",     "Toggle Bookmark (Cntl-???)","toggle bookmark at cursor",
                     "Bookmarks16.gif",                  "Bookmarks16.gif",
                     null, function(evt) { services["editor"].toggleBookmark(); } );
    addGlobalAction( true,  "/Edit/Previous Bookmark",   "Previous Bookmark (Cntl-???)",  "jump to previous bookmark",
                     "Back16.gif",                       "Back16.gif",
                     null, function(evt) { services["editor"].prevBookmark(); } );
    addGlobalAction( true,  "/Edit/Next Bookmark",       "Next Bookmark (Cntl-???)",  "jump to next bookmark",
                     "Forward16.gif",                    "Forward16.gif",
                     null, function(evt) { services["editor"].nextBookmark(); } );
    windowManager.addMenuBarItem( "/Edit", null );
    toolBar.addSeparator();
  }
  // misc actions:
  {
    addGlobalAction( true,  "/Edit/Goto Line (F5)",      null,                  "Prompt and then scroll to line",
                     null,                               null,
                     null, function(evt) { pkg.editor.util.promptAndGotoLine(); } );
    windowManager.addMenuBarItem( "/Edit", null );
    addGlobalAction( true,  null,                        "Help",                "Show Help",
                     "Help16.gif",                       "Help16.gif",
                     null, function(evt) { services["help"].displayHelp(HELP_PATH); } );
    
    am.addGlobalAction( true, "/Debugger/Remove All Breakpoints", null, null, null, null, null, '{
      services["debugger"].clearBreakpoint( null, null );
    } );
    am.addGlobalAction( true, "/Debugger/Remove All Breakpoints in File", null, null, null, null, null, '{
      services["debugger"].clearBreakpoint( pkg.editor.EditorFrame.getCurrentFrame().getFile(), null );
    } );
    windowManager.addMenuBarItem( "/Debugger", null );
  }
  /***************************************************************************/
  
  const var needsSaveIcon = pkg.util.getImageIcon("breakpointglyph.gif"); // XXX for now
  
  /**
   * Table mapping wizard name to wizard constructor function.
   */
  var wizardTable = new java.util.Hashtable();
  
  /**
   * Register a wizard for creating a new document of some sort.
   * To register a wizard, provide a function that constructs and 
   * returns the wizard.  This function takes as parameters a close 
   * function, to close the dialog the wizard is displayed in, and 
   * the editor frame that should contain the text that the wizard 
   * generates.
   * 
   * @param name         the wizard name
   * @param pathOrFxn    a function, or string to evaluate to a
   *    function, to construct and return the new wizard object
   */
  private function registerWizard( name, pathOrFxn )
  {
    var prev = wizardTable.put( name, pathOrFxn );
    
    /* Don't add menubar entry again, if one already exists, otherwise we'll 
     * end up calling newFileFromWizard once for each time the menubar entry is 
     * registered
     */
    if( prev == null )
    {
      am.addGlobalAction( false, "/File/New/" + name, 
                          null, null, null, null, null, 
                          function(evt) {
                            services["editor"].newFileFromWizard(name);
                          } );
    }
  }
  /****************************************************************************
   * The "editor" service.
   */
  registerServiceFactory( function() {
    
    return new (function() extends ti.chimera.Service("editor") {
      
      public var registerWizard = registerWizard;
      
      /**
       * Activate the named wizard to create a new document.
       * 
       * @param name   the wizard name
       */
      public function newFileFromWizard(name)
      {
        var pathOrFxn = wizardTable.get(name);
        if( pathOrFxn != null )
        {
          var file = chooseFile( null, false );
          
          if( file == null )
            return;
          
          if( ! file.exists() )
            file.createNewFile();
          else if( (file.length() > 0) && ("Cancel" == services["prompt"].askQuestion("File exists!",["Cancel","Overwrite"])) )
            return;
          
          var view = getEditorView(file);
          view.toForeground();
          
          var fxn = (pathOrFxn instanceof String) ? evalHelper(pathOrFxn) : pathOrFxn;
          var frame  = view.getFrame();
          var dialog = windowManager.getDialog(name + " Wizard");
          var wizard = fxn( function() {
            dialog.setVisible(false);
            dialog.dispose();
          }, frame.getDocumentWriter( 0, frame.getBuffer().getDocument().getLength() ) );
          
          dialog.getContentPane().add(wizard);
          dialog.pack();
          dialog.center();
          dialog.setVisible(true);
        }
      }
      
      /**
       */
      public function activateWizard( name, docwriter )
      {
        // XXXX
      }
      
      /**
       * Toggle the bookmark at the current cursor position in the current
       * editor window.  If there is currently a bookmark at the current
       * position, then remove it, else create one.
       */
      public function toggleBookmark()
      {
        var view  = getCurrentView();
        var frame = view.getFrame();
        var off   = frame.getBuffer().getCaretPosition();
        var line  = frame.getOffsetLine(off);
        
        for( var bookmark : bookmarkSet )
        {
          if( (bookmark.view == view) && 
              (frame.getOffsetLine( bookmark.position.getOffset() ) == line) )
          {
            bookmark.remove();
            return;
          }
        }
        
        // if we get here, we didn't find a bookmark to remove, so add one:
        currentBookmark = new Bookmark();
      }
      
      /**
       * Jump to the next bookmark in the bookmark ring.
       */
      public function nextBookmark()
      {
        if( currentBookmark != null )
        {
          currentBookmark = currentBookmark.next;
          currentBookmark.jumpToBookmark();
        }
      }
      
      /**
       * Jump to the previous bookmark in the bookmark ring.
       */
      public function prevBookmark()
      {
        if( currentBookmark != null )
        {
          currentBookmark = currentBookmark.prev;
          currentBookmark.jumpToBookmark();
        }
      }
      
      /**
       * Create a new file.
       * 
       * @param initialText  string containing initial text, or <code>null</code>
       */
      public function newFile(initialText)
      {
        var file = new pkg.fs.TempFile("untitled",initialText);
        var view = getEditorView(file);
        view.toForeground();
      }
      
      /**
       * Open an existing file.
       * 
       * @param file         the file to open
       */
      public function openFile(file)
      {
        file = pkg.util.getFile(file);
        
        if( (file == null) || !file.exists() )
          file = chooseFile( file, false );
        
        if( file == null )
          return;
        
        if( ! file.exists() )
          file.createNewFile();
        
        var view = getEditorView(file);
        view.toForeground();
      }
      
      /**
       * Open an existing file, scroll to specified line number.
       * 
       * @param file         the file to open
       * @param line         the line to scroll to
       */
      public function openFileToLineNum( file, line )
      {
        openFile( file=pkg.util.getFile(file) );
        var view  = getEditorView(file);
        view.invokeWhenFrameHasFocus( '{
          view.getFrame().scrollToLine(line);
        } );
      }
      
      /**
       * Add a glyph-gutter icon for the specified file on the specified line
       * number.  If the file isn't already open, this will open it.  The
       * editor plugin will track icons assigned to a particular file in case 
       * the file is closed and re-opened at a later time.
       * 
       * @param file         the file
       * @param line         the line number within the file
       * @param icon         a {@link javax.swing.Icon}
       * @see #removeGutterIcon
       */
      public function addGutterIcon( file, line, icon )
      {
        file = pkg.util.getFile(file);
        
        if( file.exists() )
          getEditorView(file).getFrame().addGutterIcon( line, icon );
        
        var lineTable = gutterIconTable.get(file);
        if( lineTable == null )
        {
          lineTable = new java.util.Hashtable();
          gutterIconTable.put( file, lineTable );
        }
        
        var iconList = lineTable.get(line);
        if( iconList == null )
        {
          iconList = new java.util.LinkedList();
          lineTable.put( line, iconList );
        }
        
        iconList.add(icon);
      }
      
      /**
       * Remove a glyph-gutter icon from the specified file at the
       * specified line number.  
       * 
       * @param file         the file
       * @param line         the line number within the file
       * @param icon         a {@link javax.swing.Icon}
       * @see #addGutterIcon
       */
      public function removeGutterIcon( file, line, icon )
      {
        file = pkg.util.getFile(file);
        
        if( file.exists() )
          getEditorView(file).getFrame().removeGutterIcon( line, icon );
        
        var lineTable = gutterIconTable.get(file);
        if( lineTable == null )
          return;
        
        var iconList = lineTable.get(line);
        if( iconList == null )
          return;
        
        iconList.remove(icon);
        
        if( iconList.size() == 0 )
          lineTable.remove(line);
        
        if( lineTable.size() == 0 )
          lineTable.remove(file);
      }
      
    })();
  } );
  
  var editorTable = null;
  var gutterIconTable = null;   // maps  file -> { line -> icon }
  var currentBookmark = null;
  
  /**
   * Initialize/cleanup
   */
  addResource( new (function() extends ti.chimera.Resource(true) {
    
    public function install()
    {
      editorTable = new java.util.Hashtable();
      gutterIconTable = new java.util.Hashtable();
    }
    
    public function uninstall()
    {
      editorTable = null;
      gutterIconTable = null;
      currentBookmark = null;
    }
    
  })() );
  
  mixin pkg.system.makePrivatePackage("/jar/ode/EditorPlugin/");
  
  
  
  /**
   * A utility function to find the view that corresponds to the current
   * editor frame.
   * 
   * @return the editor-view
   */
  private function getCurrentView()
  {
    for( var file : editorTable.keySet() )
      if( editorTable.get(file).getFrame() == pkg.editor.EditorFrame.getCurrentFrame() )
        return editorTable.get(file);
    
    throw new Exception("hmm, couldn't find the view... this probably shouldn't happen");
  }
  
  
  /**
   * Get an editor view.  If it doesn't exist, create a new one and add it.
   * 
   * @param file         the file to open
   */
  private function getEditorView(file)
  {
    /* because the function we pass in to the "swing worker" can end
     * up being called from a different thread context, we can't
     * hold a lock while we call run().  But we don't want to bother
     * having to use the "swing worker" in the case of view already
     * in table... that is the reason for the sorta bizarre logic.
     */
    var view = editorTable.get(file);
    
    if( view == null )
    {
      services["swing worker"].run( function() {
        
//        synchronized(editorTable)
        {
          // have to check again within the synchronization
          // to avoid a race condition:
          view = editorTable.get(file);
          
          if( view == null )
          {
            view = new EditorView(file);
            
            var lineTable = gutterIconTable.get(file);
            if( lineTable != null )
              for( var line : lineTable.keySet() )
                for( var icon : lineTable.get(line) )
                  view.getFrame().addGutterIcon( line, icon );
            
            addResource( new ViewResource(view) );
            editorTable.put( file, view );
          }
        }
        
      }, "Launching Editor" );
    }
    
    if( file.getName() != "untitled" )
      services["file browser"].addToRecentItems(file);
    
    return view;
  }
  
  /**
   * Display a file chooser, with the specified initial path.  Return the
   * path that the user ends up selecting.
   * 
   * @param file         the initial file, or <code>null</code>
   * @param save         <code>true</code> for save dialog, <code>false</code> for open
   * @return the chosen path, or <code>null</code> if user cancelled
   */
  private function chooseFile( file, save )
  {
    var chooser = new ti.chimera.FileChooser( windowManager.getDialog("Select File") );
    
    if( file != null )
      chooser.setSelectedFile(file);
    
    chooser.setFileFilter( new pkg.util.ExtensionFileFilter( "os", "ObjectScript Source File" ) );
    
    if( (save ? chooser.showSaveDialog(null) : chooser.showOpenDialog(null))
        == ti.chimera.FileChooser.APPROVE_OPTION )
      return chooser.getSelectedFile();
    else
      return null;
  }
  
  registry.subscribeToValue( 
    "/Docks/selected dock/selected view", null,
    function( node, view ) {
      
      am.setEnabled( view.getType().getName() == "EditorView" );
      
  } );
  
  const var HELP_PATH = "Plugins/Editor";
  services["help"].registerHelp( HELP_PATH, "editor/EditorFrameHelp.html", "Help for the Script Editor" );
  
})() );


var SUPPORTED_FILE_TYPES = [ 
  ["os", "ObjectScript Source Files"], 
  ["txt", "Text Files"], 
  ["html", "HTML Files"], 
  ["xml", "XML Files"],
  ["osp", "ObjectScript Servlet Pages"],
  ["java", "Java Files"],
  ["c",    "C Files"]
];  // XXX move this
for( var type : SUPPORTED_FILE_TYPES )
{
  services["file browser"].
    setDefaultAction( type[0], new pkg.util.ActionAdapter( "edit", null, null, function(evt) {
      pkg.cl.edit( evt.getSource() );
    } ) );
  services["file browser"].
    setDescription( type[0], type[1] );
}

// XXX not sure where this should go:
services["file browser"].
  setDefaultAction( "/any/", new pkg.util.ActionAdapter( "search", null, null, function(evt) {
    var file = evt.getSource();
    var regexp = services["prompt"].promptForString( "enter search string", "", null );
    if( regexp != null )
    {
      services["swing worker"].run( '{
        var results = pkg.cl.getMember( file.isDirectory() ? "grepr" : "grep" )( regexp, file );
        writeln("\n" + results + "\n"); // XXX eventually display in a nice search-results dialog
      }, "searching" );
    }
  } ) );

/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
