/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2001.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



/**
 * A plugin that displays a toolbar that shows the current heap size and the
 * amount of heap used.
 */
registry.register( new (function() extends ti.chimera.Plugin( main, "Free Memory" ) {
  
  // *** preferences ***
  var onColor  = java.awt.Color.black;
  var offColor = new java.awt.Color( 193, 179, 120 );
  var bgColor  = new java.awt.Color( 209, 194, 135 );
  // *******************
  
  /**
   * Timer to handle updates every 1 sec.
   */
  var timer = null;
  
  
  /**
   * Initialize/cleanup
   */
  addResource( new (function() extends ti.chimera.Resource(true) {
    
    public function install()
    {
      timer = new swing.Timer( 500, null );
      timer.start();
    }
    
    public function uninstall()
    {
      timer.stop();
      timer = null;
    }
    
  })() );
  
  var units = [ "b", "k", "m", "g" ];
  
  /**
   * Convert a number of bytes to a string.  1024 bytes become
   * 1k, 1024k becomes 1m, 1024m becomes 1g.
   */
  function bytesToString(n)
  {
    for( var i=0; i<units.length(); i++ )
    {
      if( (n / 1024) < 1 )
        return n + units[i];
      n = n / 1024;
    }
    return n + units[ units.length() - 1 ];
  }
  
  const var LCD_LENGTH = 9;
  
  /**
   * The tool-bar that implements the display.
   */
  function FreeMemoryToolBar() extends swing.JToolBar()
  {
    // add the LCD which displays memory status:
    var lcd = new ti.swing.LCDCharacterDisplay( 1, LCD_LENGTH, onColor, offColor, bgColor );
    var p = new swing.JPanel();  // lcd in panel, to add extra space around it
    p.setBorder( swing.BorderFactory.createLoweredBevelBorder() );
    p.add(lcd);
    add(p);
    
    var r = Runtime.getRuntime();
    
    timer.addActionListener( new pkg.util.ActionAdapter( "Foo", "Bar", null, function(evt) {
      
      var free  = r.freeMemory();
      var total = r.totalMemory();
      
      var str = bytesToString(total-free) + "/" + bytesToString(total);
      while( str.length() < LCD_LENGTH )
        str = " " + str;
      
      lcd.updateScreen( [ str ] );
      
    } ) );
    
    // add the GC button:
    add( new swing.JButton( new pkg.util.ActionAdapter( "GC",
                                                        "Run Garbage Collection",
                                                        null,
                                                        function(evt) {
                                                          System.gc();
                                                        } ) ) );
  }
  
  var toolBarResource = new ToolBarResource( function() { return new FreeMemoryToolBar(); }, false );
  
  /**
   * Register a service to allow other plugins to make use of the object
   * browser.
   */
  registerServiceFactory( function() {
    
    return new (function() extends ti.chimera.Service("free memory") {
      
      /**
       * Enable/disable the plugin.
       */
      public function setEnabled(enabled)
      {
        (enabled ? addResource : removeResource)(toolBarResource);
      }
      
    })();
    
  } );
  
})() );



/**
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

