/*=============================================================================
 *     Copyright Texas Instruments 2001. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* 
 * Set things up so the system knows about the "oscript" URL type... this is
 * sorta a hack, because it doesn't really fit the traditional definition of
 * what a URL is, but is needed due to how java.net.URL works.
 */
var prop = "java.protocol.handler.pkgs";
var val  = "ti.protocols"; // + System.getProperty(prop);
System.setProperty( prop, val );


/**
 * Basically an JEditorPane with some customizations to handle HTML with
 * embedded script, etc.
 */
public function HTMLEditorPane() extends swing.JEditorPane()
{
  /* 
   * Rather than using a "normal" HTMLEditorKit, we need to create one that 
   * uses it's own ViewFactory to create views for certain types of elements 
   * (such as FORM), in order to intercept events and eval embedded script 
   * code where appropriate.
   */
  setEditorKit( new (function() extends swing.text.html.HTMLEditorKit() {
    
    const var NAME = swing.text.html.HTML.getAttributeKey("name");
    
    /**
     * The view factory is provided by the editor kit to create views for
     * elements.
     */
    var viewFactory = new (function() extends swing.text.ViewFactory() {
      
      /**
       * The "normal" ViewFactory is used for most elements.  It is kinda weird 
       * to do it this way, rather than subclass HTMLEditorKit's inner-class 
       * HTMLFactory, but the script interpreter currently has limitations when 
       * it comes to access to (and thus subclassing) inner-classes.
       */
      var defaultViewFactory = getViewFactory();
      
      /**
       * Our own FormView implementation... which handles script.
       */
      function ScriptFormView(elem) extends swing.text.html.FormView(elem)
      {
        var _super_actionPerformed = actionPerformed;
        
        public function actionPerformed(evt)
        {
          var script = elem.getAttributes().getAttribute("onclick");
          if( script != null )
            scripting.evalScript( sanitizeScriptStr(script) );
          
          // hack, to work around focus problems seen with JDK v1.4 beta 2, 3
//           evt.getSource().requestFocus();
          
          _super_actionPerformed(evt);
        }
        
        // hacky, but a way to get access to the actual component w/o having
        // to subclass package-private or protected methods:
        var exportedComponent = false;
        var _super_setParent = setParent;
        public function setParent(p)
        {
          _super_setParent(p);
          
          if( !exportedComponent )
          {
            var name = elem.getAttributes().getAttribute(NAME);
            var c    = getComponent();
            
            if( c instanceof swing.JScrollPane )
              c = c.getViewport().getView();
            
            if( c instanceof swing.JTextField )
            {
              c.getDocument().addDocumentListener( new (function() extends swing.event.DocumentListener() {
                
                public function insertUpdate(evt)
                {
                  pkg.output.writeln("requesting focus");
                  c.requestFocus();
                }
                public var removeUpdate  = insertUpdate;
                public var changedUpdate = insertUpdate;
                
              })() );
            }
            
            if( c != null )
            {
              if( name != null )
                scripting.exportObj( name, c );
              exportedComponent = true;
            }
          }
        }
      }
      
      /**
       * Creates a view from the given structural element of a document.
       *
       * @param elem  the piece of the document to build a view of
       * @return the view
       * @see View
       */
      public function create(elem)
      {
        var kind = elem.getAttributes().
          getAttribute( swing.text.StyleConstants.NameAttribute ).toString();
        
        if( (kind == "input") ||
            (kind == "select") ||
            (kind == "textarea") )
          return new ScriptFormView(elem);
        else
          return defaultViewFactory.create(elem);
      }
      
    })();
    
    /**
     * Fetch a factory that is suitable for producing views of any models that 
     * are produced by this kit.  
     *
     * @return the factory
     */
    public function getViewFactory()
    {
      return viewFactory;
    }
    
  })() );
  
  
  var url = null;
  
  /**
   * Set the currently visible url.
   * 
   * @param url          an instance of java.net.URL
   */
  public function setURL(url)
  {
    scripting.reset();
    loadURL(url);
  }
  
  /* if we are actually loading a URL, postpone the new load request...
   * not really thread safe, but everything should be happening from the
   * context of same thread anyways.  (Think <SCRIPT> in document which
   * triggers loading a new document.)
   */
  var loadingURL = false;
  var postponedURL = null;
  
  function loadURL(url)
  {
    if( !loadingURL )
    {
      postponedURL = null;
      loadingURL = true;
      
      this.url = url;
      
      // This method of doing things is fairly undocumented, but seems to
      // work and avoids loading the document asynchronously, which was
      // causing all sorts of fun race conditions:
      var editorKit = getEditorKit();
      var doc = editorKit.createDefaultDocument();
      doc.putProperty( swing.text.Document.StreamDescriptionProperty, url );
      editorKit.read( new java.io.InputStreamReader( url.openStream() ), doc, 0 );
      expandScript(doc);
      var title = doc.getProperty( swing.text.Document.TitleProperty );
      if( title != null )
        setName(title);
      setDocument(doc);
      
      loadingURL = false;
      
      if( postponedURL != null )
        loadURL(postponedURL);
    }
    else
    {
      postponedURL = url;
    }
  }
  
  
  /**
   * Get the title of the current document.
   * 
   * @return a string, or <code>null</code>
   */
  public function getTitle()
  {
    var doc = getDocument();
    
    if( doc != null )
      return doc.getProperty( swing.text.Document.TitleProperty );
    else
      return null;
  }
  
  
  /**
   * Override firing of hyperlink events to intercept and handle script
   * events.
   * 
   * @param evt          the event to dispatch
   */
  var _super_fireHyperlinkUpdate = fireHyperlinkUpdate;
  public function fireHyperlinkUpdate(evt)
  {
    // intercept URL's of the form 'oscript:':
    if( (evt.getEventType().toString() == "ACTIVATED") &&
        (evt.getURL().getProtocol() == "oscript") )
    {
      var str = evt.getURL().getPath();
      scripting.evalScript( sanitizeScriptStr(str) );
    }
    else
    {
      _super_fireHyperlinkUpdate(evt);
    }
  }
  
  /**
   * Utility fxn... any embedded script from a tag (ie anchor or form element)
   * passes thur here to transform single->double quote, make sure ends w/ ;
   * etc.
   */
  function sanitizeScriptStr(str)
  {
    var idx;
    
    while( (idx=str.indexOf("'")) != -1 )
      str = str.substring( 0, idx ) + "\"" + ((idx < str.length())?str.substring(idx+1):"");
    
    //var str = str.strip();
    if( (str[ str.length() - 1 ] != ";") && 
        (str[ str.length() - 1 ] != "}") )
      str += ";";
    return str;
  }
  
  /**
   * Given an <code>HTMLDocument</code>, find and evaluate any embedded 
   * script.  This is pretty lame, but we have to do it because swing's
   * HTML parsing doesn't support embedded script, which the autogenerated
   * javadoc pages need.  (It is actually the add-on UML stuff that needs
   * it!)
   */
  function expandScript(doc)
  {
    var language = null;
    var script   = null;
    
    // I figure this is easy enough that we don't need a state machine... 
    function handleElement(element)
    {
      if( element == null ) // XXX work around for some problem... not entirely sure why we get here!
        return;
      
      if( element.isLeaf() )
      {
        // we basically have two states, we are either between a
        // <script> and </script>, or we aren't
        if( script == null )
        {
          if( element.getName() == "script" )
          {
            if( getAttribute( element, "endtag" ) == null )
            {
              language = getAttribute( element, "language" );
              script   = "";
            }
          }
        }
        else
        {
          if( element.getName() == "content" )
          {
            var start = element.getStartOffset();
            var end   = element.getEndOffset();
            
            script += doc.getText( start, end - start );
          }
          else if( element.getName() == "comment" )
          {
            script += getAttribute( element, "comment" );
          }
          else if( element.getName() == "script" )
          {
            if( getAttribute( element, "endtag" ) != null )
            {
              // XXX should do a case-insensitive compare:
              if( language == "ObjectScript" )
                handleObjectScript( element, script );
              
              language = script = null;
            }
          }
        }
      }
      else
      {
        var cnt = element.getElementCount();
        for( var i=0; i<cnt; i++ )
        {
          handleElement( element.getElement(i) );
        }
      }
    }
    
    // this is lame, but we don't have an easy way to specify an attribute
    // by string name without iterating through all the attributes of an
    // element:  XXX maybe HTML.getAttributeKey() would work???
    function getAttribute( element, attrName )
    {
      var attr = element.getAttributes();
      
      for( var e=attr.getAttributeNames(); e.hasMoreElements(); )
      {
        var name = e.nextElement();
        
        if( name.toString() == attrName )
          return attr.getAttribute(name);
      }
      
      return null;
    }
    
    function handleObjectScript( element, script )
    {
      scripting.evalScript(script);
      scripting.evalScript("document.flush;")(element);
    }
    
    var rootElements = doc.getRootElements();
    
    for( var i=0; i<rootElements.length(); i++ )
      handleElement( rootElements[i] );
  }
  
  
  /**
   * All the script handling is encapsulated in this object, just to better
   * partition things.
   */
  var scripting = new (function() {
    
    /**
     * Utility to evaluate a piece of script code embedded within a page.  This
     * can be used for embedded script tags, onAction (etc.), and links of the
     * form "oscript:<SCRIPT_CODE>".  This method handles creating the page's
     * document and scope the first time a piece of script is evaluated for any
     * given page.
     * 
     * @param script       the string to evaluate.
     */
    public function evalScript(script)
    {
      init();
      
      try
      {
        return oscript.OscriptInterpreter.__eval( script + "\n", scope );
      }
      catch(e)
      {
        java.lang.System.err.println("error evaluating: '" + script + "': " + e);
        return null;
      }
    }
    
    /**
     */
    public function reset()
    {
      document = null;
      scope = null;
    }
    
    /**
     * Export an object to the script environment visible from the page.
     * 
     * @param name         the name of the object
     * @param obj          the object
     */
    public function exportObj( name, obj )
    {
      pkg.system.export( false, name, obj, scope );
    }
    
    var exportObjList = null;
    
    /**
     * Permanently export the object.  The object will still be visible if the
     * currently displayed page is changed.
     * 
     * @param name         the name of the object
     * @param obj          the object
     */
    public function permanentlyExportObj( name, obj )
    {
      if( exportObjList == null )
        exportObjList = new java.util.LinkedList();
      
      exportObjList.add( new (function() {
        public var name = name;
        public var obj  = obj;
      })() );
      
      if( scope != null )
        exportObj( name, obj );
    }
    /**
     * The script-environment.... probably should be split into it's own object...
     */
    var document = null;
    var scope = null;
    
    function init()
    {
      // we lazily create the scope, including the "document" object, for each
      // page.  This way it is only created if needed:
      if( scope == null )
      {
        // construct the document object:
        document = new (function() {
          
          // XXX probably way more to "document" than this, but this is all we need for now
          
          var accumulatedStr = "";
          
          public const var URL = (url != null) ? url.toExternalForm() : null;
          
          public function write(str)
          {
            accumulatedStr += str;
          }
          
          public function flush(element)
          {
            element.getDocument().insertBeforeStart( element, accumulatedStr );
            accumulatedStr = "";
          }
          
          public function open(urlStr)
          {
            loadURL( pkg.util.getSystemResource(urlStr) );
          }
          
        })();
        
        // construct scope object, ie. the context to evaluate script in:
        scope = new oscript.data.BasicScope( oscript.OscriptInterpreter.getGlobalScope() );
        // export document:
        exportObj( "document", document );
        
        if( exportObjList != null )
        {
          for( var itr=exportObjList.iterator(); itr.hasNext(); )
          {
            var o = itr.next();
            exportObj( o.name, o.obj );
          }
        }
      }
    }
    
    permanentlyExportObj( "pkg", pkg );
    
  })();
  
  
  /**
   * Export an object to the script environment visible from the page.  The object
   * will be visible even if a new URL is set.
   * 
   * @param name         the name of the object
   * @param obj          the object
   */
  public var exportObj = scripting.permanentlyExportObj;
  
}




/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
