/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2001.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



/* NOTE: this could still use a bit of clean-up... for example url could
 * refer to a java.net.URL or url string depending on the part of the code.
 */


/**
 * The help-browser plugin allows the user to browse online help.  Other 
 * code (such as plugins) can provide online help in the form of HTML pages,
 * which can be physically locate in a jar file, and register with this
 * plugin by calling the <code>registerHelp</code> method, which will add
 * the page to the main index.
 */
registry.register( new (function() extends ti.chimera.Plugin( main, "Help" ) {
  
  mixin pkg.system.makePrivatePackage("/jar/chimera/HelpPlugin/");
  
  var helpTree = new HelpTree();
  
  /**
   * The help table maps a path to a help-node.
   */
  function HelpNode( path, url, desc )
  {
    public function getPath() { return path; }
    public function getURL() { return url; }
    public function getDescription() { return desc; }
  }
  
  
  /****************************************************************************/
  /**
   * The "help" service implemented by the help plugin.
   */
  registerServiceFactory( function() {
    
    return new (function() extends ti.chimera.service.Help()  {
      
      /**
       * Register help.  The help will be made available under the main help
       * index, and the help pull-down menu.  You can also programmatically
       * open help by calling <code>displayHelp</code> and passing in the
       * same path.  The path is a "/" seperated path, which allows for
       * multiple sub-headings.
       * 
       * @param path         the "/" seperated path name of the help
       * @param url          the string URL to open to view this help
       * @param desc         description of the help
       * @see #displayHelp to programmatically display help
       */
      public function registerHelp( path, url, desc )
      {
        helpTree.add( path, new HelpNode( path, url, desc ) );
        
        var idx = path.lastIndexOf("/");
        var menuBarPath = "Help";
        var menuBarName;
        
        if( idx == -1 )
        {
          menuBarName = path;
        }
        else
        {
          menuBarPath += "/" + path.substring( 0, idx );
          menuBarName  = path.substring( idx + 1 );
        }
        
        addResource( 
          new MenuBarItemResource( menuBarPath, 
                                   new pkg.util.ActionAdapter( 
                                     menuBarName, null, null, 
                                     function(evt) { displayHelp(path); }
                                   ),
                                   true ) );
      }
      
      /**
       * Programmatically display the specified help.  If the path is
       * <code>null</code>, then display the main help index, which is
       * automatically generated from all the registered help pages.
       * 
       * @param path         the "/" seperated path name of the help
       * @see #registerHelp to register help
       */
      public function displayHelp(path)
      {
        var help = helpTree.get(path);
        
        if( help != null )
          services["swing worker"].run( function() {
            addResource( new ViewResource( new HelpView( help.getURL() ) ) );
          }, "Launching Help" );
      }
      
      /**
       * This isn't part of the well defined "help" service interface,
       * but is used by the index page to get access to the tree of
       * help.
       */
      public function getHelpTreeRoot()
      {
        return helpTree;
      }
      
    })();
  } );
})() );

services["help"].registerHelp( "Index", "index.html", "this page" );
services["help"].registerHelp( "Plugins/Help", "HelpPlugin/HelpPluginHelp.html", "Help for the Help Plugin" );



/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

