/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2001.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



/**
 * A Tree of all registered help nodes.
 */
function HelpTreeNode(name)
  extends pkg.SortedTreeNode(name)
{
  var nodeTable = new java.util.Hashtable(); // maps string to child node:
  var helpNode = null;
  
  /**
   * Add the element with the specified path to the tree.
   * 
   * @param path         the path to the helpNode
   * @param helpNode     the helpNode
   */
  public function add( path, helpNode )
  {
    if( path.length() == 0 )
    {
      if( (this.helpNode != null) && (this.helpNode.getURL() != helpNode.getURL()) )
        throw new ProgrammingErrorException("helpNode already set for \"" + name + "\"!");
      
      this.helpNode = helpNode;
    }
    else
    {
      var idx = path.indexOf("/");
      var name;
      
      if( idx == -1 )
      {
        name = path;
        path = "";
      }
      else
      {
        name = path.substring( 0, idx );
        path = path.substring( idx + 1 );
      }
      
      var node = nodeTable.get(name);
      
      if( node == null )
      {
        node = new HelpTreeNode(name);
        nodeTable.put( name, node );
        insert(node);
      }
      
      node.add( path, helpNode );
    }
  }
  
  /**
   * Get an element.
   * 
   * @param path         the path to the helpNode
   * @return the help-node
   */
  public function get(path)
  {
    if( path.length() == 0 )
    {
      return helpNode;
    }
    else
    {
      var idx = path.indexOf("/");
      var name;
      
      if( idx == -1 )
      {
        name = path;
        path = "";
      }
      else
      {
        name = path.substring( 0, idx );
        path = path.substring( idx + 1 );
      }
      
      var node = nodeTable.get(name);
      
      if( node == null )
        return null;
      else
        return node.get(path);
      
    }
  }
  
  /**
   * Get the help-node associated with this node in the tree.
   * 
   * @return help-node or <code>null</code>
   */
  public function getHelpNode()
  {
    return helpNode;
  }
}


/**
 */
public function HelpTree() extends HelpTreeNode("Help")
{
}




/**
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

