/******************************************************************************
 *     Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * The debugger plugin provides a way to debug objectscript code.
 */
registry.register( new (function() extends ti.chimera.Plugin( main, "Shell" ) {
  
  const var ev = oscript.OscriptInterpreter.__eval;
  
  /****************************************************************************/
  /**
   * The "shell" service.  Eventually this should be split out into another
   * plugin, and used by the debugger, but for now:
   */
  registerServiceFactory( function() {
    
    return new (function() extends ti.chimera.service.Shell() {
      
      /**
       * Read an expression.  Be nice and append a ";" if needed.  Also, handle multi-
       * line expressions when a line ends in "\"
       * 
       * @param in  the buffered reader
       * @return the read expression
       */
      public function readExpr(in)
      {
        var input = "";
        while(true)
        {
          var line = in.readLine();
          if( line == null )
            return null;
          
          input += line;
          
          // Strip out trailing whitespace:
          while( (input.length() > 0) && (input[ input.length() - 1 ] == " ") )
            input = input.substring( 0, input.length() - 1 );
          
          // Ignore empty lines and EOF:
          if( input == "" )
            return input;
          
          // If input ends with "\" then read next line:
          if( input[ input.length() - 1 ] == "\\" )
          {
            input = input.substring( 0, input.length() - 1 );
            continue;
          }
          
          // Be user-friendly, add a ";" to end of line:
          if( (input[ input.length() - 1 ] != ";") &&
              (input[ input.length() - 1 ] != "}") )
            input += ";";
          
          return input;
        }
      }
      
      /**
       * Run the shell read-eval-print loop.
       * 
       * @param scope        the scope evaluate in
       * @param in           the input stream reader (has readLine())
       * @param out          the output stream writer
       * @param err          the error stream writer
       */
      public function run( scope, in, out, err )
      {
        // we want in to have a readLine(), but other than that we don't 
        // really care what it is:
        if( (in instanceof java.io.Reader) && !(in instanceof java.io.BufferedReader) )
          in = new java.io.BufferedReader(in);
        
        if( (out instanceof java.io.Writer) && !(out instanceof java.io.PrintWriter) )
          out = new java.io.PrintWriter( out, true );
        
        out.println("entering debug shell, type \"exit\" to escape");
        
        while(true)
        {
          out.print( pkg.output.fgColor( java.awt.Color.green, "os>" ) + " " );
          out.flush();
          
          var input = readExpr(in);
          
          if( input == null )
            return;
          
          if( input == "" )
            continue;
          
          if( input == "exit;" )
            break;
          
          // Evaluate statement:
          try
          {
            var val = ev( input, scope );
            
            // store the last result as "_":
            pkg.system.export( false, "_", val, scope );
            
            input = null;
            
            if( val != null )
              out.println( "" + val );
          }
          catch(e)
          {
            main.debug( 1, "** error evaluating '" + input + "':\n" + e );
            out.println("** error evaluating '" + input + "':");
            out.println( services["debugger"].makeExceptionMessage(e) );
          }
        }
      }
      
    })();
    
  } );
  
})() );




/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
