/******************************************************************************
 *        Copyright Texas Instruments 2002-2004. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Get a string that includes environment details.  This is included in the
 * error message that gets sent back to the developers.
 */
function getEnvironmentDetails()
{
  var str = "";
  for( var itr=reportDataFxnList.iterator(); itr.hasNext(); )
    str += itr.next()() + "\n";
  return str;
}

/**
 * Get a list of all attachments to add to the talkback report.
 */
function getAttachmentList()
{
  var list = new java.util.LinkedList();
  for( var itr=reportAttachmentFxnList.iterator(); itr.hasNext(); )
  {
    var attachment = itr.next()();
    if( attachment != null )
      list.add(attachment);
  }
  return list;
}

/**
 * Utility for accessing registry parameters.  If the value isn't set (ie.
 * empty string), this will launch the registry browser so the user can
 * configure the parameters, and throws an exception
 */
function regGet( field, emailAddress )
{
  var val = registry.resolve("/Preferences/Talkback/" + field).getValue();
  if( val.length() > 0 && (emailAddress ? val.indexOf("@") != -1 : true) )
    return val;
  services["prompt"].showErrorMessage(
    "The \"" + field + "\" parameter is invalid or not set.  Please\n" +
    "correct this in the Talkback section of the Preferences window."
  );
  services["registry browser"].showBrowser("/Preferences/Talkback");
  throw new Exception("bad field");
}

/**
 * The talkback view.
 * 
 * @param msg        the error report message
 */
public function TalkbackView(msg)
  extends ti.chimera.View( main, "Unhandled Error", null )
{
  viewCount++;
  addCloseRunnable( function() {
    viewCount--;
    services["file browser"].removeDragAndDropSupport(list);
  } );
  
  var attachmentList = getAttachmentList();
  var panel = new swing.JPanel( new java.awt.BorderLayout() );
  
  ////////////////////////////////////////////////////////////////////////////////////
  // at the top of the window is a label describing the purpose of this dialog:
  panel.add( new swing.JLabel("<html>" + ((msg != null) ? "An unhandled error has occurred.  " : "") +
                              "To mail a bug report to " +
                              "the developer(s), click <i>Send</i>.  You may edit the " +
                              "report below to include a description of what you " +
                              "were doing when the error occurred, which can help " + 
                              "us reproduce the problem.  The more detail the better.  " +
                              "You can also add attachments (such as screenshots or " + 
                              "script files) by clicking the <i>Add</i> button or by " +
                              "dragging from the file browser window to the attachment " +
                              "list." +
                              "</html>"),
             java.awt.BorderLayout.NORTH );
  
  ////////////////////////////////////////////////////////////////////////////////////
  // in the center is the "error report" and attachment section, including user 
  // editable section, some environment details, and then the actual error msg 
  // itself:
  var centerBox = new swing.Box( swing.BoxLayout.Y_AXIS );
  var reportPanel = new swing.JPanel( new java.awt.BorderLayout() );
  
  var userCommentsTextArea = new swing.JTextArea( "<add your comments here>", 5, 60 );
  reportPanel.add( userCommentsTextArea, java.awt.BorderLayout.NORTH );
  
  var nonEditableReportText = getEnvironmentDetails();
  
  if( msg != null )
    attachmentList.add( new pkg.email.PlainTextAttachment( "backtrace.txt", msg ) );
  
  var reportLabel = new swing.JLabel("<html><pre>" + nonEditableReportText + "</pre></html>");
  reportPanel.add( reportLabel, java.awt.BorderLayout.SOUTH );
  
  {
    var scrollPane = new swing.JScrollPane(reportPanel);
    scrollPane.setBorder( new swing.border.TitledBorder("report: ") );
    centerBox.add(scrollPane);
  }
  
  var attachmentPanel = new swing.JPanel( new java.awt.BorderLayout() );
  attachmentPanel.setBorder( new swing.border.TitledBorder("attachments: ") );
  
  var listModel = new swing.DefaultListModel();
  
  for( var itr=attachmentList.iterator(); itr.hasNext(); )
    listModel.addElement( itr.next() );
  
  var list = new swing.JList(listModel);
  list.setSelectionMode( swing.ListSelectionModel.SINGLE_SELECTION );
  list.setCellRenderer( new (function() extends swing.DefaultListCellRenderer() {
    
    const var defaultIcon = pkg.util.getImageIcon("plainFile.gif");
    var _super_getListCellRendererComponent = getListCellRendererComponent;
    
    public function getListCellRendererComponent( list, attachment, index, isSelected, inFocus )
    {
      _super_getListCellRendererComponent( list, attachment.getFilename(), index, isSelected, inFocus );
      setIcon( services["file browser"].getFileTypeInfo( attachment.getFile() ).getFileIcon() );
      return this;
    }
    
  })() );
  
  /* Set-up popup menu:
   */
  var popupTrigger = new ti.chimera.PopupTrigger( new (function()
    extends ti.chimera.PopupTrigger.PopupListener() {
      
      public function showPopup(evt)
      {
        var attachment = listModel.getElementAt( list.locationToIndex( evt.getPoint() ) );
        
        if( attachment != null )
        {
          var file = attachment.getFile();
          var popup = services["file browser"].getPopupMenu(file);
          if( popup != null )
            popup.show( list, evt.getX(), evt.getY() );
        }
      }
      
    } )() );
  
  list.addMouseListener(popupTrigger);
  list.addMouseMotionListener(popupTrigger);
  
  /* handle double clicks:
   */
  list.addMouseListener( new (function() extends java.awt.event.MouseAdapter() {
    
    public function mouseClicked(evt)
    {
      if( evt.getClickCount() > 1 )
      {
        var attachment = listModel.getElementAt( list.locationToIndex( evt.getPoint() ) );
        if( attachment != null )
        {
          var file = attachment.getFile();
          services["file browser"].fireAction( file, null );
        }
      }
      }
    
  })() );
  
  attachmentPanel.add( new swing.JScrollPane(list), java.awt.BorderLayout.CENTER );
  
  /* register DnD drop source support:
   */
  services["file browser"].addDragSourceSupport( list, function(evt) {
    var attachment = listModel.getElementAt( list.locationToIndex( evt.getDragOrigin() ) );
    if( attachment != null )
      return attachment.getFile();
      return null;
  } );
  
  /* register DnD drop target support:
   */
  services["file browser"].addDragAndDropSupport( "/any/", list, function(file) {
    if( ! file.isDirectory() )
    {
      var attachment = new pkg.email.BinaryFileAttachment(file);
      attachmentList.add(attachment);
      listModel.addElement(attachment);
    }
  } );
    
  var buttonPanel = new swing.Box( swing.BoxLayout.Y_AXIS );
  
  var openButton = new swing.JButton("Open");
  openButton.addActionListener( function(evt) {
    var attachment = list.getSelectedValue();
    if( attachment != null )
    {
      var file = attachment.getFile();
      services["file browser"].fireAction( file, null );
    }
  } );
  buttonPanel.add(openButton);
  
  var addButton = new swing.JButton("Add");
  addButton.addActionListener( function(evt) {
    try
    {
      var chooser = new ti.chimera.FileChooser( windowManager.getDialog("Select File") );
      //chooser.setFileFilter( new pkg.util.ExtensionFileFilter( "os", "ObjectScript Source File" ) );
      
      if( chooser.showOpenDialog(null) == ti.chimera.FileChooser.APPROVE_OPTION )
      {
        var file = chooser.getSelectedFile();
        var attachment = new pkg.email.BinaryFileAttachment(file);
        
        attachmentList.add(attachment);
        listModel.addElement(attachment);
      }
    }
    finally
    {
      resetButtons();
    }
  } );
  buttonPanel.add(addButton);
  
  var removeButton = new swing.JButton("Remove");
  removeButton.addActionListener( function(evt) {
    var attachment = list.getSelectedValue();
    if( attachment != null )
    {
      listModel.removeElement(attachment);
      attachmentList.remove(attachment);
    }
    resetButtons();
  } );
  buttonPanel.add(removeButton);
  
  function resetButtons()
  {
    var attachment = list.getSelectedValue();
    if( attachment == null )
    {
      openButton.setEnabled(false);
      removeButton.setEnabled(false);
    }
    else
    {
      var file = attachment.getFile();
      openButton.setEnabled( services["file browser"].getFileTypeInfo(file).getDefaultAction(file) != null );
      removeButton.setEnabled(true);
    }
  }
  resetButtons();
  
  attachmentPanel.add( buttonPanel, java.awt.BorderLayout.EAST );
  
  list.addListSelectionListener( function(evt) {
    resetButtons();
  } );
  
  centerBox.add(attachmentPanel);
  panel.add( centerBox, java.awt.BorderLayout.CENTER );
  
  /* 
   * Subscribe to monospaced font size:
   */
  pkg.util.subscribeToValue(
    this,
    "/Preferences/System/Console Font", 
    new ti.chimera.registry.TypeNodeContract(java.awt.Font),
    new ti.chimera.registry.SwingNodeSubscriber( function( node, font ) {
      
      reportLabel.setFont(font);
      userCommentsTextArea.setFont(font);
      
    } ) );
  
  ////////////////////////////////////////////////////////////////////////////////////
  // in the bottom section is the panel of user editable fields, such as the
  // "To" and "From" email addresses, and the SMTP server:
  var buttonPanel = new swing.JPanel( new java.awt.GridLayout( 0, 2 ) );
  
  //    the "Send" button:
  var sendButton = new swing.JButton("Send");
  sendButton.addActionListener( new pkg.util.ActionAdapter( null, null, null, function(evt) { 
    
    try
    {
      sendReport();
      close();
    }
    catch(Exception e)
    {
      // bad registry parameter
    }
    
  } ) );
  buttonPanel.add(sendButton);
  
  //    the "Cancel" button:
  var cancelButton = new swing.JButton("Cancel");
  cancelButton.addActionListener( new pkg.util.ActionAdapter( null, null, null, function(evt) { close(); } ) );
  buttonPanel.add(cancelButton);
  
  panel.add( buttonPanel, java.awt.BorderLayout.SOUTH );
  
  panel.setPreferredSize( new java.awt.Dimension( 480, 480 ) );
  
  ////////////////////////////////////////////////////////////////////////////////////
  
  /**
   * Get the component that makes up this view.  The view must always 
   * implement this method.
   * 
   * @return any swing component will do
   */
  public function getComponent()
  {
    return panel;
  }
  
  function sendReport()
  {
    var email = new pkg.email.EmailMessage();
    var from = regGet( "Email Address", true );
    
    email.setSender(from);
    email.setSubject("Error report from: " + from);
    email.setBody( userCommentsTextArea.getText() + "\n\n" + nonEditableReportText );
    email.addRecipient( regGet( "Recipient", true ) );
    
    for( var itr=attachmentList.iterator(); itr.hasNext(); )
      email.addAttachment( itr.next() );
    
    try
    {
      (new pkg.email.Emailer( regGet( "SMTP Server", false ), 25 )).sendMessage(email);
    }
    catch(e)
    {
      e = unwrapException(e);
      services["prompt"].showErrorMessage("error mailing report:\n" + e.getMessage());
      return;
    }
  }
}
