/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2001-2002.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* This file contains standard bootstrap code that most other chimera
 * products will want to use.
 */

import "bootstrap-core.os";

// export windowManager etc. to global-scope... note that the way
// this is done is sort of "undocumented", so you shouldn't do this in script
// code you write unless you know what you are doing, and you are willing to
// fix your code if a future version of the interpreter changes how this works.
// In other words, There's nothing to see here folks, move along.
pkg.system.export( true,  "jui",           new JavaPackage("jui") );
pkg.system.export( true,  "swing",         javax.swing );
pkg.system.export( true,  "windowManager", null );

// make sure the system knows about some java packages that it might not
// otherwise know about:
pkg.system.declareJavaPackage("ti.swing");
pkg.system.declareJavaPackage("ti.swing.console");
pkg.system.declareJavaPackage("javax.swing.table");
pkg.system.declareJavaPackage("jui.layout");


if(mrj)
{
  pkg.system.declareJavaPackage("com.apple.eawt");
  pkg.system.declareJavaPackage("com.apple");
  pkg.system.export( true, "mrjApplication", new com.apple.eawt.Application() );
}

// remap normal output fxns to their pkg.output versions
//writeln = pkg.output.writeln;
//write   = pkg.output.write;
writeln = '{ pkg.output.writeln.callAsFunction(args); };
write   = '{ pkg.output.write.callAsFunction(args); };

pkg.system.registerFunctionTransformer( 
  new (function() extends pkg.system.FunctionTransformer(swing.event.ListSelectionListener) {
    public function transform(fxn)
    {
      return new (function() extends swing.event.ListSelectionListener() {
        public function valueChanged(evt) { fxn(evt); }
      })();
    }
  })() );

pkg.system.registerFunctionTransformer( 
  new (function() extends pkg.system.FunctionTransformer(swing.event.ChangeListener) {
    public function transform(fxn)
    {
      return new (function() extends swing.event.ChangeListener() {
        public function stateChanged(evt) { fxn(evt); }
      })();
    }
  })() );

/* XXX this is sortof a hack doing this here... at least the "pds" type should
 * be setup in bootstrap-esf.os, but it is needed before the file browser is
 * shown, which is here in this file...
 */
ti.chimera.fs.FileSystemTreeModel.setFileInfo( "os",  pkg.util.getImageIcon("osFile.gif"),  "ObjectScript Source File" );
ti.chimera.fs.FileSystemTreeModel.setFileInfo( "jar", pkg.util.getImageIcon("jarFile.gif"), "Java Archive File" );
ti.chimera.fs.FileSystemTreeModel.setFileInfo( "pds", pkg.util.getImageIcon("logFile.gif"), "Series Log File" );
ti.chimera.fs.FileSystemTreeModel.setFileInfo( "/any/", pkg.util.getImageIcon("plainFile.gif"), null );
ti.chimera.fs.FileSystemTreeModel.setFileInfo( "/jfs/", pkg.util.getImageIcon("jarFS.gif"), "Java Archive Mountpoint" );
ti.chimera.fs.FileSystemTreeModel.setFileInfo( "/lfs/", pkg.util.getImageIcon("defaultFS.gif"), "Local Mountpoint" );
ti.chimera.fs.FileSystemTreeModel.setFileInfo( "/vfs/", pkg.util.getImageIcon("defaultFS.gif"), "Virtual Mountpoint" );


/*=============================================================================
 * Setup the plug-ins:
 */

registry.link( 
  new ti.chimera.registry.PersistentNode( 
    mrj ? "window mode" : "desktop mode", 
    new ti.chimera.pref.ChoiceNodeContract( [ "window mode", "desktop mode" ],
                                            [ "Window",      "Desktop" ] ),
    "The current mode implementation.  In <code>desktop mode</code> all dialogs " +
    "are internal windows inside a desktop pane.  In <code>window mode</code> " +
    "all dialogs are separate windows." ),
  "/Preferences/Window Manager/Mode" );

{
  var vals = [ 
    swing.UIManager.getSystemLookAndFeelClassName(), 
    "javax.swing.plaf.metal.MetalLookAndFeel"
  ];
  
  var names = [ 
    "Native",
    "Metal"
  ];
  
  // NOTE: reason for "try" on same line is so I can grep "instanceof JavaClass",
  //   and filter out the places already converted to use try/catch
  try if( com.sun.java.swing.plaf.motif.MotifLookAndFeel instanceof JavaClass )
  {
    vals[ vals.length() ]   = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
    names[ names.length() ] = "Motif";
  }
  catch(NoSuchMemberException e) { /* ignore */ }
  
  try if( com.incors.plaf.kunststoff.KunststoffLookAndFeel instanceof JavaClass )
  {
    vals[ vals.length() ]   = "com.incors.plaf.kunststoff.KunststoffLookAndFeel";
    names[ names.length() ] = "Kunststoff";
    
    registerLibrary( "Kunststoff Look & Feel",
                     "kunststoff.jar", 
                     "http://www.incors.org",
                     "The Kunststoff (German for \"plastic\") Look & Feel.",
                     "LGPL",
                     "Copyright &copy; 2000-2002 Incors, Inc." );
  }
  catch(NoSuchMemberException e) { /* ignore */ }
  
  try if( com.jgoodies.plaf.windows.ExtWindowsLookAndFeel instanceof JavaClass )
  {
    vals[ vals.length() ]   = "com.jgoodies.plaf.windows.ExtWindowsLookAndFeel";
    names[ names.length() ] = "Extended Windows";
  }
  catch(NoSuchMemberException e) { /* ignore */ }
  
  try if( com.jgoodies.plaf.plastic.PlasticLookAndFeel instanceof JavaClass )
  {
    vals[ vals.length() ]   = "com.jgoodies.plaf.plastic.PlasticLookAndFeel";
    names[ names.length() ] = "Plastic";
  }
  catch(NoSuchMemberException e) { /* ignore */ }
  
  try if( com.jgoodies.plaf.plastic.Plastic3DLookAndFeel instanceof JavaClass )
  {
    vals[ vals.length() ]   = "com.jgoodies.plaf.plastic.Plastic3DLookAndFeel";
    names[ names.length() ] = "Plastic3D";
  }
  catch(NoSuchMemberException e) { /* ignore */ }
  
  try if( com.jgoodies.plaf.plastic.PlasticXPLookAndFeel instanceof JavaClass )
  {
    vals[ vals.length() ]   = "com.jgoodies.plaf.plastic.PlasticXPLookAndFeel";
    names[ names.length() ] = "PlasticXP";
  }
  catch(NoSuchMemberException e) { /* ignore */ }
  
  registry.link( 
    new ti.chimera.registry.PersistentNode( 
      swing.UIManager.getSystemLookAndFeelClassName(), 
      new ti.chimera.pref.ChoiceNodeContract( vals, names ),
      "The current Look & Feel." ),
    "/Preferences/Window Manager/Look And Feel" );
  
  pkg.plugin.register("ti.chimera.plugin.WindowModePlugin");
  pkg.plugin.register("ti.chimera.plugin.DesktopModePlugin");
  pkg.plugin.register("ti.chimera.plugin.WindowManagerPlugin");
  windowManager = services["window manager"];
  
  // force the order of some of the entries in the menubar:
  registry.mkdir("/MenuBar/File/New");
  pkg.util.addMenuBarItem( "File", "Open", function(evt) {} );
  pkg.util.addMenuBarItem( "File", null, null );   // add a seperator
  pkg.util.addMenuBarItem( "File/Recent Items", "<placeholder>", function(evt) {} );
  pkg.util.addMenuBarItem( "File", null, null );   // add a seperator
  registry.mkdir("/MenuBar/Edit/Mode");
  registry.mkdir("/MenuBar/Edit/Look & Feel");
  pkg.util.addMenuBarItem( "Edit", null, null );   // add a seperator
  registry.mkdir("/MenuBar/Plugins");
  
  for( var i=0; i<names.length(); i++ )
  {
    pkg.util.addMenuBarItem( "Edit/Look & Feel", names[i], function(val) {
      return function(evt) { windowManager.setLookAndFeel(val); };
    }(vals[i]) );
  }
}

pkg.plugin.register("ti.chimera.plugin.SwingWorkerPlugin");

// setup close-view menubar action:
{
  const var path = "/Docks/selected dock/selected view";
  
  var close = new pkg.util.ActionAdapter( "Close", null, null, function(evt) {
    registry.resolve(path).getValue().close();
  } );
  
  close.setEnabled(false);
  close.putValue( swing.Action.ACCELERATOR_KEY, pkg.util.getAcceleratorKeyStroke("W") );
  
  registry.subscribeToValue( path, null, function( node, val ) {
    close.setEnabled(true);
  } );
  
  registry.subscribeToDeletion( path, function(node) {
    close.setEnabled(false);
  } );
  
  services["window manager"].addMenuBarItem( "File", close );
}

pkg.util.addMenuBarItem( "File", null, null );   // add a seperator

// setup bogus Save/Save As menubar actions, disabled so only the real menubar 
// actions will determine if menubar entry is enabled or not
{
  var save   = new pkg.util.ActionAdapter( "Save", null, null, function(evt) {} );
  var saveAs = new pkg.util.ActionAdapter( "Save As", null, null, function(evt) {} );
  save.setEnabled(false);
  saveAs.setEnabled(false);
  windowManager.addMenuBarItem("File",save);
  windowManager.addMenuBarItem("File",saveAs);
}

pkg.util.addMenuBarItem( "File", null, null );   // add a seperator



pkg.plugin.register("TalkbackPlugin/TalkbackPlugin.os");
pkg.util.addMenuBarItem( "Help", "Report Bug", '{ services["talkback"].reportError(); } );

services["talkback"].addReportDataFxn( '{
  
  var str = ("Script Path: \n" + 
             "------ ----- \n");
  
  for( var path : pkg.system.getScriptPath() )
    str += "   " + path + "\n";
  
  return str;
  
} );

services["talkback"].addReportAttachmentFxn( '{
  return new pkg.email.PlainTextAttachment( "history.txt", main.getHistory() );
} );

services["talkback"].addReportAttachmentFxn( '{
  
  var visitedSet = new java.util.HashSet();
  var str = "";
  
  (function( prefix, node ){
    
    visitedSet.add(node);
    var dt = node.getValue();
    for( var name : dt.getChildNames() )
    {
      str += prefix + "/" + name + "\n";
      
      var node = dt.get(name);
      if( (node.getValue() instanceof ti.chimera.registry.DirectoryTable) &&
          !visitedSet.contains(node) )
        callee( prefix + "/" + name, node );
    }
    
  })( "", registry.resolve("/") );
  
  return new pkg.email.PlainTextAttachment( "registry.txt", str );
  
} );

// many other things depend on help, so load it first:
pkg.plugin.register("HelpPlugin/HelpPlugin.os");

services["help"].registerHelp( "3rd Party Library(s)",  "libraries.html",      "A description of the licenses of 3rd party libraries distributed with this application" );
services["help"].registerHelp( "API/Chimera", "chimera/overview-summary.html", "API Documentation for <i>chimeral.jar</i> library" );

// file browser is used by ConsolePlugin, so load it first:
pkg.plugin.register("FileBrowserPlugin/FileBrowserPlugin.os");
pkg.plugin.deferredRegister( "PromptPlugin.os", ["prompt"] );
pkg.plugin.deferredRegister( "RegistryBrowserPlugin.os", ["registry browser"] );

pkg.util.addon.register( 
  false, 
  "clock", 
  "The clock plugin displays a digital clock in the toolbar.<br><br>" +
  "This preference controls whether the clock is enabled.",
  '{ pkg.plugin.register("ClockPlugin.os"); }
);

pkg.util.addon.register( 
  false, 
  "fortune", 
  "The fortune plugin can print out a random epigram (quote, fortune, " +
  "etc.) when a new console is opened, etc.  This option controls whether " +
  "fortune is enabled",
  '{ pkg.plugin.register("FortunePlugin.os"); }
);

pkg.util.addon.register( 
  true, 
  "pager", 
  "The pager plugin provides you with multiple virtual workspaces, or " +
  "<i>pages</i>.  It also implements a Window menu, which will let you " +
  "bring any open dialog to the front.  When a new dialog is opened, it " +
  "appears in the current workspace.  You can switch between workspaces " +
  "using the pager toolbar, which should have a toggle-button for each " +
  "workspace.  Dialogs can be made sticky (so they appear in each workspace) " +
  "with the \"Stick\" and \"Unstick\" options under the Window menu.<br><br>" +
  "This preference controls whether the pager plugin is enabled.",
  '{ pkg.plugin.register("PagerPlugin.os"); }
);


if( pkg.system.resolve("bootstrap-ode.os").canRead() )
  import "bootstrap-ode.os";

/*=============================================================================
 * Setup the Menu-Bar:
 */

pkg.util.addMenuBarItem( "File/Browse", "File System", function(evt) { services["file browser"].showBrowser("/"); } );
pkg.util.addMenuBarItem( "File", "Open", function(evt) { services["file browser"].open(null); } );

if(!mrj)
{
  pkg.util.addMenuBarItem( "File", null, null );   // add a seperator
  pkg.util.addMenuBarItem( "File", "Exit", function(evt) { exit(0); } );
}

// XXX "Edit/Mode" should be generated from the appropriate ChoiceNodeContract
pkg.util.addMenuBarItem( "Edit/Mode", "Window", function(evt) { windowManager.setMode( services["window mode"] ); } );
pkg.util.addMenuBarItem( "Edit/Mode", "Desktop", function(evt) { windowManager.setMode( services["desktop mode"] ); } );

if(!mrj)
  pkg.util.addMenuBarItem( "Edit", "Preferences", function(evt) { services["registry browser"].showBrowser("/Preferences"); } );

if(mrj)
{
  mrjApplication.addApplicationListener(
    new (function() extends com.apple.eawt.ApplicationAdapter() {
      public function handlePreferences(evt) { 
        evt.setHandled(true);
        services["registry browser"].showBrowser("/Preferences");
      }
      public function handleQuit(evt) {
        evt.setHandled(true);
        exit(0);
      }
//      public function handleOpenApplication(evt) { evt.setHandled(true); }
    })()
  );
  mrjApplication.setEnabledPreferencesMenu(true);
}


/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

