/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2003.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// make sure the system knows about some java packages that it might not
// otherwise know about:
pkg.system.declareJavaPackage("oscript.swing");
pkg.system.declareJavaPackage("oscript.swing.text");


pkg.plugin.deferredRegister( "ConsolePlugin.os", ["console"] );


pkg.plugin.register("EditorPlugin/EditorPlugin.os");

pkg.plugin.deferredRegister( "ObjectBrowserPlugin.os", ["Object Browser"] );
pkg.plugin.deferredRegister( "ShellPlugin.os", ["shell"] );
pkg.plugin.deferredRegister( "DebuggerPlugin.os", ["debugger"] );
pkg.plugin.deferredRegister( "DocBrowser.os", ["doc browser"] );

services["file browser"].addAction( "os", new pkg.util.ActionAdapter( "show api", null, null, function(evt) {
  
  services["doc browser"].showApiDocs( evt.getSource() );
  
} ) );

pkg.plugin.register("ScriptPathPlugin.os");
services["editor"].registerWizard( "Plugin", "pkg.wizards.newPluginWizard" );
try if( (jui.JUIModel instanceof JavaClass) && (System.getProperty("java.version").indexOf("1.4") == 0) )
  services["editor"].registerWizard( "JUI Layout", "pkg.wizards.juiWizard" );
catch(NoSuchMemberException e)
{ /* ignore */ }

//pkg.plugin.register("jui-wizard.os");
pkg.plugin.register("new-script-app.os");

pkg.util.addon.register(
  false,
  "output console",
  "The output console plugin redirects output to <code>stdout</code> " +
  "or <code>stderr</code> to a console window.  This is helpful in " +
  "cases where you would otherwise not see the output.",
  function() {
    pkg.plugin.register("OutputConsolePlugin.os");
  }
);

pkg.util.addon.register( 
  false, 
  "free memory", 
  "The free-memory plugin creates a toolbar that displays the current " +
  "heap size and the amount of heap used, in the form <i>heap used</i> / " +
  "<i>total heap size</i>.<br><br>This option determines whether the " +
  "free-memory plugin is activated or not.",
  function() {
    pkg.plugin.register("FreeMemPlugin.os");
  } 
);


// restore the opened consoles early in the startup sequence... this
// would be done inside ConsolePlugin.os, but we want to avoid loading
// the console if no consoles are displayed:
registry.link( 
  new ti.chimera.registry.PersistentNode( 
    1, 
    ti.chimera.registry.NodeContract.NUMBER_CONTRACT,
    "the number of open console windows"
  ),
  "/Console/count"
);
{
  services["window manager"].addMenuBarItem( 
    "File/New",
    new pkg.util.ActionAdapter(
      "Console", 
      null, 
      pkg.util.getImageIcon("Host16.gif"), 
      '{ services["console"].newConsole(); }
    )
  );
  
  var node = registry.resolve("/Console/count");
  var cnt  = node.getValue();
  node.setValue(0);
  for( var i=0; i<cnt; i++ )
    services["console"].newScriptConsole();
}

pkg.util.addMenuBarItem( "File/Browse", "Registry ", '{ services["registry browser"].showBrowser("/"); } );

pkg.util.addMenuBarItem( "Edit", "Script Path", '{ services["script path"].show(); } );
if(!mrj)
  pkg.util.addMenuBarItem( "Edit", null, null );   // add a seperator


registry.link( 
  new ti.chimera.registry.PersistentNode(
    new java.awt.Font( "Monospaced", java.awt.Font.PLAIN, 12 ), 
    new ti.chimera.pref.FontNodeContract( ti.chimera.pref.FontNodeContract.MONOSPACE ),
    "the monospace font for all console windows"
  ),
  "/Preferences/System/Console Font"
);


pkg.util.addMenuBarItem( "Help/Browse Service", "All Services", '{
  services["Object Browser"].browseServices();
} );

pkg.registry.applyFunction( "/Services", "/MenuBar/Help/Browse Service", registry, false, function( srcNode, dstNode, val ) {
  
  var serviceName = registry.basename( srcNode.getPrimaryPath() );
  var l = new java.util.LinkedList();
  
  l.add( new pkg.util.ActionAdapter( serviceName, null, null, '{
    services["Object Browser"].browseService(serviceName);
  } ) );
  
  return l;
  
} );

if(mrj)
{
  mrjApplication.addApplicationListener(
    new (function() extends com.apple.eawt.ApplicationAdapter() {
      public function handleOpenFile(evt) {
System.err.println("open file: " + evt.filename);
        evt.setHandled(true);
        pkg.cl.edit( evt.filename );
      }
      public function handleOpenApplication(evt) {
System.err.println("open app: " + evt.filename);
      }
    })()
  );
}


/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

