/******************************************************************************
 *     Copyright Texas Instruments 2004. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * The panel
 */
public function DebuggerPanel(view)
  extends swing.JPanel()
{
  /***************************************************************************/
  
  const var HorizontalNode = jui.layout.HorizontalNode;
  const var ConsoleTextArea = ti.swing.ConsoleTextArea;
  const var ComponentNode = jui.layout.ComponentNode;
  const var ComponentNodeInfo = jui.layout.ComponentNodeInfo;
  const var Insets = java.awt.Insets;
  const var JSplitPane = javax.swing.JSplitPane;
  const var JButton = javax.swing.JButton;
  const var Dimension = java.awt.Dimension;
  const var BoxTreeLayout = jui.layout.BoxTreeLayout;
  
  
  public const var console;
  this.console = pkg.util.makeConsole( view, 200, 80, false );
  console.setName("console");
  console.setToolTipText("the interactive debugger session");
  console.setEnabled(true);
  console.setRequestFocusEnabled(true);
  console.setMinimumSize(null);
  console.setMaximumSize(null);
  console.setPreferredSize(null);
  add(console);
  
  public const var splitPane;
  this.splitPane = new JSplitPane();
  splitPane.setName("splitPane");
  splitPane.setToolTipText(null);
  splitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
  splitPane.setOneTouchExpandable(true);
  splitPane.setContinuousLayout(false);
  splitPane.setEnabled(true);
  splitPane.setDividerLocation(180);
  splitPane.setRequestFocusEnabled(true);
  splitPane.setMinimumSize(null);
  splitPane.setMaximumSize(null);
  splitPane.setPreferredSize(new Dimension(187, 258));
  add(splitPane);
  
  setLayout( new BoxTreeLayout( new HorizontalNode(new ComponentNode(console, new Insets(1, 1, 1, 7), new ComponentNodeInfo(true, true)), new ComponentNode(splitPane, new Insets(1, 0, 1, 1), new ComponentNodeInfo(true, true))) ) );
  revalidate();
  
  /***************************************************************************/
  
  private var scopeChainViewPanel = new pkg.debugger.ScopeChainViewPanel();
  splitPane.setTopComponent( new swing.JScrollPane(scopeChainViewPanel) );
  
  private var backtraceTable = new javax.swing.JTable();
  backtraceTable.setToolTipText("The call-stack.  Double click to open editor to current file:lineno of selected stack-frame.  Debug shell sessions run in the scope of selected stack-frame.");
  backtraceTable.setColumnSelectionAllowed(false);
  backtraceTable.setRowSelectionAllowed(true);
  backtraceTable.setSelectionMode( javax.swing.ListSelectionModel.SINGLE_SELECTION );
  splitPane.setBottomComponent( new javax.swing.JScrollPane(backtraceTable) );
  
  backtraceTable.addMouseListener( new function() extends java.awt.event.MouseAdapter() {
    
    public function mouseClicked(evt)
    {
      if( evt.getClickCount() > 1 )
      {
        var idx = backtraceTable.getSelectedRow();
        var sf  = backtraceTable.getModel().getStackFrame(idx);
        
        if( sf != null )
          services["editor"].openFileToLineNum( sf.getFile(), sf.getLineNumber() );
      }
    }
    
  }() );
  
  /**
   * Track the last selected stack-frame, so when we re-enter a breakpoint
   * we can attempt to re-select the same stack-frame that was previously
   * selected.
   */
  private var lastSf = null;
  
  /**
   * Setup listener to update scope-chain-view-panel to reflect the selected
   * row in the backtrace table
   */
  backtraceTable.getSelectionModel().addListSelectionListener( function(evt) {
    
    setSelectedStackFrame( backtraceTable.getSelectedRow() );
    
  } );
  
  /**
   * Table-model for contents of backtrace-table
   */
  private function StackFrameTableModel(backtrace)
    extends swing.table.AbstractTableModel()
  {
    static var COLUMN_NAMES = ["Function","File","Line"];
    
    public function getStackFrame(idx)
    {
      if( (idx < 0) || (idx >= backtrace.length()) )
        return null;
      return backtrace[idx][3];
    }
    
    public function getColumnName(col) { return COLUMN_NAMES[col]; }
    public function getRowCount() { return backtrace.length(); }
    public function getColumnCount() { return COLUMN_NAMES.length(); }
    public function isCellEditable( row, col ) { return false; }
    
    public function getValueAt( row, col )
    {
      return backtrace[row][col];
    }
  }
  
  /**
   * Set the selected stack frame row in the backtraceTable
   */
  private function setSelectedStackFrame(idx)
  {
    var sf = backtraceTable.getModel().getStackFrame(idx);
    scopeChainViewPanel.update(sf);
    backtraceTable.getSelectionModel().setSelectionInterval(idx,idx);
    if( sf != null )
      lastSf = sf;
  }
  
  /**
   * Return the stack-frame entry that is currently selected.
   */
  public function getSelectedStackFrame()
  {
    return backtraceTable.getModel().
      getStackFrame( backtraceTable.getSelectedRow() );
  }
  
  /**
   * Called by debugger upon entry of breakpoint
   */
  public function enterBreakpoint( currentStackFrame, scope, file, line )
  {
    if( currentStackFrame == null )
      currentStackFrame = pkg.system.currentStackFrame();
    
    var backtrace = [];
    var foundTop = false;
    
    for( var sf : currentStackFrame )
    {
      if(!foundTop)
        if( (line == sf.getLineNumber()) &&
            (file == sf.getFile()) )
          foundTop = true;
      
      if(foundTop)
      {
        var name = sf.getName();
        var file = sf.getFile();
        backtrace[ backtrace.length() ] = [ 
          (name == null) ? "unknown" : name,
          (file == null) ? "unknown" : file.getName(), 
          sf.getLineNumber(), 
          sf 
        ];
      }
    }
    
    backtraceTable.setModel( new StackFrameTableModel(backtrace) );
    backtraceTable.getColumnModel().getColumn(0).setPreferredWidth(75);
    backtraceTable.getColumnModel().getColumn(1).setPreferredWidth(75);
    backtraceTable.getColumnModel().getColumn(2).setPreferredWidth(10);
    
    console.setEnabled(true);
    splitPane.setEnabled(true);
    
    if( lastSf != null )
      for( var i=0; i<backtrace.length(); i++ )
        if( lastSf == backtrace[i][3] )
        { setSelectedStackFrame(i); return; }
    
    setSelectedStackFrame(0);
  }
  
  
  /**
   * Called as handler leaves breakpoint, to do any needed cleanup.
   */
  public function leaveBreakpoint()
  {
    backtraceTable.setModel( new StackFrameTableModel([]) );
    console.setEnabled(false);
    splitPane.setEnabled(false);
    scopeChainViewPanel.update(null);
  }
  
  leaveBreakpoint();
}

