/******************************************************************************
 *     Copyright Texas Instruments 2004. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

mixin javax.swing;

var iconTable = new java.util.Hashtable();

iconTable.put( "function", pkg.util.getImageIcon("methodPublic.gif") );
iconTable.put( "object", pkg.util.getImageIcon("constructorPublic.gif") );
iconTable.put( "file", pkg.util.getImageIcon("osFile.gif") );
iconTable.put( "global", pkg.util.getImageIcon("globe16.gif") );

/**
 * The panel
 */
public function ScopeChainViewPanel()
  extends JPanel( new java.awt.BorderLayout() )
{
  /**
   */
  private var db = new pkg.util.DrawerBar();
  db.setToolTipText("Scope viewer shows variables declared within scope chain of currently selected stack-frame.");
  add( new JScrollPane(db), java.awt.BorderLayout.CENTER );
  
  /**
   * The simplify control, if selected, causes scopes to be coalesced
   * into only function, object, file, and global scopes, with intemediate
   * basic scopes being folded into their parent scope (and so on)
   */
  private var simplifyCheckBox = new JCheckBox("Simplify Scope",true);
  simplifyCheckBox.setToolTipText("When checked, the number of scopes displayed is reduced by only showing function, object, file, and global scopes.  Other scopes folded into parent.");
  add( simplifyCheckBox, java.awt.BorderLayout.SOUTH );
  simplifyCheckBox.addActionListener( function(evt) {
    update(sf);
  } );
  private var sf;
  
  /**
   * Set the scope to be displayed currently
   * 
   * @param sf   the current stack frame being dislayed
   */
  public function update(sf)
  {
    this.sf = sf;
    
    db.removeAll();
    
    var scopeSets = [];
    
    if( sf != null )
    {
      setEnabled(true);
      
      var simplify = simplifyCheckBox.isSelected();
      var scopeSet = scopeSets[0] = [];
      var scope = sf.getScope();
      
      while(true)
      {
        scopeSet[ scopeSet.length() ] = scope;
        
        var desc = pkg.system.debugger.getDescription(scope);
        
        scope = pkg.system.debugger.getPreviousScope(scope);
        if( (scope == null) || (scope == undefined) )
          break;
        
        if( (!simplify) || (desc.type != null) )
          scopeSet = scopeSets[ scopeSets.length() ] = [];
      }
    }
    else
    {
      setEnabled(false);
    }
    
    for( var scopeSet : scopeSets )
    {
      // use last scope in set to determine description:
      var scope = scopeSet[ scopeSet.length() - 1 ];
      
      var desc = pkg.system.debugger.getDescription(scope);
      
      var members = [];
      
      for( var i=scopeSet.length()-1; i>=0; i-- )
      {
        scope = scopeSet[i];
        
        function getMemberValue(memberName)
        {
          try
          {
            var val = Debugger.getMemberAccessor( scope, memberName ).getValue();
            if( (val instanceof ExactNumber) ||
                (val instanceof InexactNumber) ||
                (val instanceof String) ||
                (val instanceof Boolean) )
              return val.castToString();
          }
          catch(UnsupportedOperationException e)
          {
            // avoid uninitialized variables causing problems
          }
          return "..."; // XXX for now
        }
        
        for( var memberName : pkg.system.debugger.memberSet(scope) )
          members[ members.length() ] = [ memberName, getMemberValue(memberName) ];
      }
      
      db.addDrawer( desc.text, (desc.type == null) ? null : iconTable[ desc.type ], function() {
        // XXX maybe use flyweight pattern to share a single
        // table amongst all drawers?
        var memberTable = new JTable();
        memberTable.setModel( new ScopeMembersTableModel(members) );
        memberTable.getColumnModel().getColumn(0).setPreferredWidth(130);
        memberTable.getColumnModel().getColumn(1).setPreferredWidth(30);
        return memberTable;
      } );
    }
  }
  
  /**
   * Table-model for contents of backtrace-table
   */
  private function ScopeMembersTableModel(members)
    extends swing.table.AbstractTableModel()
  {
    static var COLUMN_NAMES = ["Name","Value"];
    
    public function getColumnName(col) { return COLUMN_NAMES[col]; }
    public function getRowCount() { return members.length(); }
    public function getColumnCount() { return COLUMN_NAMES.length(); }
    public function isCellEditable( row, col ) { return false; }
    
    public function getValueAt( row, col )
    {
      return members[row][col];
    }
  }
  
  update(null);
}

// XXX move this
pkg.system.declareJavaPackage("javax.swing.table");
