/******************************************************************************
 *        Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



/**
 * A utility to facilitate writing into a frame's document.
 * 
 * @param frame  text component to write into
 * @param off    offset to start writing at
 * @param len    num of chars, starting at <code>off</code>, to overwrite
 */
public function DocumentWriter( frame, off, len )
{
  private   var buf = frame.getBuffer();
  private   var origIndent = off - swing.text.Utilities.getRowStart( buf, off );
  private   var doc = buf.getDocument();
  private   var str = null;
  protected var ind;
  private   var indstr;
  
  /**
   * Increase the indent level by the specified amount, which can
   * be a negative number to decrement the indent level.
   * 
   * @param amount   the indent level to increase by, ie. either
   *    <code>+1</code> or <code>-1</code>
   */
  public function indent(amount)
  {
    ind += amount;
    indstr="";
    for( var i=0; i<ind; i++ )
      indstr += "  ";
  }
  
  /**
   * Dump a line of output at the current indent level.
   * 
   * @param line  the line to dump, not including EOL
   */
  public function dumpln(line)
  {
    str += indstr + line + "\n";
  }
  
  /**
   * Reset the writer, to begin writing again.
   */
  public function reset()
  {
    // XXX need to update off from Position, in case user modifies text
    if( str == null )
      doc.remove( off, len );
    else
      doc.remove( off, str.length() );
    str = "";
    ind = origIndent;
    indent(0);  // reset indstr
  }
  reset();
  
  /**
   * Flush any changes to the document.
   */
  public function flush()
  {
    // XXX not right, if flush is called multiple times
    doc.insertString( off, str, null );
  }
  
  /**
   * Get the frame that is being written to.
   * 
   * @return the pkg.editor.EditorFrame
   */
  public function getFrame()
  {
    return frame;
  }
}


