/*=============================================================================
 *     Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * A history buffer.
 */
public function History()
{
  var idx = 0;
  var history = [];
  
  /**
   * Append the specified string to the history.  It will become the most
   * recent entry in the history buffer.  This also resets the history index
   * so that the next call to {@link #getPrev} will return this entry in
   * the history buffer
   * 
   * @param str    the string to append to the history buffer
   */
  public function append(str)
  {
    if( (history.length() == 0) || (str != history[ history.length() - 1 ]) )
      history[ history.length() ] = str;
    idx = history.length();
  }
  
  /**
   * Get the next entry in the history buffer
   * 
   * @return the next entry, or <code>null</code> if none
   */
  public function getNext()
  {
    if( idx < (history.length() - 1) )
      return history[++idx];
    else if( idx < history.length() )
      return "";
    else
      return null;
  }
  
  /**
   * Get the previous entry in the history buffer
   * 
   * @return the previous entry, or <code>null</code> if none
   */
  public function getPrev()
  {
    if( idx > 0 )
      return history[--idx];
    return null;
  }
}


