/*=============================================================================
 *     Copyright Texas Instruments 2002. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//pkg.output.writeln("loading pkg.editor.globalMode");

mixin pkg.editor;

public const var globalMode = new (function() extends Mode( null, "global" ) {
  
  /* handling for the "globalMode" is special, in order to have default
   * behaviour for any key without having to register every single key:
   */
  
  inputMap.setParent( new (function() extends swing.InputMap() {
    
    public function get(key)
    {
      if( key.getKeyCode() == 0 )
        return swing.text.DefaultEditorKit.defaultKeyTypedAction;
      return null;
    }
    
  })() );
  
  // setup default input-map:
  swing.LookAndFeel.loadKeyBindings( inputMap, [
                                     
                           "ctrl C", swing.text.DefaultEditorKit.copyAction,
                           "ctrl V", swing.text.DefaultEditorKit.pasteAction,
                           "ctrl X", swing.text.DefaultEditorKit.cutAction,
                             "COPY", swing.text.DefaultEditorKit.copyAction,
                            "PASTE", swing.text.DefaultEditorKit.pasteAction,
                              "CUT", swing.text.DefaultEditorKit.cutAction,
                       "shift LEFT", swing.text.DefaultEditorKit.selectionBackwardAction,
                    "shift KP_LEFT", swing.text.DefaultEditorKit.selectionBackwardAction,
                      "shift RIGHT", swing.text.DefaultEditorKit.selectionForwardAction,
                   "shift KP_RIGHT", swing.text.DefaultEditorKit.selectionForwardAction,
                        "ctrl LEFT", swing.text.DefaultEditorKit.previousWordAction,
                     "ctrl KP_LEFT", swing.text.DefaultEditorKit.previousWordAction,
                       "ctrl RIGHT", swing.text.DefaultEditorKit.nextWordAction,
                    "ctrl KP_RIGHT", swing.text.DefaultEditorKit.nextWordAction,
                  "ctrl shift LEFT", swing.text.DefaultEditorKit.selectionPreviousWordAction,
               "ctrl shift KP_LEFT", swing.text.DefaultEditorKit.selectionPreviousWordAction,
                 "ctrl shift RIGHT", swing.text.DefaultEditorKit.selectionNextWordAction,
              "ctrl shift KP_RIGHT", swing.text.DefaultEditorKit.selectionNextWordAction,
                           "ctrl A", swing.text.DefaultEditorKit.selectAllAction,
                             "HOME", swing.text.DefaultEditorKit.beginLineAction,
                              "END", swing.text.DefaultEditorKit.endLineAction,
                       "shift HOME", swing.text.DefaultEditorKit.selectionBeginLineAction,
                        "shift END", swing.text.DefaultEditorKit.selectionEndLineAction,

                               "UP", swing.text.DefaultEditorKit.upAction,
                            "KP_UP", swing.text.DefaultEditorKit.upAction,
                             "DOWN", swing.text.DefaultEditorKit.downAction,
                          "KP_DOWN", swing.text.DefaultEditorKit.downAction,
                          "PAGE_UP", swing.text.DefaultEditorKit.pageUpAction,
                        "PAGE_DOWN", swing.text.DefaultEditorKit.pageDownAction,
                    "shift PAGE_UP", "selection-page-up",
                  "shift PAGE_DOWN", "selection-page-down",
               "ctrl shift PAGE_UP", "selection-page-left",
             "ctrl shift PAGE_DOWN", "selection-page-right",
                         "shift UP", swing.text.DefaultEditorKit.selectionUpAction,
                      "shift KP_UP", swing.text.DefaultEditorKit.selectionUpAction,
                       "shift DOWN", swing.text.DefaultEditorKit.selectionDownAction,
                    "shift KP_DOWN", swing.text.DefaultEditorKit.selectionDownAction,
                            "ENTER", swing.text.DefaultEditorKit.insertBreakAction,
                       "BACK_SPACE", swing.text.DefaultEditorKit.deletePrevCharAction,
                       "typed \010", swing.text.DefaultEditorKit.deletePrevCharAction,
                           "DELETE", swing.text.DefaultEditorKit.deleteNextCharAction,
                            "RIGHT", swing.text.DefaultEditorKit.forwardAction,
                             "LEFT", swing.text.DefaultEditorKit.backwardAction, 
                         "KP_RIGHT", swing.text.DefaultEditorKit.forwardAction,
                          "KP_LEFT", swing.text.DefaultEditorKit.backwardAction,
                              "TAB", swing.text.DefaultEditorKit.insertTabAction,
                        "ctrl HOME", swing.text.DefaultEditorKit.beginAction,
                         "ctrl END", swing.text.DefaultEditorKit.endAction,
                  "ctrl shift HOME", swing.text.DefaultEditorKit.selectionBeginAction,
                   "ctrl shift END", swing.text.DefaultEditorKit.selectionEndAction
                                        
    ] );
  
  // setup default actions:
  var actions = (new swing.text.DefaultEditorKit()).getActions();
  for( var i=0; i<actions.length(); i++ )
    actionMap.put( actions[i].getValue(swing.Action.NAME), actions[i] );
  
})();


/* setup any global-mode specific keybinding here */

/**
 * To implement search, create a search-mode, which temporarily replaces the
 * editor's mode while the search is going on.  The search is an instance of
 * escape-mode, which sets up keybindings for key-pressed events, but nothing
 * else.
 * <p>
 * Note: we need to refactor some of this back to EscapeMode so it can be
 * shared with the other escape modes.
 */
function SearchMode()
  extends EscapeMode()
{
  var frame = EditorFrame.getCurrentFrame();
  var caret = frame.getBuffer().getCaret();
  
  var buf  = frame.getBuffer();
  var mark = buf.getCaretPosition();
  
  public static var lastStr = "";
  
  var replaceStr = null;
  var str  = "";
  var status = "search";
  
  /**
   * Called by base class for valid keyboard input.
   * 
   * @param c    the character typed
   */
  function handleChar(c)
  {
    if( (c != ",") && (c != "\n") )
    {
      lastStr = (str += c);
      search();
    }
  }
  
  /**
   * Called by base class when backspace is typed.
   */
  function handleBackSpace()
  {
    if( str.length() > 0 )
    {
      lastStr = str = str[0..str.length()-2];
      caret.moveDot( mark + str.length() );
      refresh();
    }
  }
  
  function search()
  {
    status = "search";
    var off = util.searchForwardFrom( str, mark );
    if( off == -1 )
    {
      off = util.searchForwardFrom( str, 0 );
      if( off == -1 )
        status = "failed";
      else
        status = "wrapped";
    }
    
    if( off != -1 )
    {
      mark = off;
      caret.setDot(mark);
      caret.moveDot( mark + str.length() );
    }
    
    refresh();
  }
  
  function searchAgain(evt)
  {
    if( str.length() == 0 )
      str = lastStr;
    mark += str.length();
    search();
  }
  
  function refresh()
  {
    frame.setStatusLineText(status + ": " + str);
  }
  
  addKeyFxn( "ctrl F", "search-again", null, searchAgain );
  addKeyFxn( "ENTER",  "search-again", null, searchAgain );
  
  frame.setStatusLineIcon( pkg.util.getImageIcon("Find16.gif") );
  refresh();
  
  // if the user moves the cursor, such as by clicking the
  // mouse, we want to bail out of search mode:
  buf.addMouseListener( 
    new (function() extends java.awt.event.MouseListener() {
      
      private function exit(evt) { leaveMode(); }
      private function noop(evt) {}
      
      public var mouseClicked  = exit;
      public var mousePressed  = exit;
      public var mouseReleased = exit;
      public var mouseEntered  = noop;
      public var mouseExited   = noop;
      
    })() 
  );
}


globalMode.addKeyFxn( "ctrl F", "search-mode", "interactive search", '{
  new SearchMode();
} );
globalMode.addKeyFxn( "ctrl COMMA", "search-mode", "interactive search", '{
  new SearchMode();
} );

globalMode.addKeyFxn( "ctrl shift F", "replace-mode", "search and replace", '{
  
  var replace = util.prompt("replace");
  if( replace == null )
    return;
  
  if( replace != "" )
  {
    var with = util.prompt("replace \"" + replace + "\" with");
    if( with == null )
      return;
    
    if( with != "" )
    {
      var frame = EditorFrame.getCurrentFrame();
      var buf   = frame.getBuffer();
      var doc   = buf.getDocument();
      var caret = buf.getCaret();
      var mark  = caret.getDot();
      
      while(true)
      {
        var off = util.searchForwardFrom( replace, mark );
        
        if( off == -1 )
        {
          if( ! util.promptYesNo("end of document; continue from top?") )
            break;
          
          mark = 0;
        }
        else
        {
          mark = off + replace.length();
          caret.setDot(off);
          caret.moveDot( off + replace.length() );
          
          var choice = util.promptChoice("replace?",["y","n","c"]);
          
          if( choice == "n" )
          {
            break;
          }
          else if( choice == "y" )
          {
            doc.remove( off, replace.length() );
            doc.insertString( off, with, null );
          }
          
          mark = off + with.length();
        }
      }
    }
  }
} );

globalMode.addKeyFxn( "ctrl S", "save-buffer", "save the contents of the current buffer", '{ 
  EditorFrame.getCurrentFrame().save();
} );

globalMode.addKeyFxn( "ctrl Z", "undo", "undo the previous operation", '{ 
  EditorFrame.getCurrentFrame().undo();
} );

globalMode.addKeyFxn( "ctrl shift Z", "redo", "redo the previous undo", '{ 
  EditorFrame.getCurrentFrame().undo();
} );

function promptAndGotoLine(evt)
{
  util.promptAndGotoLine();
}

globalMode.addKeyFxn( "F5", "goto-line", "prompt and goto line", promptAndGotoLine );
globalMode.addKeyFxn( "ctrl G", "goto-line", "prompt and goto line", promptAndGotoLine );
globalMode.addPopupMenuAction( "Goto Line  (F5)", "Prompt and goto line", null, null, promptAndGotoLine );

globalMode.addPopupMenuAction( "Cut",
                               "Cut the selected text to the clipboard",
                               "Cut16.gif",
                               null,
                               '{ EditorFrame.getCurrentFrame().cut(); } );

globalMode.addPopupMenuAction( "Copy",
                               "Copy the selected text to the clipboard",
                               "Copy16.gif",
                               null,
                               '{ EditorFrame.getCurrentFrame().copy(); } );

globalMode.addPopupMenuAction( "Paste",
                               "Paste the text from the clipboard",
                               "Paste16.gif",
                               null,
                               '{ EditorFrame.getCurrentFrame().paste(); } );

globalMode.addHook( "open-file", function(frame) {
  
  // XXX should have a more general way to determine what mode to use
  // based on extension:
  var file = frame.getFile();
  if( file != null )
  {
    var ext = file.getExtension();
    if( ext == "os" )
      frame.setMode(osMode);
    else if( (ext == "c") || (ext == "java") )
      frame.setMode(cMode);
  }
  
} );



/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
