/******************************************************************************
 *     Copyright Texas Instruments 2002-2004. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//pkg.output.writeln("loading pkg.editor.osMode");

mixin pkg.editor;

//const var currentTimeMillis = java.lang.System.currentTimeMillis;
const var st = oscript.syntaxtree;
const var ParserConstants = oscript.parser.OscriptParserConstants;

public const var osMode = new (function() extends Mode( cMode, "os" ) {
  
  const var ERROR_GLYPH = pkg.util.getImageIcon("error-glyph.gif");
  const var Color = java.awt.Color;
  const var Font  = java.awt.Font;
  
  /****************************************************************************
   * need to track the parser for a given frame, so that goToDeclaration()
   * (and others?) can get at the syntax tree for the current frame.  Also
   * handy for tracking frames that need repaint when attribute changes.
   */
  var docToParserTable = new java.util.Hashtable();
  
  /****************************************************************************
   * the attribute-set-table maps token kind to an attribute-set.  It is
   * possible that multiple different kinds map to the same attribute set.
   */
  var attrSetTable = new oscript.util.SymbolMap();
  
  /****************************************************************************
   * Attribute set factory.
   */
  function makeAttributeSet( name, color, style )
  {
    function repaintAllFrames()
    {
      var frame = EditorFrame.getCurrentFrame();
      if( frame != null )
        frame.getBuffer().repaint();
    }
    
    if( color == null )
      color = Color.black;
    
    if( style == null )
      style = Font.PLAIN;
    
    var attrSet = new swing.text.SimpleAttributeSet();
    
    if( ! registry.exists("/Preferences/Editor/ObjectScript Mode/" + name) )
    {
      registry.link( 
        new ti.chimera.registry.PersistentNode( 
          color, 
          new ti.chimera.pref.ColorNodeContract(),
          "the color for <i>" + name + "</i> tokens" ),
        "/Preferences/Editor/ObjectScript Mode/" + name + "/Color" );
      
      registry.link( 
        new ti.chimera.registry.PersistentNode( 
          style.castToString(), 
          new ti.chimera.pref.ChoiceNodeContract(
            [ Font.PLAIN.castToString(), Font.BOLD.castToString(), Font.ITALIC.castToString(), (Font.BOLD | Font.ITALIC).castToString() ],
            [ "Plain",                   "Bold",                   "Italic",                   "Italic Bold" ] ),
          "the style for <i>" + name + "</i> tokens" ),
        "/Preferences/Editor/ObjectScript Mode/" + name + "/Style" );
    }
    
    registry.subscribeToValue( "/Preferences/Editor/ObjectScript Mode/" + name + "/Color", null, function( node, color ) {
      attrSet.addAttribute( swing.text.StyleConstants.Foreground, color );
      repaintAllFrames();
    } );
    
    
    registry.subscribeToValue( "/Preferences/Editor/ObjectScript Mode/" + name + "/Style", null, function( node, style ) {
      attrSet.addAttribute( "style", style.castToExactNumber() );
      repaintAllFrames();
    } );
    
    return attrSet;
  }
  
  registry.subscribeToValue( "/Preferences/Editor/Foreground Color", null, function( node, color ) {
    oscript.swing.text.ODEEditorKit.setDefaultFgColor(color);
  } );
  
  var COMMENT_ATTR = makeAttributeSet( "Comment", Color.green.darker(), Font.ITALIC );
  attrSetTable.put( ParserConstants.SINGLE_LINE_COMMENT, COMMENT_ATTR );
  attrSetTable.put( ParserConstants.FORMAL_COMMENT, COMMENT_ATTR );
  attrSetTable.put( ParserConstants.MULTI_LINE_COMMENT, COMMENT_ATTR );

  var JAVADOC_TAG_ATTR = makeAttributeSet( "Javadoc Tag", Color.red.darker(), Font.ITALIC );
  attrSetTable.put( ParserConstants.JAVADOC_TAG, JAVADOC_TAG_ATTR );
  attrSetTable.put( ParserConstants.JAVADOC_TAG_WITH_ARG, JAVADOC_TAG_ATTR );

  var JAVADOC_TAG_ARG_ATTR = makeAttributeSet( "Javadoc Tag Argument", Color.blue.darker(), Font.ITALIC );
  attrSetTable.put( ParserConstants.JAVADOC_TAG_ARG, JAVADOC_TAG_ARG_ATTR );
  
  var FLOW_CONTROL_ATTR = makeAttributeSet( "Flow Control Keyword", Color.red, null );
  attrSetTable.put( ParserConstants.BREAK, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.CATCH, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.CONTINUE, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.ELSE, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.FINALLY, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.FOR, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.IF, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.RETURN, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.SYNCHRONIZED, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.THROW, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.TRY, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.WHILE, FLOW_CONTROL_ATTR );
  
  var KEY_WORD_ATTR = makeAttributeSet( "Keyword", Color.blue, Font.BOLD );
  attrSetTable.put( ParserConstants.EVAL, KEY_WORD_ATTR );
  attrSetTable.put( ParserConstants.FUNCTION, KEY_WORD_ATTR );
  attrSetTable.put( ParserConstants.IMPORT, KEY_WORD_ATTR );
  attrSetTable.put( ParserConstants.MIXIN, KEY_WORD_ATTR );
  attrSetTable.put( ParserConstants.VAR, KEY_WORD_ATTR );
  attrSetTable.put( ParserConstants.NEW, KEY_WORD_ATTR );
  attrSetTable.put( ParserConstants.EXTENDS, KEY_WORD_ATTR );
  
  var KEY_VAL_ATTR = makeAttributeSet( "Special Value", Color.orange, Font.BOLD );
  attrSetTable.put( ParserConstants.FALSE, KEY_VAL_ATTR );
  attrSetTable.put( ParserConstants.NULL, KEY_VAL_ATTR );
  attrSetTable.put( ParserConstants.UNDEFINED, KEY_VAL_ATTR );
  attrSetTable.put( ParserConstants.THIS, KEY_VAL_ATTR );
  attrSetTable.put( ParserConstants.CALLEE, KEY_VAL_ATTR );
  attrSetTable.put( ParserConstants.TRUE, KEY_VAL_ATTR );
  
  var KEY_MODIFIER_ATTR = makeAttributeSet( "Modifier", Color.cyan, Font.ITALIC );
  attrSetTable.put( ParserConstants.STATIC, KEY_MODIFIER_ATTR );
  attrSetTable.put( ParserConstants.CONST, KEY_MODIFIER_ATTR );
  attrSetTable.put( ParserConstants.PRIVATE, KEY_MODIFIER_ATTR );
  attrSetTable.put( ParserConstants.PROTECTED, KEY_MODIFIER_ATTR );
  attrSetTable.put( ParserConstants.PUBLIC, KEY_MODIFIER_ATTR );
  
  var LITERAL_ATTR = makeAttributeSet( "Literal", Color.orange.darker(), null );
  attrSetTable.put( ParserConstants.INTEGER_LITERAL, LITERAL_ATTR );
  attrSetTable.put( ParserConstants.DECIMAL_LITERAL, LITERAL_ATTR );
  attrSetTable.put( ParserConstants.HEX_LITERAL, LITERAL_ATTR );
  attrSetTable.put( ParserConstants.OCTAL_LITERAL, LITERAL_ATTR );
  attrSetTable.put( ParserConstants.FLOATING_POINT_LITERAL, LITERAL_ATTR );
  attrSetTable.put( ParserConstants.EXPONENT, LITERAL_ATTR );
  attrSetTable.put( ParserConstants.ESCAPE_SEQUENCE, LITERAL_ATTR );
  attrSetTable.put( ParserConstants.STRING_LITERAL, LITERAL_ATTR );
  attrSetTable.put( ParserConstants.REGEXP_LITERAL, LITERAL_ATTR );
  
  var PAREN_ATTR = makeAttributeSet( "Paren", Color.green.darker(), Font.BOLD );
  attrSetTable.put( ParserConstants.LPAREN, PAREN_ATTR );
  attrSetTable.put( ParserConstants.RPAREN, PAREN_ATTR );
  attrSetTable.put( ParserConstants.LFXNBRACE, PAREN_ATTR );
  attrSetTable.put( ParserConstants.LBRACE, PAREN_ATTR );
  attrSetTable.put( ParserConstants.RBRACE, PAREN_ATTR );
  attrSetTable.put( ParserConstants.LBRACKET, PAREN_ATTR );
  attrSetTable.put( ParserConstants.RBRACKET, PAREN_ATTR );
  
  var OPERATOR_ATTR = makeAttributeSet( "Operator", Color.blue.darker().darker(), Font.BOLD );
  attrSetTable.put( ParserConstants.INSTANCEOF, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.SEMICOLON, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.COMMA, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.DOT, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.ASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.GT, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.LT, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.BANG, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.TILDE, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.HOOK, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.COLON, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.EQ, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.LE, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.GE, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.NE, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.SC_OR, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.SC_AND, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.INCR, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.DECR, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.PLUS, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.MINUS, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.STAR, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.SLASH, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.BIT_AND, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.BIT_OR, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.XOR, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.REM, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.LSHIFT, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.RSIGNEDSHIFT, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.RUNSIGNEDSHIFT, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.PLUSASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.MINUSASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.STARASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.SLASHASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.ANDASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.ORASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.XORASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.REMASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.LSHIFTASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.RSIGNEDSHIFTASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.RUNSIGNEDSHIFTASSIGN, OPERATOR_ATTR );
  
  
  /****************************************************************************
   * Install this mode into the frame.
   */
  public function initializeFrame(frame)
  {
    var editorKit = frame.getBuffer().getEditorKit();
    editorKit.setAttributeSetTable(attrSetTable);
    
    var doc = frame.getBuffer().getDocument();
    var parser = docToParserTable.get(doc);
    
    if( parser == null )
    {
      parser = new ObjectScriptParser( new WeakReference(doc), editorKit );
      docToParserTable.put( doc, parser );
      frame.addCloseListener( '{
        docToParserTable.remove(doc);
      } );
      workerThread.invokeLater( '{ parser.parse(); } );
    }
    
    parser.addFrame(frame);
    
    // for debug:
    if(false)
      addKeyFxn( "ctrl T", "print-token", "print current token", '{
        var off = EditorFrame.getCurrentFrame().getBuffer().getCaretPosition();
        pkg.output.writeln("token: off=" + off + ", " + editorKit.offsetToNodeToken(off));
      } );
  }
  
  /**
   * The visitor that handles walking the syntax tree after the parse
   * is complete.
   */
  var visitor = new oscript.visitor.ExtensibleVisitor();
  
  /**
   * parser corresponding to <code>doc</code>... logically it could be an
   * inner class of the <code>Document</code>, but I wanted to implement this
   * control code in script.
   */
  function ObjectScriptParser( doc, editorKit )
  {
    var tokensVector = null;
    var syntaxTree = null;
    var frameTable = new java.util.WeakHashMap();
    var errorMsg   = null;
    var errorOff   = -1;
    
    var root = new FunctionTreeNode( "functions", true, null );
    var treeModel = new swing.tree.DefaultTreeModel(root);
    root.setTreeModel(treeModel);
    
    /* to improve performance, set a short timer after document mutated
     * before parsing.  Reset this timer whenever the document is mutated
     * in order to avoid reparsing while the user is still typing
     */
    var parseTimer = new swing.Timer( 300, '{
      parseTimer.stop();
      reparse();
    } );
    
    /**
     * Get the syntax-tree from the last parse of the this document
     * 
     * @return the root of the syntax-tree
     */
    public function getSyntaxTree()
    {
      return syntaxTree;
    }
    
    /**
     * Get the offset of the current parse error.
     * 
     * @return the offset, or <code>null</code> if no parse error
     */
    public function getErrorOffset()
    {
      return errorOff;
    }
    
    /**
     * Add frame to the list of frames being served by this parser.
     * This is needed to, for example, message the frame to repaint
     * when a parse is completed, etc.
     * 
     * @param frame   the frame
     */
    public function addFrame(frame)
    {
      if( frameTable.get(frame) == null )
      {
        frameTable.put( frame, "foo" );
        frame.getContainer().addUtilityTab( "functions", '{
          return makeFunctionTree(frame);
        } );
      }
    }
    
    private function makeFunctionTree(frame)
    {
      var tree = new swing.JTree(treeModel);
      
      tree.setCellRenderer( new (function() extends swing.tree.DefaultTreeCellRenderer() {
        var _super_getTreeCellRendererComponent = getTreeCellRendererComponent;
        public function getTreeCellRendererComponent( tree, value, selected, expanded, leaf, row, hasFocus )
        {
          _super_getTreeCellRendererComponent( tree, value, selected, expanded, leaf, row, hasFocus );
          setIcon( value.getIcon() );
          // XXX setToolTipText(...)
          return this;
        }
      })() );
      
      // when user selects different element in tree, scroll to that
      // functions declaration in the code
      tree.addTreeSelectionListener( new (function() extends swing.event.TreeSelectionListener() {
        
        public function valueChanged(evt)
        {
          var buf = frame.getBuffer();
          var fxn = evt.getPath().getLastPathComponent();
          var off = fxn.getOffset();
          if( off != -1 )
            frame.scrollToOffset(off);
          buf.requestFocus();
        }
        
      })() );
      
      // automatically fully expand the tree:
      treeModel.addTreeModelListener( new (function() extends swing.event.TreeModelListener() {
        
        public function treeNodesChanged(evt) {}
        
        public function treeNodesInserted(evt)
        {
          swing.SwingUtilities.invokeLater( function() {
            tree.expandPath( evt.getTreePath() );
          } );
        }
        
        public function treeNodesRemoved(evt) {}
        public function treeStructureChanged(evt) {}
        
      })() );
      
      // expand any rows that might exist prior to adding the TreeModelListener:
      for( var i=0; i<tree.getRowCount(); i++ )
        tree.expandRow(i);
      
      return tree;
    }
    
    public function removeFrame(frame)
    {
      frameTable.remove(frame);
    }
    
    var waitingToParse = false;
    
    /**
     * Called from the context of the parser thread, to parse this document.
     */
    public function parse()
    {
      var lastMutateTime;
      var in = doc.getDocumentReader();
      
//      synchronized(this)
      {
        lastMutateTime = doc.getLastMutateTime();
        waitingToParse = false;
      }
      
      try
      {
        oscript.parser.OscriptParser.resetNodeTokens();
        
        // if the input stream is already out of date, bail out:
        if( lastMutateTime != doc.getLastMutateTime() )
        {
          reparse();
          return;
        }
        
        syntaxTree = oscript.DefaultParser.parse( in, "bogus.os" );
        
        // if the parse is already out of date, bail out:
        if( lastMutateTime != doc.getLastMutateTime() )
        {
          reparse();
          return;
        }
        
        editorKit.setNodeTokens( tokensVector = oscript.parser.OscriptParser.getNodeTokens() );
        
        var needsClearStatusLine = errorMsg != null;
        
        errorMsg = null;
        errorOff = -1;
        
        swing.SwingUtilities.invokeLater( '{
          
          for( var itr=frameTable.keySet().iterator(); itr.hasNext(); )
          {
            var frame = itr.next();
            if(needsClearStatusLine)
              frame.clearStatusLine();
            frame.getBuffer().repaint();
          }
          
          // XXX we could use a "hook" mechanism to move FunctionTree out of osMode
          root.visitParse( visitor, syntaxTree, null );
          
        } );
      }
      catch(oscript.parser.ParseException e)
      {
        if( e.currentToken != null )
        {
          var lastErrorLine = e.currentToken.endLine;
          errorMsg = "parse error near \"" + e.currentToken + "\" (line " + lastErrorLine + ")";
          errorOff = e.currentToken.endOffset;
          setErrorMsg(errorMsg);
        }
        else
        {
          setErrorMsg( e.getMessage() );
        }
      }
      catch(oscript.parser.TokenMgrError e)
      {
        setErrorMsg( e.getMessage() );
      }
    }
    
    function setErrorMsg(msg)
    {
      swing.SwingUtilities.invokeLater( '{
        for( var itr=frameTable.keySet().iterator(); itr.hasNext(); )
        {
          var frame = itr.next();
          frame.setStatusLineIcon(ERROR_GLYPH);
          frame.setStatusLineText(msg);
        }
      } );
    }
    
    /**
     * Requeust a reparse.
     */
    function reparse()
    {
      synchronized(this)
      {
        if( !waitingToParse )
        {
          waitingToParse = true;
          workerThread.invokeLater(parse);
        }
      }
      return null;
    }
    
    doc.addDocumentListener( new (function() extends swing.event.DocumentListener() {
      public function insertUpdate(evt)  { parseTimer.restart(); }
      public function removeUpdate(evt)  { parseTimer.restart(); }
      public function changedUpdate(evt) { parseTimer.restart(); }
    })() );
  }
  
  /**
   * Move the cursor to the offset of the current parse error,
   * if there is one.
   */
  function goToError()
  {
    var frame = EditorFrame.getCurrentFrame();
    var off   = docToParserTable.get( frame.getBuffer().getDocument() ).getErrorOffset();
    if( off == -1 )
    {
      frame.displayStatusMessage("no error");
      return;
    }
    frame.scrollToOffset(off);
  }
  
  /**
   * Move the cursor to the offset of the declaration of the 
   * <i>IdentifierPrimaryPrefix<i> at the specified offset into 
   * the document.
   * 
   * @param off    the offset of the <i>IDENTIFIER</i> that is a
   *    reference to a variable/function
   */
  function goToDeclaration(off)
  {
    var frame = EditorFrame.getCurrentFrame();
    var doff  = findDeclaration( off, true );
    if( doff == -1 )
    {
      frame.displayStatusMessage("no declaration found");
      return;
    }
    frame.scrollToOffset(doff);
  }
  
  /**
   * Move the cursor to the next or previous reference to the
   * <i>IdentifierPrimaryPrefix</i> at the specified offset in the
   * document.
   * 
   * @param off    the offset of the <i>IDENTIFIER</i> that is a
   *    reference to a variable/function
   * @param next   if <code>true</code>, search forward, else
   *    search backward
   */
  function goToReference( off, next )
  {
    var frame = EditorFrame.getCurrentFrame();
    var roff = findReference( off, next );
    if( roff == -1 )
    {
      frame.displayStatusMessage("no " + (next ? "next" : "previous") + " reference");
      return;
    }
    frame.scrollToOffset(roff);
  }
  
  /**
   * Exception used in findReference/findDeclaration to bail out
   * of the syntaxtree search once we find what we're looking for.
   */
  function Done() extends Exception("done") {}
  
  /**
   * Find the offset of the next or previous reference to the
   * <i>IdentifierPrimaryPrefix</i> at the specified offset in the
   * document.
   * 
   * @param off    the offset of the <i>IDENTIFIER</i> that is a
   *    reference to a variable/function
   * @param next   if <code>true</code>, search forward, else
   *    search backward
   * @return the offset, or <code>-1</code> if not found
   */
  public function findReference( off, next )
  {
    var frame      = EditorFrame.getCurrentFrame();
    var syntaxTree = docToParserTable.get( frame.getBuffer().getDocument() ).getSyntaxTree();
    var visitor    = new oscript.visitor.ExtensibleVisitor();
    
    var doff = findDeclaration( off, true );
    if( doff == -1 )
      return -1;
    
    var token = frame.getBuffer().getEditorKit().getToken(off);
    var img   = frame.getBuffer().getEditorKit().getToken(doff).otokenImage; // not always same as token.otokenImage
    var roff  = -1;
    
    /**
     * Test for the termination condition of the algorithm.
     * 
     * @param n   the node to test
     */
    function test(n)
    {
      /* next:  first matching with n.beginOffset > token.endOffset
       * prev:  last matching with n.endOffset < token.beginOffset
       */
      
      /* check if this identifier refers to the same thing as token...
       * string compare first, because that is the least expensive to 
       * perform.  If passes both test, then this is a reference to 
       * the same variable, so we need to figure out when and what to 
       * return (via <code>roff</code>)
       */
      if( (n.otokenImage == img) && (findDeclaration( n.beginOffset, false ) == doff) )
      {
        if(next)
          roff = n.beginOffset;
        
        if( (!next && (n == token)) ||   // if looking for prev, bail when starting point reached
            (n.beginOffset >= token.endOffset) )
          throw new Done();
        
        if(!next)
          roff = n.beginOffset;
      }
    }
    
    /**
     * <PRE>
     * f0 -> &lt;IDENTIFIER&gt;
     * </PRE>
     */
    visitor.register( st.IdentifierPrimaryPrefix, function( n, arg ) {
      test(n.f0);
    } );
    
    /**
     * <PRE>
     * f0 -> Permissions(false)
     * f1 -> &lt;IDENTIFIER&gt;
     * f2 -> ( "," Permissions(false) &lt;IDENTIFIER&gt; )*
     * f3 -> ( "..." )?
     * </PRE>
     */
    visitor.register( st.Arglist, function( n, scope ) {
      test(n.f1);
      for( var i=0; i<n.f2.size(); i++ )
        test( n.f2.elementAt(i).elementAt(2) );
    } );
    
    /**
     * <PRE>
     * f0 -> Permissions()
     * f1 -> "function"
     * f2 -> &lt;IDENTIFIER&gt;
     * f3 -> "("
     * f4 -> ( Arglist() )?
     * f5 -> ")"
     * f6 -> ( "extends" PrimaryExpressionWithTrailingFxnCallExpList() FunctionCallExpressionList() )?
     * f7 -> "{"
     * f8 -> Program()
     * f9 -> "}"
     * </PRE>
     */
    visitor.register( st.FunctionDeclaration, function( n, scope ) {
      test(n.f2);
      n.f4.accept( visitor, scope );
      n.f6.accept( visitor, scope );
      n.f8.accept( visitor, scope );
    } );
    
    /**
     * <PRE>
     * f0 -> Permissions()
     * f1 -> "var"
     * f2 -> &lt;IDENTIFIER&gt;
     * f3 -> ( "=" Expression() )?
     * </PRE>
     */
    visitor.register( st.VariableDeclaration, function( n, scope ) {
      test(n.f2);
      n.f3.accept( visitor, scope );
    } );
    
    /**
     * f0 -> "try"
     * f1 -> EvaluationUnit()
     * f2 -> ( "catch" "(" Expression() &lt;IDENTIFIER&gt; ")" EvaluationUnit() )*
     * f3 -> ( "catch" "(" &lt;IDENTIFIER&gt; ")" EvaluationUnit() )?
     * f4 -> ( "finally" EvaluationUnit() )?
     */
    visitor.register( st.TryStatement, function( n, scope ) {
      n.f1.accept( visitor, scope );
      for( var i=0; i<n.f2.size(); i++ )
      {
        var elem = n.f2.elementAt(i);
        elem.elementAt(2).accept( visitor, scope );
        test( elem.elementAt(3) );
        elem.elementAt(5).accept( visitor, scope );
      }
      if( n.f3.present() )
      {
        test( n.f3.node.elementAt(2) );
        n.f3.node.elementAt(4).accept( visitor, scope );
      }
      n.f4.accept( visitor, scope );
    } );
    
    try
      syntaxTree.accept( visitor, null );
    catch(Done e)
      return roff;
    
    return -1;
  }
  
  var previousDeclarationPositionTable = new java.util.WeakHashMap(); // maps frame -> position
  
  /**
   * Find the offset of the declaration of the 
   * <i>IdentifierPrimaryPrefix<i> at the specified offset into 
   * the document.
   * 
   * @param off    the offset of the <i>IDENTIFIER</i> that is a
   *    reference to a variable/function
   * @param reuse  if <code>true</code>, and <code>off</code> does
   *    not refer to a reference or declaration, then reuse the
   *    previous declaration 
   * @return the offset of the declaration, if found, or <code>-1</code>
   *    if not found
   */
  public function findDeclaration( off, reuse )
  {
    var frame      = EditorFrame.getCurrentFrame();
    var syntaxTree = docToParserTable.get( frame.getBuffer().getDocument() ).getSyntaxTree();
    var visitor    = new oscript.visitor.ExtensibleVisitor();
    var doff = -1;
    var isReference = false;
    
    /**
     * Instances of this class are used to track what members are
     * declared in a given scope, and for mapping member names to
     * a token where that member was declared.
     * <p>
     * To get a better "hit" rate for references within functions
     * to variables declared outside the function, we defer parsing
     * the function's body until the outer scope is closed, and
     * so the scope is also used to make a list of nodes that need
     * to be visited when the scope is closed.
     */
    function Scope(prev)
    {
      var memberTable = null;
      var deferredFxnList = null;
      
      /**
       * Declare a variable/function.
       * 
       * @param token    the <i>IDENTIFIER</i> token for the 
       *    variable declaration
       */
      public function declare(token)
      {
        if( (token.beginOffset <= off) && (off <= token.endOffset) )
        {
          doff = token.beginOffset;
          throw new Done();
        }
        
        if( memberTable == null )
          memberTable = new java.util.Hashtable();
          
        memberTable.put( token.otokenImage, token );
      }
      
      /**
       * Lookup a variable/function.
       * 
       * @param token    the <i>IDENTIFIER</i> token for the 
       *    variable reference
       * @return the token, or <code>null</code> if not found
       */
      public function lookup(token)
      {
        var name = token.otokenImage;
        var val = null;
        if( memberTable != null )
          val = memberTable.get(name);
        if( (val == null) && (prev != null) )
          val = prev.lookup(token);
        return val;
      }
      
      /**
       * Evaluate functions when the scope is closed.
       * 
       * @param fxn      the function to evaluate
       */
      public function deferEvaluation(fxn)
      {
        if( deferredFxnList == null )
          deferredFxnList = new java.util.LinkedList();
        deferredFxnList.add(fxn);
      }
      
      /**
       * Close this scope.
       */
      public function close()
      {
        while( deferredFxnList != null )
        {
          var tmp = deferredFxnList;
          deferredFxnList = null;
          
          for( var itr=tmp.iterator(); itr.hasNext(); )
            itr.next()();
        }
      }
      
      /**
       * Get the root of the scope tree.  The root is the scope
       * node that has no previous scope.
       * 
       * @return the root scope node
       */
      public function getRoot()
      {
        return (prev != null) ? prev.getRoot() : this;
      }
    }
    
    /**
     * <PRE>
     * f0 -> "{"
     * f1 -> Program()
     * f2 -> "}"
     * </PRE>
     */
    visitor.register( st.ScopeBlock, function( n, scope ) {
      parseProgram( [n.f1], scope );
    } );
    
    /**
     * <PRE>
     * f0 -> Permissions(false)
     * f1 -> &lt;IDENTIFIER&gt;
     * f2 -> ( "," Permissions(false) &lt;IDENTIFIER&gt; )*
     * f3 -> ( "..." )?
     * </PRE>
     */
    visitor.register( st.Arglist, function( n, scope ) {
      scope.declare(n.f1);
      for( var i=0; i<n.f2.size(); i++ )
        scope.declare( n.f2.elementAt(i).elementAt(2) );
    } );
    
    /**
     * <PRE>
     * f0 -> Permissions()
     * f1 -> "function"
     * f2 -> &lt;IDENTIFIER&gt;
     * f3 -> "("
     * f4 -> ( Arglist() )?
     * f5 -> ")"
     * f6 -> ( "extends" PrimaryExpressionWithTrailingFxnCallExpList() FunctionCallExpressionList() )?
     * f7 -> "{"
     * f8 -> Program()
     * f9 -> "}"
     * </PRE>
     */
    visitor.register( st.FunctionDeclaration, function( n, scope ) {
      scope.declare(n.f2);
      n.f6.accept( visitor, scope );
      scope.deferEvaluation( function() {
        parseProgram( [ n.f4, n.f8 ], scope );
      } );
    } );
    
    /**
     * <PRE>
     * f0 -> Permissions()
     * f1 -> "var"
     * f2 -> &lt;IDENTIFIER&gt;
     * f3 -> ( "=" Expression() )?
     * </PRE>
     */
    visitor.register( st.VariableDeclaration, function( n, scope ) {
      scope.declare(n.f2);
      n.f3.accept( visitor, scope );
    } );
    
    /**
     * f0 -> "try"
     * f1 -> EvaluationUnit()
     * f2 -> ( "catch" "(" Expression() &lt;IDENTIFIER&gt; ")" EvaluationUnit() )*
     * f3 -> ( "catch" "(" &lt;IDENTIFIER&gt; ")" EvaluationUnit() )?
     * f4 -> ( "finally" EvaluationUnit() )?
     */
    visitor.register( st.TryStatement, function( n, scope ) {
      n.f1.accept( visitor, scope );
      for( var i=0; i<n.f2.size(); i++ )
      {
        var elem = n.f2.elementAt(i);
        elem.elementAt(2).accept( visitor, scope );
        scope.deferEvaluation( function() {
          parseProgram( [ elem.elementAt(5) ], scope, elem.elementAt(3) );
        } );
      }
      if( n.f3.present() )
      {
        scope.deferEvaluation( function() {
          parseProgram( [ n.f3.node.elementAt(4) ], scope, n.f3.node.elementAt(2) );
        } );
      }
      n.f4.accept( visitor, scope );
    } );
    
    /**
     * <PRE>
     * f0 -> "function"
     * f1 -> "("
     * f2 -> ( Arglist() )?
     * f3 -> ")"
     * f4 -> ( "extends" PrimaryExpressionWithTrailingFxnCallExpList() FunctionCallExpressionList() )?
     * f5 -> "{"
     * f6 -> Program()
     * f7 -> "}"
     * </PRE>
     */
    visitor.register( st.FunctionPrimaryPrefix, function( n, scope ) {
      n.f4.accept( visitor, scope );
      scope.deferEvaluation( function() {
        parseProgram( [ n.f2, n.f6 ], scope );
      } );
    } );
    
    /**
     * <PRE>
     * f0 -> &lt;IDENTIFIER&gt;
     * </PRE>
     */
    visitor.register( st.IdentifierPrimaryPrefix, function( n, scope ) {
      
      /* NOTE: need to defer lookup() until scope is closed (via
       *       deferEvaluation()) to deal with cases of forward 
       *       references
       */
      if( (n.f0.beginOffset <= off) && (off <= n.f0.endOffset) )
      {
        isReference = true;
        scope.getRoot().deferEvaluation( function() {
          
          var token = scope.lookup(n.f0);
          if( token != null )
          {
            doff = token.beginOffset;
            throw new Done();
          }
          
        } );
      }
    } );
    
    function parseProgram( nodes, scope, declareNodes... )
    {
      var newScope = new Scope(scope);
      for( var i=0; i<declareNodes.length(); i++ )
        newScope.declare( declareNodes[i] );
      for( var i=0; i<nodes.length(); i++ )
        nodes[i].accept( visitor, newScope );
      newScope.close();
    }
    
    try
    {
      parseProgram( [syntaxTree], null );
    }
    catch(Done e)
    {
      previousDeclarationPositionTable.put( frame, frame.getBuffer().getDocument().createPosition(doff) );
      return doff;
    }
    
    if(reuse)
    {
      var previousDeclarationPosition = previousDeclarationPositionTable.get(frame);
      if( (previousDeclarationPosition != null) && !isReference )
        return previousDeclarationPosition.getOffset();
    }
    
    frame.displayStatusMessage("declaration not found");
    return -1;
  }
  
  addKeyFxn( "ctrl E", "goto-error", "go to the parse error", '{ goToError(); } );
  
  addKeyFxn( "ctrl D", "goto-declaration", "go to the declaration of the variable at the cursor", '{
    
    goToDeclaration( EditorFrame.getCurrentFrame().getBuffer().getCaretPosition() );
    
  } );
  
  addKeyFxn( "ctrl shift PERIOD", "goto-next-reference", "go to the next reference to the variable at the cursor", '{
    
    goToReference( EditorFrame.getCurrentFrame().getBuffer().getCaretPosition(), true );
    
  } );
  
  addKeyFxn( "ctrl shift COMMA", "goto-prev-reference", "go to the previous reference to the variable at the cursor", '{
    
    goToReference( EditorFrame.getCurrentFrame().getBuffer().getCaretPosition(), false );
    
  } );
  
  function evalBuffer(evt)
  {
    var frame = EditorFrame.getCurrentFrame();
    frame.save();
    var file = frame.getFile();
    
    if( file == null )
      return;
    
    services["console"].doImport(file);
  }
  
  addKeyFxn( "ctrl shift I", "eval-buffer", "Evaluate the current buffer", evalBuffer );
  addPopupMenuAction( "Eval Buffer", "Evaluate the current buffer", "Play16.gif", null, evalBuffer );
  
  addKeyFxn( null, "eval-selected", "Evaluate the selected text", '{
    var frame = EditorFrame.getCurrentFrame();
    var buf   = frame.getBuffer();
    var text  = buf.getSelectedText();
    oscript.OscriptInterpreter.__eval(text);
  } );
  
})();



/**
 * Set a breakpoint on the current line of the current buffer.
 */
osMode.addPopupMenuActionFxn( '{
  
  var frame = EditorFrame.getCurrentFrame();
  var file  = frame.getFile();
  var line  = frame.getLineNumber();
  
  if( file == null )
    return null;
  
  if( ! services["debugger"].hasBreakpoint( file, line ) )
  {
    return [[
      "Set Breakpoint",
      "Set a breakpoint at the cursor",
      "breakpointglyph.gif",
      null,
      '{
        frame.displayStatusMessage("setting breakpoint at " + file + ":" + line);
        services["debugger"].setBreakpoint( file, line );
      } 
    ]];
  }
  
  return null;
} );

/**
 * Clear the breakpoint (if any) on the current line of the current buffer.
 */
osMode.addPopupMenuActionFxn( '{
  
  var frame = EditorFrame.getCurrentFrame();
  var file  = frame.getFile();
  var line  = frame.getLineNumber();
  
  if( file == null )
    return null;
  
  if( services["debugger"].hasBreakpoint( file, line ) )
  {
    return [[
      "Clear Breakpoint", 
      "Clear an existing breakpoint at the cursor", 
      "disabledbreakpointglyph.gif", 
      null, 
      '{
        frame.displayStatusMessage("clearing breakpoint at " + file + ":" + line);
        services["debugger"].clearBreakpoint( file, line );
      } 
    ]];
  }
  
  return null;
} );

osMode.addHook( "set-mode", function(frame) {
  frame.addGutterClickListener( function(line) {
    
    var file = frame.getFile();
    
    if( file == null )
      return;
    
    var debugger = services["debugger"];
    (debugger.hasBreakpoint( file, line ) ? debugger.clearBreakpoint : debugger.setBreakpoint)( file, line );
    
  } );
} );

var functionIconTable = new java.util.Hashtable();
functionIconTable.put( "public-method",       pkg.util.getImageIcon("methodPublic.gif") );
functionIconTable.put( "private-method",      pkg.util.getImageIcon("methodPrivate.gif") );
functionIconTable.put( "public-constructor",  pkg.util.getImageIcon("constructorPublic.gif") );
functionIconTable.put( "private-constructor", pkg.util.getImageIcon("constructorPrivate.gif") );


/**
 * Represents a function declaration in the source code.
 * 
 * @param name      the function name
 * @param isPublic  is this function public
 * @param model     the tree model
 */
function FunctionTreeNode( name, isPublic, model ) extends swing.tree.DefaultMutableTreeNode()
{
  /**
   * Table of child functions, maps   name -> fxn
   */
  var fxnTable = new java.util.Hashtable();
  
  /**
   * Store a current token within the function, so we can determine what
   * offset to jump to to jump to function.
   */
  var token = null;
  
  public function setTreeModel(m) { model = m; }
  public function getName()  { return name; }
  public function toString() { return getName(); }
  
  /**
   * Get the offset of this function in the document.
   * 
   * @return <code>-1</code> if unknown, else offset
   */
  public function getOffset()
  {
    if( token != null )
      return token.beginOffset;
    return -1;
  }
  
  /**
   * Get the icon for this type of function.
   */
  public function getIcon()
  {
    var type;
    var protection;
    
    if( java.lang.Character.isUpperCase( name[0] ) )
      type = "constructor";
    else
      type = "method";
    
    if(isPublic)
      protection = "public";
    else
      protection = "private";
    
    return functionIconTable.get( protection + "-" + type );
  }
  
  /**
   * Parse this function body recursively using the specified visitor (as
   * opposed to sequential scanning thru the tokens).
   * 
   * @param visitor     the visitor to use
   * @param syntaxTree  the syntax-tree to visit
   */
  public function visitParse( visitor, syntaxTree, token )
  {
    this.token = token;
    
    var idx = 0;
    var oldTable = fxnTable;
    fxnTable = new java.util.Hashtable();
    
    function getUniqueName(name)
    {
      if( fxnTable.get(name) == null )
        return name;
      
      var idx = 2;
      var uniqueName;
      
      while( fxnTable.get( uniqueName = (name + " " + idx) ) != null )
        idx++;
      
      return uniqueName;
    }
    
    function handleChild( name, isPublic, syntaxTree, token )
    {
      name = getUniqueName(name);
      
      var fxn  = oldTable.get(name);
      
      if( fxn == null )
        addFunctionNode( fxn = new FunctionTreeNode( name, isPublic, model ), idx );
      else
        oldTable.remove(name);
      
      idx++;
      
      fxnTable.put( name, fxn );
      
      fxn.visitParse( visitor, syntaxTree, token );
    }
    
    /**
     * Grammar production:
     * <PRE>
     * f0 -> Permissions()
     * f1 -> "function"
     * f2 -> &lt;IDENTIFIER&gt;
     * f3 -> "("
     * f4 -> ( Arglist() )?
     * f5 -> ")"
     * f6 -> ( "extends" PrimaryExpressionWithTrailingFxnCallExpList() FunctionCallExpressionList() )?
     * f7 -> "{"
     * f8 -> Program(true)
     * f9 -> "}"
     * </PRE>
     */
    var oldFunctionDeclarationFxn = visitor.register( st.FunctionDeclaration, function( n, arg ) {
      
      var isPublic = false;
      
      for( var i=0; (i<n.f0.f0.size()) && !isPublic; i++ )
        if( n.f0.f0.elementAt(i).choice.kind == ParserConstants.PUBLIC )
          isPublic = true;
      
      handleChild( n.f2.otokenImage, isPublic, n.f8, n.f7 );
      
    } );
    
    /**
     * Grammar production:
     * <PRE>
     * f0 -> "function"
     * f1 -> "("
     * f2 -> ( Arglist() )?
     * f3 -> ")"
     * f4 -> ( "extends" PrimaryExpressionWithTrailingFxnCallExpList() FunctionCallExpressionList() )?
     * f5 -> "{"
     * f6 -> Program(true)
     * f7 -> "}"
     * </PRE>
     */
    var oldFunctionPrimaryPrefixFxn = visitor.register( st.FunctionPrimaryPrefix, function( n, arg ) {
      
      handleChild( "anon", true, n.f6, n.f5 );
      
    } );
    
    visitor.visit( syntaxTree, null );
    
    if( oldFunctionDeclarationFxn == null )
    {
      visitor.remove( st.FunctionDeclaration );
      visitor.remove( st.FunctionPrimaryPrefix );
    }
    else
    {
      visitor.register( st.FunctionDeclaration, oldFunctionDeclarationFxn );
      visitor.register( st.FunctionPrimaryPrefix, oldFunctionPrimaryPrefixFxn );
    }
    
    // any left in oldTable means they need to be removed:
    for( var e=oldTable.elements(); e.hasMoreElements(); )
      removeFunctionNode( e.nextElement() );
    
  }
  
  /**
   * Add child function node, firing appropriate events.
   */
  public function addFunctionNode( fxn, idx )
  {
    // XXX idx is ignored
    swing.SwingUtilities.invokeLater( function() {
      var cnt = model.getChildCount(this);
      for( var i=0; i<cnt; i++ )
        if( model.getChild( this, i ).getName() > fxn.getName() )
          return model.insertNodeInto( fxn, this, i );
      model.insertNodeInto( fxn, this, cnt );
    } );
  }
  
  /**
   * Remove child function node, firing appropriate events.
   */
  public function removeFunctionNode(fxn)
  {
    model.removeNodeFromParent(fxn);
    fxn.setParent(null);
  }
}

/* setup any object-script-mode specific keybinding here */

// keeping this external, bacause maybe I should split osMode out into multiple files...
osMode.addKeyFxn( "ctrl R", "rename-reference", "rename the reference at the cursor", '{
  
  var frame = EditorFrame.getCurrentFrame();
  var buf   = frame.getBuffer();
  var doc   = buf.getDocument();
  var off   = buf.getCaretPosition();
  var doff  = osMode.findDeclaration( off, false );
  
  if( doff == -1 )
    return;
  
  var roffs = [];
  var roff  = 0;
  
  while( (roff = osMode.findReference( roff, true )) != -1 )
    roffs[ roffs.length() ] = roff;
  
  if( roffs.length() == 0 )
    return;
  
  var editorKit = buf.getEditorKit();
  var oldImg = editorKit.getToken( roffs[0] ).otokenImage;
  var newImg = util.prompt("rename " + roffs.length() + " references of \"" + oldImg + "\" to");
  
  // go last to first so we don't worry about oldImg.length() != newImg.length()
  if( (null != newImg) && (newImg.length() > 0) )
  {
    for( var i=roffs.length()-1; i>=0; i-- )
    {
      var token = editorKit.getToken( roffs[i] );
      doc.remove( token.beginOffset, token.endOffset - token.beginOffset + 1 );
      doc.insertString( roffs[i], newImg, null );
    }
    frame.setStatusLineText("renamed " + roffs.length() + " references of \"" + oldImg + "\" to \"" + newImg + "\"");
  }
  
} );


/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
