/*=============================================================================
 *     Copyright Texas Instruments 2003. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

const var ENCODING_TABLE = [ 
  "A","B","C","D","E","F","G","H","I","J","K","L","M","N","O","P","Q","R","S","T","U","V","W","X","Y","Z",
  "a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z",
  "0","1","2","3","4","5","6","7","8","9","+","/"
];

/**
 * An attachment type for binary files.  This attachment type implements
 * base64 encoding as described in RFC-1521.
 * 
 * @param file  the file to attach
 */
public function BinaryFileAttachment(file)
  extends pkg.email.Attachment( file.getName(), "application/octet-stream; name=\"" + file.getName() + "\"", "base64" )
{
  /**
   * Get the attachment as a file.  This just returns a file representation
   * of the un-decoded attachment body, so attachment types which encode
   * their payload probably need to re-implement this.
   * 
   * @return a file
   */
  public function getFile()
  {
    return file;
  }
  
  /**
   * Get the encoded contents of the attachment.  This implements base64 
   * encoding as described in RFC-1521.
   * 
   * @return a string
   */
  public function getEncodedBody()
  {
try {
    var is  = file.getInputStream();
    var idx = 0;
    var str = "";
    var line = "";
    
    var read = is.read;
    
    while(true)
    {
      var b1 = read();
      var b2 = read();
      var b3 = read();
      
      if( (idx++ % 19) == 0 )
      {
        line += "\n";
        line[0];  // hack to flatten line
        str += line;
        line = "";
      }
      
      // case 1:  the final quantum of encoding is an integral multiple of 24 bits:
      if( b1 == -1 )
      {
        break;
      }
      
      var b = (b1 << 16) | (((b2 == -1) ? 0 : b2) << 8) | ((b3 == -1) ? 0 : b3);
      
      line += ENCODING_TABLE[(b >> 18) & 0x3f];
      line += ENCODING_TABLE[(b >> 12) & 0x3f];
      
      // case 2:  the final quantum of encoding is exactl 8 bits:
      if( b2 == -1 )
      {
        line += "=";
        line += "=";
        break;
      }
      
      line += ENCODING_TABLE[(b >> 6) & 0x3f];
      
      // case 3:  the final quantum of encoding input is exactly 16 bits:
      if( b3 == -1 )
      {
        line += "=";
        break;
      }
      
      line += ENCODING_TABLE[b & 0x3f];
    }
    
    line[0];  // hack to flatten line
    str += line;
    
    return str;
} catch(e) {
  pkg.output.writeln("e=" + e);
}
  }
}




/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

