/*=============================================================================
 *     Copyright Rob Clark 2000. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * An instance of this class represents a message to be sent.
 */
public function EmailMessage()
{
  // the headers for the message:
  var headerList = new java.util.LinkedList();
  
  // the recipients of the message:
  var recipientList = new java.util.LinkedList();
  
  // the list of attachments:
  var attachmentList = new java.util.LinkedList();
  
  // the body of the message:
  var body = "";
  
  // the sender of the message:
  var sender = "\"Beelzebub\" <satan@ti.com>";
  
  // the boundary string:
  var boundaryString = "Chimera-Mail-" + java.lang.Math.abs( java.lang.Math.random() * java.lang.Math.pow(2,32) ).castToExactNumber();
  
  /**
   * Add a new header to the message.
   * 
   * @param name         the name of the header, a string
   * @param val          the value of the header, a string
   */
  public function addHeader( name, val )
  {
    headerList.add( new Header(name,val) );
  }
  
  /**
   * Add a recipient of this message.  A message can have one or more
   * recipients.
   * 
   * @param recipient    the email address to send this message to
   */
  public function addRecipient(recipient)
  {
    recipientList.add(recipient);
  }
  
  /**
   * Add an attachment to this message.
   * 
   * @param attachment   the attachment to add to the message
   */
  public function addAttachment(attachment)
  {
    attachmentList.add(attachment);
  }
  
  /**
   * Get the recipients of this message.
   * 
   * @return a Iterator
   */
  public function recipients()
  {
    return recipientList.iterator();
  }
  
  /**
   * Add the subject header to this message.
   * 
   * @param subject      the subject of the message
   */
  public function setSubject(subject)
  {
    addHeader( "Subject", subject );
  }
  
  /**
   * Set the body of this message.
   * 
   * @param body         the body of the message
   */
  public function setBody(body)
  {
    this.body = body;
  }
  
  /**
   * Set the sender of this message.
   * 
   * @param sender       the email address of the sender (a string)
   */
  public function setSender(sender)
  {
    this.sender = sender;
  }
  
  /**
   * Get the sender of this message
   * 
   * @return the email address of the sender (ie. a string)
   */
  public function getSender()
  {
    return sender;
  }
  
  /**
   * Send this message to the specified output stream.  This is called by
   * the Emailer to send the body of the message.  The "To" and "From"
   * headers are handled specially by Emailer.
   * 
   * @param out          the output stream to write the msg to
   */
  public function send(out)
  {
    for( var itr=headerList.iterator(); itr.hasNext(); )
      out.write( "" + itr.next() + "\n" );
    
    out.write("Content-Type: multipart/mixed; boundary=" + boundaryString + "\n");
    out.write("--" + boundaryString + "\n");
    out.write("Content-Type: text/plain\n");
    out.write("\n" + body + "\n\n");
    
    for( var itr=attachmentList.iterator(); itr.hasNext(); )
      out.write("--" + boundaryString + "\n" + itr.next() + "\n");
    
    out.write("--" + boundaryString + "--\n.\n\n\n");
  }
  
  /**
   */
  function Header( name, val )
  {
    /**
     * Convert the header to a string
     * 
     * @return a string
     */
    public function castToString()
    {
      return name + ": " + val;
    }
  }
}




/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

