/*=============================================================================
 *     Copyright Rob Clark 2000. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * An emailer can be used to send messages.
 * 
 * @param hostname     the name of the SMTP server to send msgs to
 * @param port         the port number for the SMTP server, probably 25
 * @see EmailMessage
 */
public function Emailer( hostname, port )
{
  const var localhost = java.net.InetAddress.getLocalHost().getHostName();
  
  /**
   * Send a message.
   * 
   * @param msg          a subclass of <code>EmailMessage</code>
   */
  public function sendMessage(msg)
  {
    var sender = msg.getSender();
    
    /* make connection to SMTP server synchronously, instead of
     * from the worker thread, in order that if we can't connect
     * the caller sees the exception that gets thrown... other-
     * wise the caller won't get a chance to handle the exception
     * 
     * XXX really this should be handled by whatever callgate I
     * come up with to do long running jobs safely from the
     * context of the AWT event thread... which should deal with
     * exceptions that get thrown by rethrowing them from this
     * thread context
     */
    var s   = new java.net.Socket( hostname, port );
    var out = new java.io.OutputStreamWriter( s.getOutputStream() );
    var in  = new java.io.InputStreamReader( s.getInputStream() );
    
    (new java.lang.Thread( function() {
      
      out.write( "HELO " + localhost + "\n" );
      
      for( var itr=msg.recipients(); itr.hasNext(); )
      {
        var recipient = itr.next();
        
        out.write( "MAIL FROM: " + sender + "\n" );
        out.write( "RCPT TO: " + recipient + "\n" );
        
        out.write( "DATA\n" );
        
        out.write( "To: " + recipient + "\n" );
        out.write( "From: " + sender + "\n" );
        
        msg.send(out);
      }
      
      out.flush(); // XXX shouldn't be needed, but seems to make solaris happy
      
      // XXX debug: XXX
      var inout = new java.io.FileWriter("mailrsp");
      var c;
      while( (c = in.read()) != -1 )
        inout.write(c);
      inout.flush();
      inout.close();
      in.close();
      // XXXXXXXXXXXXXX
      
      out.close();
      s.close();
      
    } )).start();
  }
}




/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

