/******************************************************************************
 *        Copyright Texas Instruments 2003. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



if( registry.exists("/MenuBar/File/New/Script Application") )
  registry.unlink("/MenuBar/File/New/Script Application");

services["window manager"].addMenuBarItem( 
  "File/New",
  new pkg.util.ActionAdapter(
    "Script Application", 
    null, 
    null, 
    function(evt) {
      
      var chooser = new ti.chimera.FileChooser( windowManager.getDialog("Create Jar File") );
      chooser.setFileFilter( new pkg.util.ExtensionFileFilter( "jar", "Java ARchive File" ) );
      
      if( chooser.showOpenDialog(null) == ti.chimera.FileChooser.APPROVE_OPTION )
      {
        var file = chooser.getSelectedFile();
        var vpath = "/jar/" + file.getName();
        
        // strip trailing .jar off of vpath:
        vpath = vpath.substring( 0, vpath.length() - ".jar".length() );
        
        if( file.length() == 0 )
          file.delete();  // an empty, but existing file will confuse java.io.ZipFile...
        
        // mount the filesystem
        var fs;
        oscript.fs.AbstractFileSystem.mount( fs = new oscript.fs.JarFileSystem( file, true ), vpath );
        
        // create manifest:
        {
          (new pkg.fs.File(vpath + "/META-INF")).mkdir();
          
          var file = new pkg.fs.File(vpath + "/META-INF/MANIFEST.MF");
          file.createNewFile();
          
          var out = new java.io.PrintWriter( file.getOutputStream(false) );
          
          out.println("Manifest-Version: 1.0");
          out.println("Created-By: " + oscript.OscriptInterpreter.getVersionString());
          out.println("Class-Path: oscript.jar chimera.jar");
          out.println("Main-Class: oscript.MainRunner");
          
          out.flush();
          out.close();
        }
        
        // create main.os:
        {
          var file = new pkg.fs.File(vpath + "/main.os");
          file.createNewFile();
          
          var out = new java.io.PrintWriter( file.getOutputStream(false) );
          
          out.println("");
          out.println("// this is the entry point to the program, the global");
          out.println("// \"args\" contains any command-line arguments");
          out.println("");
          
          out.flush();
          out.close();
          
          fs.flush();
          
          services["editor"].openFile( file.getPath() );
        }
      }
      
    }
  )
);

