/*=============================================================================
 *     Copyright Texas Instruments 2003. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * A tool for generating and displaying documentation from .os files.
 */
public var os2html = new (function() {
const var st    = oscript.syntaxtree;
const var Font  = java.awt.Font;
const var Color = java.awt.Color;
const var ParserConstants = oscript.parser.OscriptParserConstants;

  
  private var visitor = new oscript.visitor.ExtensibleVisitor();
  
  /* 
   * Setup Attributes
   * 
   * XXX need some way to share this with editor.osMode...
   */
  
  private var attrSetTable = new java.util.Hashtable();
  
  private function makeAttributeSet( name, color, style )
  {
    function hex(val)
    {
      var str = java.lang.Integer.toHexString( val & 0xff );
      if( str.length() < 2 )
        str = "0" + str;
      return str;
    }
    
    var openAttr  = "";
    var closeAttr = "";
    
    function ao(str) { openAttr += str; }
    function ac(str) { closeAttr = str + closeAttr; }
    
    if( style != null )
    {
      if( (style & Font.ITALIC) != 0 )
      {
        ao("<i>");
        ac("</i>");
      }
      
      if( (style & Font.BOLD) != 0 )
      {
        ao("<b>");
        ac("</b>");
      }
    }
    
    ao("<font color=\"#" + hex(color.getRed()) + hex(color.getGreen()) + hex(color.getBlue()) + "\">");
    ac("</font>");
    
    return new function( public pre, public post ) {} ( openAttr, closeAttr );
  }
  
  private var COMMENT_ATTR = makeAttributeSet( "Comment", Color.green.darker(), Font.ITALIC );
  attrSetTable.put( ParserConstants.SINGLE_LINE_COMMENT, COMMENT_ATTR );
  attrSetTable.put( ParserConstants.FORMAL_COMMENT, COMMENT_ATTR );
  attrSetTable.put( ParserConstants.MULTI_LINE_COMMENT, COMMENT_ATTR );

  private var JAVADOC_TAG_ATTR = makeAttributeSet( "Javadoc Tag", Color.red.darker(), Font.ITALIC );
  attrSetTable.put( ParserConstants.JAVADOC_TAG, JAVADOC_TAG_ATTR );
  attrSetTable.put( ParserConstants.JAVADOC_TAG_WITH_ARG, JAVADOC_TAG_ATTR );

  private var JAVADOC_TAG_ARG_ATTR = makeAttributeSet( "Javadoc Tag Argument", Color.blue.darker(), Font.ITALIC );
  attrSetTable.put( ParserConstants.JAVADOC_TAG_ARG, JAVADOC_TAG_ARG_ATTR );
  
  private var FLOW_CONTROL_ATTR = makeAttributeSet( "Flow Control Keyword", Color.red, null );
  attrSetTable.put( ParserConstants.BREAK, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.CATCH, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.CONTINUE, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.ELSE, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.FINALLY, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.FOR, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.IF, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.RETURN, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.SYNCHRONIZED, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.THROW, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.TRY, FLOW_CONTROL_ATTR );
  attrSetTable.put( ParserConstants.WHILE, FLOW_CONTROL_ATTR );
  
  private var KEY_WORD_ATTR = makeAttributeSet( "Keyword", Color.blue, Font.BOLD );
  attrSetTable.put( ParserConstants.EVAL, KEY_WORD_ATTR );
  attrSetTable.put( ParserConstants.FUNCTION, KEY_WORD_ATTR );
  attrSetTable.put( ParserConstants.IMPORT, KEY_WORD_ATTR );
  attrSetTable.put( ParserConstants.MIXIN, KEY_WORD_ATTR );
  attrSetTable.put( ParserConstants.VAR, KEY_WORD_ATTR );
  attrSetTable.put( ParserConstants.NEW, KEY_WORD_ATTR );
  attrSetTable.put( ParserConstants.EXTENDS, KEY_WORD_ATTR );
  
  private var KEY_VAL_ATTR = makeAttributeSet( "Special Value", Color.orange, Font.BOLD );
  attrSetTable.put( ParserConstants.FALSE, KEY_VAL_ATTR );
  attrSetTable.put( ParserConstants.NULL, KEY_VAL_ATTR );
  attrSetTable.put( ParserConstants.UNDEFINED, KEY_VAL_ATTR );
  attrSetTable.put( ParserConstants.THIS, KEY_VAL_ATTR );
  attrSetTable.put( ParserConstants.CALLEE, KEY_VAL_ATTR );
  attrSetTable.put( ParserConstants.TRUE, KEY_VAL_ATTR );
  
  private var KEY_MODIFIER_ATTR = makeAttributeSet( "Modifier", new Color( 0x81, 0x2b, 0xe2 ), Font.ITALIC );
  attrSetTable.put( ParserConstants.STATIC, KEY_MODIFIER_ATTR );
  attrSetTable.put( ParserConstants.CONST, KEY_MODIFIER_ATTR );
  attrSetTable.put( ParserConstants.PRIVATE, KEY_MODIFIER_ATTR );
  attrSetTable.put( ParserConstants.PROTECTED, KEY_MODIFIER_ATTR );
  attrSetTable.put( ParserConstants.PUBLIC, KEY_MODIFIER_ATTR );
  
  private var LITERAL_ATTR = makeAttributeSet( "Literal", Color.orange.darker(), null );
  attrSetTable.put( ParserConstants.INTEGER_LITERAL, LITERAL_ATTR );
  attrSetTable.put( ParserConstants.DECIMAL_LITERAL, LITERAL_ATTR );
  attrSetTable.put( ParserConstants.HEX_LITERAL, LITERAL_ATTR );
  attrSetTable.put( ParserConstants.OCTAL_LITERAL, LITERAL_ATTR );
  attrSetTable.put( ParserConstants.FLOATING_POINT_LITERAL, LITERAL_ATTR );
  attrSetTable.put( ParserConstants.EXPONENT, LITERAL_ATTR );
  attrSetTable.put( ParserConstants.ESCAPE_SEQUENCE, LITERAL_ATTR );
  attrSetTable.put( ParserConstants.STRING_LITERAL, LITERAL_ATTR );
  attrSetTable.put( ParserConstants.REGEXP_LITERAL, LITERAL_ATTR );
  
  private var PAREN_ATTR = makeAttributeSet( "Paren", Color.green.darker(), Font.BOLD );
  attrSetTable.put( ParserConstants.LPAREN, PAREN_ATTR );
  attrSetTable.put( ParserConstants.RPAREN, PAREN_ATTR );
  attrSetTable.put( ParserConstants.LBRACE, PAREN_ATTR );
  attrSetTable.put( ParserConstants.RBRACE, PAREN_ATTR );
  attrSetTable.put( ParserConstants.LBRACKET, PAREN_ATTR );
  attrSetTable.put( ParserConstants.RBRACKET, PAREN_ATTR );
  
  private var OPERATOR_ATTR = makeAttributeSet( "Operator", Color.blue.darker().darker(), Font.BOLD );
  attrSetTable.put( ParserConstants.INSTANCEOF, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.SEMICOLON, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.COMMA, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.DOT, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.ASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.GT, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.LT, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.BANG, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.TILDE, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.HOOK, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.COLON, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.EQ, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.LE, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.GE, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.NE, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.SC_OR, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.SC_AND, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.INCR, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.DECR, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.PLUS, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.MINUS, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.STAR, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.SLASH, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.BIT_AND, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.BIT_OR, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.XOR, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.REM, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.LSHIFT, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.RSIGNEDSHIFT, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.RUNSIGNEDSHIFT, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.PLUSASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.MINUSASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.STARASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.SLASHASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.ANDASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.ORASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.XORASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.REMASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.LSHIFTASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.RSIGNEDSHIFTASSIGN, OPERATOR_ATTR );
  attrSetTable.put( ParserConstants.RUNSIGNEDSHIFTASSIGN, OPERATOR_ATTR );
  
  /* 
   * Register visitor functions
   */
  
  /**
   * We only have to implement handling NodeToken, to call the
   * ctx's printToken() method, which is where all the work gets
   * done.
   */
  visitor.register( st.NodeToken, function( n, ctx ) {
    
    if( n.numSpecials() > 0 )
      for( var e=n.specialTokens.elements(); e.hasMoreElements(); )
        visitor.visit( e.nextElement(), ctx );
    
    ctx.printToken(n);
    
  } );
  
  /**
   * The visit context, where the HTML representation of an ObjectScript
   * document is accumulated.
   */
  private function Context()
  {
    private var str  = "";
    private var line = 1;
    private var col  = 1;
    
    /**
     * Print the specified token, at the correct line and column.
     * 
     * @param token   the token to print
     */
    public function printToken(token)
    {
      if( (token.beginLine < line) ||
          ( (token.beginLine == line) &&
            (token.beginColumn < col) ) )
        throw new Exception("error at: " + token);
      
      while( line < token.beginLine )
        print("\n");
      
      // XXX hack, to prevent StackOverflowError... should have some threshold
      // to do this within OString so compound strings made of many small pieces
      // don't cause a stack overflow:
      if( col == 1 )
        str = str.castToString();
      
      while( col < token.beginColumn )
        print(" ");
      
      var attr = attrSetTable.get( token.kind );
      
      if( attr != null ) str += attr.pre;  // doesn't change line/col, so don't use print()
      print( token.tokenImage );
      if( attr != null ) str += attr.post; // doesn't change line/col, so don't use print()
    }
    
    private function print(img)
    {
      for( var i=0; i<img.length(); i++ )
      {
        var c = img[i];
        ++col;
        if( c == "\n" )
        {
          ++line;
          col = 1;
          str += "\n";
        }
        else if( c == "<" )
        {
          str += "&lt;";
        }
        else if( c == ">" )
        {
          str += "&gt;";
        }
        else
        {
          str += c;
        }
      }
    }
    
    /**
     * Return the string representation of the context, which is
     * the accumulated HTML document body contents.
     */
    public function castToString()
    {
      return str;
    }
  }
  
  /**
   * Implements the parsing of a script file to a string that can be
   * part of an HTML document's body.
   * 
   * @param file       the file to parse 
   * @return the string that is a portion of the HTML document for
   *    the specified file
   */
  public function parseToBodyPart(file)
  {
    file = pkg.cl.resolveFile(file);
    
    if( !file.exists() )
      throw new Exception("file doesn't exist: " + file);
    
    var node = oscript.OscriptInterpreter.parse(file);
    
    var ctx = new Context();
    visitor.visit( node, ctx );
    
    return ctx.castToString();
  }
  
  /**
   * Return an HTML document with the documentation generated from the
   * specified <code>file</code>.
   * 
   * @param file       the file to parse 
   * @return a string contaning the HTML document
   */
  public function parseToDocument(file)
  {
    return "<html><body><pre>" + parseToBodyPart(file) + "</pre></body></html>";
  }
  
  /**
   * Parse a .os file and write the generated HTML document to file
   * 
   * @param srcFile   the file to parse
   * @param outFile   the file to write output to
   */
  public function parseToFile( srcFile, outFile )
  {
    outFile = pkg.cl.resolveFile(outFile);
    
    if( !outFile.exists() )
      outFile.createNewFile();
    
    var out = outFile.getOutputStream(false);
    var ps = new java.io.PrintStream(out);
    ps.println( parseToDocument(srcFile) );
    ps.flush();
    ps.close();
    out.flush();
    out.close();
  }
  
})();


/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */
