/******************************************************************************
 *     Copyright Texas Instruments 2004. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

mixin ti.chimera.registry;


/**
 * Register a "placeholder" plugin, which will enable deferred loading of
 * the actual plugin until the first time a service provided by the plugin
 * is accessed.
 * 
 * @param pathOrFxn    the script file to import, or a <code>function()</code>
 *    to call which performs the import and possibly other actions.
 * @param serviceNames the array of names of services provided by this
 *    plugin.  It is an error if this array does not match the set of
 *    services actually provided by this plugin
 */
public function deferredRegister( pathOrFxn, serviceNames )
{
  registry.register( new function() extends ti.chimera.Plugin( main, serviceNames[0] ) {
    
    for( var serviceName : serviceNames )
    {
      registerServiceFactory( new function() extends ServiceFactory() {
        
        public function getName()
        {
          return serviceName;
        }
        
        public function createService()
        {
          main.debug( 1, "loading: " + serviceName );
          
          if( pathOrFxn instanceof String )
            pkg.plugin.register(pathOrFxn);
          else
            pathOrFxn();
          
          return services[serviceName];
        }
        
      }() );
    }
    
  }() );
}

