/******************************************************************************
 *     Copyright Texas Instruments, Inc., 2002.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Install the standard node-view factories for the registry browser.  These
 * are split out into a separate file like this just for logistical reasons...
 */
public function installStandardNodeViewFactories()
{
  main.log("installing standard node view factories");
  
  /**
   * Get a panel that subscribes the specified <code>subscriber</code> to
   * the specified <code>node</code>
   * 
   * @param node         the node to subscribe to
   * @param subscriber   the subscriber to subscribe
   */
  function getSubscriberPanel( node, subscriber )
  {
    node.subscribe(subscriber);
    
    var panel = new (function() extends swing.JPanel() {
      var super_removeNotify = removeNotify;
      public function removeNotify() { node.unsubscribe(subscriber); super_removeNotify(); }
    })();
    
    return panel;
  }
  
  /**
   * Register node-view factory which can handle basic TypeNodeContract types,
   * such as String, Boolean, Number...
   */
  services["registry browser"].
    registerNodeViewFactory( 
      ti.chimera.registry.TypeNodeContract,
      function(node) {
        
        var type = node.getNodeContract().getType();
        var component = null;
        var newValue;
        
        function makeAcceptPanel( textField, update )
        {
        /* XXX add Ok button, because action listener isn't invoked 
         * when users switches to different node... maybe there is
         * a better long term solution? 
         */
          var panel = new swing.JPanel();
          panel.add(textField);
          
          var acceptButton = new swing.JButton("Accept");
          acceptButton.addActionListener( function(evt) {
            acceptButton.setEnabled(false);
            update(evt);
          } );
          panel.add(acceptButton);
          
          textField.getDocument().addDocumentListener(
            new function() extends swing.event.DocumentListener() {
              
              public function changedUpdate(evt)
              {
                acceptButton.setEnabled(true);
              }
              
              public var removeUpdate = changedUpdate;
              public var insertUpdate = changedUpdate;
              
            }()
          );
          
          acceptButton.setEnabled(false);
          
          return panel;
        }
        
        if( (type == java.lang.String)  || (type == java.lang.Number) )
        {
          var textField = new swing.JTextField(20);
          
          if( type == java.lang.String )
            newValue = function(val) { textField.setText(val); };
          else
            newValue = function(val) { textField.setText( val.castToString() ); };
          
          function update(evt)
          {
            var val = textField.getText();
            
            if( type == java.lang.Number )
            {
              try
              {
                val = val.castToExactNumber();
              }
              catch(e)
              {
                services["prompt"].showErrorMessage("Must be a number");
                newValue( node.getValue() );
              }
            }
            
            setValue(val);
          }
          
          textField.addActionListener(update);
          
          /* XXX add Accept button, because action listener isn't invoked 
           * when users switches to different node... maybe there is
           * a better long term solution? 
           */
          var panel = new swing.JPanel();
          panel.add(textField);
          
          var acceptButton = new swing.JButton("Accept");
          acceptButton.addActionListener( function(evt) {
            acceptButton.setEnabled(false);
            update(evt);
          } );
          panel.add(acceptButton);
          
          textField.getDocument().addDocumentListener(
            new function() extends swing.event.DocumentListener() {
              
              public function changedUpdate(evt)
              {
                acceptButton.setEnabled(
                  textField.getText() != node.getValue()
                );
              }
              
              public var removeUpdate = changedUpdate;
              public var insertUpdate = changedUpdate;
              
            }()
          );
          
          acceptButton.setEnabled(false);
          
          component = panel;
        }
        else if( type == java.lang.Boolean )
        {
          var comboBox = new swing.JComboBox( [true, false] );
          newValue = comboBox.setSelectedItem;
  
          comboBox.addActionListener( function(evt) {
            setValue( comboBox.getSelectedItem() );
          } );
          
          component = comboBox;
        }
        else if( type == java.awt.Rectangle )
        {
          var panel = new swing.JPanel( new java.awt.GridLayout(0,2,1,1) );
          
          var xtf = new swing.JTextField(10);
          var ytf = new swing.JTextField(10);
          var wtf = new swing.JTextField(10);
          var htf = new swing.JTextField(10);
          
          panel.add( new swing.JLabel("x") ); panel.add(xtf);
          panel.add( new swing.JLabel("y") ); panel.add(ytf);
          panel.add( new swing.JLabel("w") ); panel.add(wtf);
          panel.add( new swing.JLabel("h") ); panel.add(htf);
          
          newValue = function(val) {
            xtf.setText( val.x.castToString() );
            ytf.setText( val.y.castToString() );
            wtf.setText( val.width.castToString() );
            htf.setText( val.height.castToString() );
          };
          
          function update(evt)
          {
            try
            {
              setValue( new java.awt.Rectangle( xtf.getText().castToExactNumber(),
                                                ytf.getText().castToExactNumber(),
                                                wtf.getText().castToExactNumber(),
                                                htf.getText().castToExactNumber() ) );
            }
            catch(e)
            {
              services["prompt"].showErrorMessage("Must be a number");
              newValue( node.getValue() );
            }
          }
          
          xtf.addActionListener(update);
          ytf.addActionListener(update);
          wtf.addActionListener(update);
          htf.addActionListener(update);
          
          component = panel;
        }
        else
        {
          return null;
        }
        
        function setValue(val)
        {
          if( val == node.getValue() )
            return;
          
          try
            node.setValue(val);
          catch(e)
            services["prompt"].showErrorMessage( e.getMessage() );
        }
        
        var panel = getSubscriberPanel( node, function( node, value ) {
          newValue(value);
        } );
        panel.add(component);
        
        return panel;
        
      } );
  
  /**
   * Register node-view-factory which handles DirectoryTable
   */
  services["registry browser"].
    registerNodeViewFactory( 
      ti.chimera.registry.TypeNodeContract,
      function(node) {
        
        if( node.getNodeContract().getType() == ti.chimera.registry.DirectoryTable )
        {
          var panel = new swing.JPanel();
          panel.add( new swing.JLabel("<html><i>This page intentionally left blank</i></html>") );
          return panel;
        }
        
        return null;
        
      } );
  
  /**
   * Register node-view factory which can handle FontNodeContract 
   */
  services["registry browser"].
    registerNodeViewFactory( 
      ti.chimera.pref.FontNodeContract,
      (function() {
        
        var sizes = [ 8, 9, 10, 11, 12, 13, 14, 16, 18, 20, 24, 26, 28, 32, 36, 40, 48, 56, 64, 72 ];
        var fontNames = java.awt.GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
        
        return function(node) {
          
          var subscriber;
          
          // figure out which font names are ok:  (XXX maybe should try an cache this!)
          var acceptibleFontNames = [];
          
          {
            var nc = node.getNodeContract();
            var style = 0;
            
            if( (nc.getStyle() & ti.chimera.pref.FontNodeContract.BOLD) != 0 )
              style |= java.awt.Font.BOLD;
            
            if( (nc.getStyle() & ti.chimera.pref.FontNodeContract.ITALIC) != 0 )
              style |= java.awt.Font.ITALIC;
            
            for( var i=0; i<fontNames.length(); i++ )
              if( nc.accepts( new java.awt.Font( fontNames[i], style, 10 ) ) )
                acceptibleFontNames[ acceptibleFontNames.length() ] = fontNames[i];
          }
          
          var box = new (function() extends swing.Box( swing.BoxLayout.Y_AXIS ) {
            var super_removeNotify = removeNotify;
            public function removeNotify() { node.unsubscribe(subscriber); super_removeNotify(); }
          })();
          
          function update(evt)
          {
            var name  = nameList.getSelectedValue();
            var size  = sizeList.getSelectedValue();
            var style = 0;
            
            // this could happen if we haven't had the values published to us:
            if( (name == null) || (size == null) )
              return;
            
            if( boldCheckBox.isSelected() )
              style |= java.awt.Font.BOLD;
            
            if( italicCheckBox.isSelected() )
              style |= java.awt.Font.ITALIC;
            
            var font = new java.awt.Font( name, style, size );
            
            if( font != node.getValue() )
              node.setValue(font);
          }
          
          var nameAndSizePanel = new swing.JPanel();
          var nameList = new swing.JList(acceptibleFontNames);
          nameList.setSelectionMode( swing.ListSelectionModel.SINGLE_SELECTION );
          nameAndSizePanel.add( new swing.JScrollPane(nameList) );
          var sizeList = new swing.JList(sizes);
          sizeList.setSelectionMode( swing.ListSelectionModel.SINGLE_SELECTION );
          nameAndSizePanel.add( new swing.JScrollPane(sizeList) );
          box.add(nameAndSizePanel);
          
          var stylePanel = new swing.JPanel( new java.awt.GridLayout(1,0) );
          var boldCheckBox = new swing.JCheckBox("Bold");
          stylePanel.add(boldCheckBox);
          var italicCheckBox = new swing.JCheckBox("Italic");
          stylePanel.add(italicCheckBox);
          box.add(stylePanel);
          
          var sampleTextArea = new swing.JTextArea("Mmmm, Chimera");
          box.add(sampleTextArea);
          
          node.subscribe( subscriber = function( node, font ) {
            sampleTextArea.setFont(font);
            nameList.setSelectedValue( font.getName(), true );
            sizeList.setSelectedValue( font.getSize(), true );
            boldCheckBox.setSelected( font.isBold() );
            italicCheckBox.setSelected( font.isItalic() );
          } );
          
          if( (node.getNodeContract().getStyle() & ti.chimera.pref.FontNodeContract.BOLD) != 0 )
          {
            boldCheckBox.setSelected(true);
            boldCheckBox.setEnabled(false);
          }
          else if( (node.getNodeContract().getStyle() & ti.chimera.pref.FontNodeContract.NOT_BOLD) != 0 )
          {
            boldCheckBox.setSelected(false);
            boldCheckBox.setEnabled(false);
          }
          
          if( (node.getNodeContract().getStyle() & ti.chimera.pref.FontNodeContract.ITALIC) != 0 )
          {
            italicCheckBox.setSelected(true);
            italicCheckBox.setEnabled(false);
          }
          else if( (node.getNodeContract().getStyle() & ti.chimera.pref.FontNodeContract.NOT_ITALIC) != 0 )
          {
            italicCheckBox.setSelected(false);
            italicCheckBox.setEnabled(false);
          }
          
          nameList.addListSelectionListener(update);
          sizeList.addListSelectionListener(update);
          boldCheckBox.addActionListener(update);
          italicCheckBox.addActionListener(update);
          
          return box;
          
        };
      
      })() );
  
  /**
   * Register node-view factory which can handle ChoiceNodeContract 
   */
  services["registry browser"].
    registerNodeViewFactory( 
      ti.chimera.pref.ChoiceNodeContract,
      function(node) {
        
        var nc = node.getNodeContract();
        var choices = [];
        
        for( var i=0; i<nc.getChoicesCount(); i++ )
          choices[i] = nc.getDisplayName(i);
        
        var list = new swing.JList(choices);
        list.setSelectionMode( swing.ListSelectionModel.SINGLE_SELECTION );
        
        list.addListSelectionListener( function(evt) {
          
          var idx = list.getSelectedIndex();
          
          if( (idx != -1) && (nc.getChoice(idx) != node.getValue()) )
            node.setValue( nc.getChoice(idx) );
          
        } );
        
        var panel = getSubscriberPanel( node, function( node, choice ) {
          
          for( var i=0; i<nc.getChoicesCount(); i++ )
            if( nc.getChoice(i) == choice )
              return list.setSelectedIndex(i);
          
        } );
        panel.add(list);
        
        return panel;
        
      } );
  
  /**
   * Register node-view factory which can handle ColorNodeContract 
   */
  services["registry browser"].
    registerNodeViewFactory( 
      ti.chimera.pref.ColorNodeContract,
      function(node) {
        
        var chooser = new swing.JColorChooser();
        chooser.getSelectionModel().addChangeListener( function(evt) {
          
          var c = chooser.getColor();
          if( c != node.getValue() )
            node.setValue( chooser.getColor() );
          
        } );
        
        var panel = getSubscriberPanel( node, function( node, color ) {
          chooser.setColor(color);
        } );
        panel.add(chooser);
        
        return panel;
        
      } );
  
}



/**
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

