
/**
 * Run a test, calling proxy.adder.plus and proxy.multiplier.times.
 * Can be called with a regular object, as a control group, or with
 * an XmlRpcClient to test remote invocation.
 * 
 * @param proxy  
 */
function runtest(proxy)
{
  function add( a, b )
  {
    writeln("add( " + a + ", " + b + " ) -> " + proxy.adder.plus( a, b ));
  }
  
  function multiply( a, b )
  {
    writeln("multiply( " + a + ", " + b + " ) -> " + proxy.multiplier.times( a, b ));
  }
  
  function echo(val)
  {
    writeln("**********************");
    writeln("going to echo: " + val);
    var result = proxy.echo.echo(val);
    writeln("result: " + result);
  }
  
  add( 2, 3 );
  add( 2.0, 3.0 );
  add( "2", "3" );
  multiply( 2, 3 );
  multiply( 2.0, 3.0 );
  echo(2);
  echo(true);
  echo("foo");
  echo(2.0);
  echo( [ 2, true, "foo", 2.0, [] ] );
  
  var struct = new function( public a, public b ) {}( "a", true );
  writeln("before:  a=" + struct.a + ", b=" + struct.b);
  struct = proxy.echo.structTest(struct);
  writeln("after:   a=" + struct.a + ", b=" + struct.b);
  
}

{
  /**
   * The dummy server, that can be called either directly, or via rpc
   */
  public var test = new function() {
    
    public var adder = new function() {
      
      public function plus( a, b )
      {
        return a + b;
      }
      
    }();
    
    public var multiplier = new function() {
      
      public function times( a, b )
      {
        return a * b;
      }
      
    }();
    
    public var echo = new function() {
      
      public function echo(val)
      {
        writeln("val=" + val);
        return val;
      }
      
      public function structTest(struct)
      {
        writeln("during:  a=" + struct.a + ", b=" + struct.b);
        return struct;
      }
      
    }();
    
  }();
  
  
  writeln("calling directly");
  runtest(test);
  
  writeln("calling via XML-RPC");
  
  var server = new org.apache.xmlrpc.WebServer(24680);
  server.start();
  
  writeln("adding handlers");
  server.addHandler( "adder", new XmlRpcHandler(test.adder) );
  server.addHandler( "multiplier", new XmlRpcHandler(test.multiplier) );
  server.addHandler( "echo", new XmlRpcHandler(test.echo) );
  writeln("done");
  
  runtest( new XmlRpcClient("http://localhost:24680/") );
  
  server.shutdown();
}

