/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2001-2002.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * The <code>ActionAdapter</code> is a utility class for creating an
 * <code>javax.swing.Action</code> from a function, which should take
 * a single argument, the event that occurred.
 * 
 * @param name         the name of the action
 * @param tip          a tool-tip string, or <code>null</code>
 * @param icon         the icon, or <code>null</code>
 * @param fxn          the function to call when the action happens
 */
public function ActionAdapter( name, tip, icon, fxn ) 
  extends swing.AbstractAction(name)
{
  if( tip != null )
    putValue( swing.Action.SHORT_DESCRIPTION, tip );
  
  if( icon != null )
    putValue( swing.Action.SMALL_ICON, icon );
  
  public function actionPerformed(evt)
  {
    try
      fxn(evt);
    catch(e)
      services["talkback"].uncaughtException( java.lang.Thread.currentThread(), e );
  }
}

/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

