/******************************************************************************
 *     Copyright Texas Instruments 2004. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 */
public function ConsoleTabCompleter(console)
  extends ti.swing.console.ConsoleTabCompleter()
{
  /**
   * This should be overriden to return a <code>pkg.util.CompletionSet</code>
   */
  protected function getCompletionSet(buf)
  {
    return null;
  }
  
  private var lastBuf  = null;
  private var lastTime = 0;
  private var popupVisible = false;
  
  public function complete(buf)
  {
    var tmp = getCompletionSet(buf);
    if( tmp != null )
    {
      var completionSet = tmp[0];
      var match = tmp[1];
      var completions = [];
      for( var item : completionSet.getCompletions(match) )
        completions.push(item);
      var longestCompletion = completionSet.getCommonCompletion(match);
      
      if( (completions.length() > 0) && 
          (lastBuf == buf) && 
          ((System.currentTimeMillis()-500) < lastTime) )
      {
        var eventQueue = new ti.swing.SwingQueue();
        var popup = new swing.JPopupMenu();
        
        for( var completion : completions )
        {
          popup.add( new pkg.util.ActionAdapter( completion, null, null, '{
            eventQueue.enqueue(completion);
          } ) );
        }
        
        popup.addPopupMenuListener( new function() extends swing.event.PopupMenuListener() {
          
          public function popupMenuCanceled(evt)
          {
            eventQueue.enqueue("$cancel$");
            popupVisible = false;
          }
          
          public function popupMenuWillBecomeInvisible(evt)
          {
            popupVisible = false;
          }
          public function popupMenuWillBecomeVisible(evt)
          {
            popupVisible = true;
          }
          
          
        }() );
        
        // show popup:
        var off = console.getInputHandler().getOffset();
        var p   = console.toPoint(off);
        popup.show( console, p.x, p.y );
        
        var result = eventQueue.dequeue();
        if( result != "$cancel$" )
          longestCompletion = result;
      }
      
      lastBuf  = buf;
      lastTime = System.currentTimeMillis();
      
      if( (longestCompletion != null) && 
          (longestCompletion.length() > match.length() ) )
        return longestCompletion.substring( match.length() );
    }
    
    return "";
  }
  
  /**
   * Is the completer currently showing a popup?
   */
  public function isPopupVisible()
  {
    return popupVisible;
  }
}
