/******************************************************************************
 *     Copyright Texas Instruments 2004. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
  
const var KEYWORD_LIST = [ "var", "function", "for", "while", "break", "continue", "return", "synchronized", "try", "catch", "finally", "throw", "if", "else" ];
const var ev = oscript.OscriptInterpreter.__eval;

const var FILTERS = [
  /^bop/,
  /^uop/,
  /^castTo/,
  /^callAs/,
  /^unhand$/,
  /^memberSet$/,
  /^getMember$/
];

/**
 * A <code>ConsoleTabCompleter</code> that completes based on variables/members
 * in the the scope
 */
public function ScriptConsoleTabCompleter( console, scope )
  extends pkg.util.ConsoleTabCompleter(console)
{
  protected function getCompletionSet(buf)
  {
    // first check for ".something":
    var result = buf.match(/[a-zA-Z_][a-zA-Z_0-9]*(\.\ *([a-zA-Z_][a-zA-Z_0-9]*)?)*$/);
    
    if( result.length() > 0 )
    {
      var match = result[0];
      var completionSet = new pkg.util.CompletionSet();
      
      var idx;
      if( (idx=match.lastIndexOf(".")) != -1 )
      {
        try
        {
          var obj = ev( match.substring(0,idx) + ";", scope );
          match = (idx+1 < match.length()) ? match.substring(idx+1) : "";
          var filtered = false;
          
          for( var item : obj.memberSet() )
          {
            for( var filter : FILTERS )
            {
              filtered = filter.test( (String) item );
              if(filtered)
                break;
            }
            if(!filtered)
              completionSet.add(item);
          }
        }
        catch(e)
        { /* ignore */ }
      }
      else
      {
        (function(scope){
          
          completionSet.addAll( (scope == null) ? KEYWORD_LIST : pkg.system.debugger.memberSet(scope) );
          
          if( scope != null )
            callee( pkg.system.debugger.getPreviousScope(scope) );
          
        })(scope);
      }
      
      return [ completionSet, match ];
    }
    return null;
  }
}
