/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2001-2002.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * Add an item to the menu bar.  The full path to the item to
 * add is the <code>path</code> + <code>name</code>, in other
 * words  "File -> Open -> Log File" would have the <code>path</code>
 * "File/Open", and <code>name</code> "Log File".
 *
 * @param path         the path to item to add, ie. "File/Open"
 * @param name         name of item to add, ie. "Log File"
 * @param accel        the accelerator key, a single character string (optional)
 * @param fxn          the function to call when item is selected
 */
public function addMenuBarItem( path, name, args... )
{
  var a;
  var fxn;
  var accel = null;
  
  if( args.length() == 1 )
  {
    fxn = args[0];
  }
  else if( args.length() == 2 )
  {
    accel = args[0];
    fxn   = args[1];
  }
  else
  {
    throw new Exception("wrong number of args");
  }
  
  if( name == null )
  {
    a = null;
  }
  else
  {
    a = new pkg.util.ActionAdapter( name, null, null, fxn );
    
    if( accel != null )
    {
      var ks = pkg.util.getAcceleratorKeyStroke(accel);
      a.putValue( swing.Action.ACCELERATOR_KEY, ks );
    }
  }
  
  services["window manager"].addMenuBarItem( path, a );
}


/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

