/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2003.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

mixin ti.chimera.registry;

var ctm = java.lang.System.currentTimeMillis;

/**
 * Utilities common to "add-on" plugins.  Essentially this is for plugins that
 * the user may want to be able to enable/disable.  The plugin must implement
 * a service with a <code>setEnabled(enabled)</code> methods, which gets called
 * when the plugin is enabled/disabled.
 * <p>
 * This API is deprecated in favor or <code>pkg.registry.deferredRegister</code>
 * and optionally <code>pkg.util.ActiveStateResource</code>
 */
public var addon = new function() {
  
  /**
   * Register a plugin/service which can be enabled/disabled.
   * 
   * @param default   should be enabled by default?
   * @param name      the service name
   * @param desc      description of the feature being enabled/disabled
   * @param fxn       (optional) function() to call to actually load and
   *    register the appropriate plugin the first time it is enabled. 
   *    This lets you defer loading the plugin until it is enabled
   */
  public function register( default, name, desc, args... )
  {
    var fxn  = (args.length() == 0) ? null : args[0];
    var path = "/Preferences/add ons/" + name;
    var isEnabled = false;
    var isLoaded  = false;
    
    if( ! registry.exists(path) )
    {
      registry.link( 
        new PersistentNode( 
          default,
          NodeContract.BOOLEAN_CONTRACT,
          desc ),
        path );
      
      registry.subscribeToValue( 
        path, 
        null,
        new ti.chimera.registry.SwingNodeSubscriber(
          function( node, enabled ) {
            if( enabled && !isLoaded && (fxn != null) ) {
              fxn();
              isLoaded = true;
            }          
            if( enabled != isEnabled ) {
              services[name].setEnabled(enabled);
              isEnabled = enabled;
            }
          }
        )
      );
    }
  }
  
  /**
   * Is the plugin/service specified by <code>name</code> currently enabled?
   * 
   * @param name   the service name
   * @return <code>true</code> if enabled, else <code>false</code>
   */
  public function isEnabled(name)
  {
    var path = "/Preferences/add ons/" + name;
    return registry.exists(path) && registry.resolve(path).getValue();
  }
  
  /**
   * Programatically enable/disable a plugin
   * 
   * @param name     the service name
   * @param enabled  is the add-on enabled?
   */
  public function setEnabled( name, enabled )
  {
    registry.resolve("/Preferences/add ons/" + name).setValue(enabled);
  }
  
}();
