/*=============================================================================
 *     Copyright Texas Instruments, Inc., 2001-2002.  All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * Given a resource name (ie a filename), load that resource using the
 * specified CLASSPATH.  This includes jar/zip files.
 * <p>
 * You should avoid using this if possible, and rather use the
 * {@link pkg.fs#resolve}.  This function should only be used in 
 * places where java code requires a URL, and that should be 
 * avoided if possible, unless this function can be made to
 * work with other fileystem types mounted into the abstract fs, b/c
 * it would be confusing to the user if some parts of the application
 * cannot see certain files, etc.
 * 
 * @param urlStr       the name of the resource
 * @return a java.net.URL
 */
public function getSystemResource(urlStr)
{
  if( urlStr != null )
  {
    var url = oscript.OscriptInterpreter.getClassLoader().getResource(urlStr);

    if( url == null )
    {
      try
      {
        url = new java.net.URL(urlStr);
      }
      catch(e)
      {
        // must not be a valid URL string
        main.error("could not resolve: " + urlStr);
      }
    }

    return url;
  }
  else
  {
    return null;
  }
}


/*
 *   Local Variables:
 *   tab-width: 2
 *   indent-tabs-mode: nil
 *   mode: java
 *   c-indentation-style: java
 *   c-basic-offset: 2
 *   eval: (c-set-offset 'statement-cont '0)
 *   eval: (c-set-offset 'substatement-open '0)
 *   eval: (c-set-offset 'case-label '+)
 *   eval: (c-set-offset 'inclass '+)
 *   eval: (c-set-offset 'inline-open '0)
 *   End:
 */

