/******************************************************************************
 *        Copyright Texas Instruments 2003. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



/**
 * A utility function to construct a console, wire up cut/paste, install
 * color support, etc.
 * 
 * @param view      the view the console will be displayed in
 * @param rows      the number of rows
 * @param cols      the number of columns
 * @param readonly  <code>true</code> if this console is read-only
 * @return the console
 */
public function makeConsole( view, rows, cols, readonly )
{
  var console = new ti.swing.ConsoleTextArea( rows, cols, readonly );
  console.setRequestFocusEnabled(true);
  
  /* 
   * Subscribe to console font:
   */
  pkg.util.subscribeToValue(
    view,
    "/Preferences/System/Console Font", 
    new ti.chimera.registry.TypeNodeContract(java.awt.Font),
    new ti.chimera.registry.SwingNodeSubscriber( function( node, font ) {
      console.setFont(font);
    } )
  );
  
  /* 
   * install color console support:
   */
  console.setInputHandler( new ti.swing.console.ColorInputHandler(console) );
  
  /* 
   * create actions:
   */
  private var am = new pkg.util.ActionManager();
  
  function createAction( name, accel, fxn )
  {
    function efxn(evt) { fxn(); }
    
    var a = new pkg.util.ActionAdapter( name, null, null, efxn );
    
    if( accel != null )
    {
      console.getInputMap().put( pkg.util.getAcceleratorKeyStroke(accel), name );
      console.getActionMap().put( name, a );
    }
    
    view.addAction(a);
    
    am.addGlobalAction( true, "/Edit/" + name,  null, null, null, null, accel, efxn );
  }
  
  createAction( "Copy",  "C", console.copy );
  if(!readonly)
    createAction( "Paste", "V", console.paste );
  
  /* 
   * subscribe to in-focus view:
   */
  pkg.util.subscribeToValue(
    view,
    "/Docks/selected dock/selected view", null,
    function( node, selectedView ) {
      am.setEnabled( selectedView == view );
  } );
  
  return console;
}

