/******************************************************************************
 *     Copyright Texas Instruments 2004. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */



var licenseUrlTable = new java.util.Hashtable();

{
  mixin licenseUrlTable;
  put( "LGPL", "lgpl.html" );
  put( "GPL",  "gpl.html" );
  put( "DEVELOPMENTOR", "DevelopMentor.html" );
  put( "APACHE", "apache.html" );
}

/**
 * Since we don't want to hard-code all 3rd party library data (name,
 * description, home page, license, etc) in chimera.jar (which is shared 
 * between multiple applications), and we don't want to have to replicate 
 * that data in each application in the cases of libs used by chimera.jar 
 * or other shared jars, so instead at run-time we build a table of lib
 * data, mapping from a .jar filesname to data describing that library.
 * <p>
 * Eventually, once we have our own ClassLoader (to do dependency and
 * version checking based on fields in MANIFEST.MF), we can embed all
 * the needed library data in the .jar file, and get rid of the need to
 * register library data explicitly.
 * 
 * @param name       the name of the library
 * @param jarFile    the name of the jar file (ex: "oscript.jar")
 * @param home       either a homepage URL, or text describing how to 
 *    get the latest version of the library.  If the string begins
 *    with "ftp:" or "http:" it is interpreted as a URL
 * @param desc       description of what the library is
 * @param license    the name of the license (ex: "LGPL")
 */
public function registerLibrary( name, jarFile, home, desc, license, copyright, authors... )
{
  public static var libList = new java.util.LinkedList();
  
  license = license.toUpperCase();
  
  if( (home.indexOf("http://") == 0) ||
      (home.indexOf("ftp://") == 0) )
  {
    home = "<a href=\"" + home + "\">" + home + "</a>";
  }
  
  if( licenseUrlTable.get(license) != null )
  {
    license = "<a href=\"" + licenseUrlTable.get(license) + "\">" + license + "</a>";
  }
  
  libList.add( new (function() {
    
    public function getName()
    {
      return name;
    }
    
    public function getJarFile()
    {
      return jarFile;
    }
    
    public function getHome()
    {
      return home;
    }
    
    public function getDescription()
    {
      return desc;
    }
    
    public function getLicenseLink()
    {
      return license;
    }
    
    public function getCopyrightNotice()
    {
      return copyright;
    }
    
  })() );
}

registerLibrary( "ObjectScript",
                 "oscript.jar", 
                 "http://objectscript.sourceforge.net",
                 "The ObjectScript interpreter.",
                 "LGPL",
                 "Copyright &copy; 2000-2005 Texas Instruments, Inc." );

registerLibrary( "JAWIN",
                 "jawin.jar", 
                 "http://jawin.sourceforge.net",
                 "The Java/Win32 integration project.",
                 "DevelopMentor",
                 "Copyright &copy; 2001 DevelopMentor." );

registerLibrary( "Chimera",
                 "chimera.jar", 
                 "http://objectscript.sourceforge.net",
                 "The Chimera Application Framework.",
                 "GPL",
                 "Copyright &copy; 2000-2005 Texas Instruments, Inc." );

registerLibrary( "BCEL",
                 "bcel-5.1.jar", 
                 "http://jakarta.apache.org/bcel",
                 "The Byte Code Engineering Library (BCEL) is used by the ObjectScript interpreter " +
                 "to (among other things) compile script to bytecode that is executed directly by the " +
                 "Java Virtual Machine for faster execution.",
                 "Apache",
                 "Copyright &copy; 2001 The Apache Software Foundation" );


services["talkback"].addReportDataFxn( '{
  
  var str = ("Libraries/Plugins: \n" + 
             "------------------ \n");
  
  for( var lib : pkg.util.registerLibrary.libList )
    str += "   " + lib.getName() + " (" + lib.getJarFile() + ")\n";
  
  return str;
  
} );

