/******************************************************************************
 *        Copyright Texas Instruments 2003. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


pkg.system.declareJavaPackage("jui.layout");
pkg.system.declareJavaPackage("jui.i18n");

public function juiWizard( close, dw )
{
  
  var panel = new swing.JPanel( new java.awt.BorderLayout() );
  
  var model = new jui.JUIModel();
  
  /**
   * Helper to regenerate the src generated by this layout.
   */
  function refresh()
  {
    var sc = model.getLayoutModel().buildSourceCode();
    
    var imports = new java.util.Vector();
    imports.addAll( sc.getImports() );
    imports.add("jui.layout.BoxTreeLayout");
    
    dw.reset();
    
    var year = 1900 + (new java.util.Date()).getYear();
    dw.dumpln("/******************************************************************************");
    dw.dumpln(" *        Copyright " + /*XXXdm.get("copyright") + */ "??? " + year + ". All Rights Reserved.");
    // XXX license
    dw.dumpln(" */");
    dw.dumpln("");
    
    dw.comment("The panel");
    
    var name = dw.getFrame().getFile().getName();
    {
      var idx = name.lastIndexOf(".");
      if( idx != -1 )
        name = name.substring( 0, idx );
    }
    
    dw.dumpln("public function " + name + "()");
    dw.dumpln("  extends swing.JPanel()");
    dw.dumpln("{");
    dw.indent(+1);
    
    // write imports:
    for( var itr=imports.iterator(); itr.hasNext(); )
    {
      var fullClassName = itr.next();
      var idx = fullClassName.lastIndexOf(".");
      dw.dumpln("const var " + fullClassName.substring(idx+1) + " = " + fullClassName + ";");
    }
    dw.dumpln("");
    
    // write members:
    for( var itr=sc.getMemberVariables().iterator(); itr.hasNext(); )
    {
      dw.dumpln("");
      var mv = itr.next();
      dw.dumpln("public const var " + mv.getName() + ";");
      for( var itr=mv.getInitMethod().getLines().iterator(); itr.hasNext(); )
      {
        dw.dumpln( itr.next() );
      }
      dw.dumpln("add(" + mv.getName() + ");");
    }
    dw.dumpln("");
    
    dw.dumpln("setLayout( new BoxTreeLayout( " + sc.getText() + " ) );");
    dw.dumpln("revalidate();");
    
    dw.indent(-1);
    dw.dumpln("}");
    dw.dumpln("");
    
    dw.flush();
  }
  
  /****************************************************************************
   * Split-pane contains the JUIPanel and 
   */
  
  var propertiesPanel = new jui.LayoutPropertiesPanel( 
    model.getLayoutModel(), 
    model.getStatusModel(),
    new jui.i18n.I18nTableModel()
  );
  var layoutPanel = new jui.LayoutPanel(
    model.getLayoutModel(), 
    model.getStatusModel() 
  );
  
  var desktopPane = new swing.JDesktopPane();
  desktopPane.setSize(300,300);
  function addPanelToDesktop(p)
  {
    var f = new swing.JInternalFrame( "", true, false, false );
    f.getContentPane().add(p);
    desktopPane.add(f);
    f.setSize(128,128);
    f.setVisible(true);
  }
  addPanelToDesktop(layoutPanel);
  
  var splitPane = new swing.JSplitPane( swing.JSplitPane.HORIZONTAL_SPLIT,
                                        propertiesPanel, desktopPane );
  
  panel.add( java.awt.BorderLayout.CENTER, splitPane );
  
  
  
  /****************************************************************************
   * Button panel at bottom of dialog
   */
  var buttonPanel = new swing.JPanel();
  
  var generateSrcButton = new swing.JButton("Generate Src");
  generateSrcButton.addActionListener( function(evt) {
    refresh();
  } );
  buttonPanel.add(generateSrcButton);
  
  var closeButton = new swing.JButton("Close");
  closeButton.addActionListener( function(evt) {
    close();
  } );
  buttonPanel.add(closeButton);
  
  buttonPanel.add( new swing.JLabel("(Right click over panel to add components)") );
  
  panel.add( java.awt.BorderLayout.SOUTH, buttonPanel );
  
  /***************************************************************************/
  
  return panel;
  
}

