/******************************************************************************
 *        Copyright Texas Instruments 2003. All Rights Reserved.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/**
 * A wizard for creating a new plugin... see services["editor"].registerWizard
 */
public function newPluginWizard( close, dw  )
{
  
  return new function() extends pkg.wizard.Wizard() {
    
    //setImage("editor-wizard.gif");
    
    /**
     * This function is responsible for generating the src code for the plugin 
     * based on data collected from the user.
     */
    private function refresh()
    {
      var dm = getDataManager();
      
      //writeln("dm=" + dm);
      
      if( dm.get("name") != null )
      {
        dw.reset();
        
        var year = 1900 + (new java.util.Date()).getYear();
        
        dw.dumpln("/******************************************************************************");
        dw.dumpln(" *        Copyright " + dm.get("copyright") + " " + year + ". All Rights Reserved.");
        // XXX license
        dw.dumpln(" */");
        dw.dumpln("");
        dw.dumpln("{");
        dw.dumpln("");
        
        dw.comment( dm.get("comment") );
        
        dw.dumpln("registry.register( new function() extends ti.chimera.Plugin( main, \"" + dm.get("name") + "\" ) {");
        dw.indent(+1);
        dw.dumpln("");
        
        if( dm.get("resources.count") != null )
        {
          for( var i=0; i<dm.get("resources.count"); i++ )
          {
            // XXX
          }
        }
        
        for( var itr=dm.subkeys("services"); itr.hasNext(); )
        {
          var name = itr.next();
          var basename = "services." + name;
          
          dw.comment( dm.get(basename + ".comment") );
          dw.dumpln("registerServiceFactory( function() {");
          dw.indent(+1);
          dw.dumpln("return new function() extends ti.chimera.Service(\"" + name + "\") {");
          dw.indent(+1);
          dw.dumpln("");
          
          for( var mitr=dm.subkeys(basename + ".methods"); mitr.hasNext(); )
          {
            var methodName = mitr.next();
            var methodComment = "";
            
            for( var pitr=dm.subkeys(basename + ".methods." + methodName + ".params"); pitr.hasNext(); )
            {
              var paramName = pitr.next();
              methodComment = "\n@param " + paramName + "  " + dm.get(basename + ".methods." + methodName + ".params." + paramName + ".comment") + methodComment;
            }
            
            var returns = dm.get(basename + ".methods." + methodName + ".returns");
            if( returns != "" )
              methodComment += "\n@return " + returns;
            
            dw.comment( dm.get(basename + ".methods." + methodName + ".comment") + "\n" + methodComment );
            
            var paramsStr = "";
            var num = 0;
            for( var pitr=dm.subkeys(basename + ".methods." + methodName + ".params"); pitr.hasNext(); )
            {
              if( num == 0 )
                paramsStr = pitr.next();
              else
                paramsStr = pitr.next() + ", " + paramsStr;
              num++;
            }
            
            if( num > 1 )
              paramsStr = " " + paramsStr + " ";
            
            dw.dumpln("public function " + methodName + "(" + paramsStr + ")");
            dw.dumpln("{}");
            dw.dumpln("");
          }
          
          dw.indent(-1);
          dw.dumpln("}();");
          dw.indent(-1);
          dw.dumpln("} );");
          dw.dumpln("");
        }
        
        for( var itr=dm.subkeys("views"); itr.hasNext(); )
        {
          var name = itr.next();
          var basename = "views." + name;
          
          var iconStr = "null";
          if( dm.get(basename + ".icon") != null )
            iconStr = "pkg.util.getImageIcon(\"" + dm.get(basename + ".icon") + "\")";
          
          dw.comment( dm.get(basename + ".comment") );
          dw.dumpln("function " + name + "(name)");
          dw.dumpln("  extends ti.chimera.View( main, name, " + iconStr + " )");
          dw.dumpln("{");
          dw.indent(+1);
          dw.dumpln("// private var component = ...;");
          dw.dumpln("");
          dw.comment("Get the component that makes up this view.  The view must always implement this method.\n\n@return any swing component will do");
          dw.dumpln("public function getComponent()");
          dw.dumpln("{");
          dw.dumpln("  return component;");
          dw.dumpln("}");
          dw.indent(-1);
          dw.dumpln("}");
          dw.dumpln("");
        }
        
        dw.indent(-1);
        dw.dumpln("}() );");
        dw.dumpln("");
        dw.dumpln("}");
        dw.dumpln("");
        
        dw.flush();
      }
    }
    
    protected function doCancel()
    {
      close();
    }
    
    protected function doFinish()
    {
      close();
    }
    
    private var _doNext = doNext;
    protected function doNext()
    {
      _doNext();
      refresh();
    }
    
    private var _doBack = doBack;
    protected function doBack()
    {
      _doBack();
      refresh();
    }
    
    /**
     * Screen for gathering general information pertaining to all plugins
     */
    private function GeneralInfo(dm)
      extends pkg.wizard.GenericWizardPanel( dm, "General Info", "enter general information about the plugin", false, false  )
    {
      addTextField( "name", "Plugin Name", null, true );
      addTextField( "copyright", "Copyright Holder", "Texas Instruments", true );
//      addRadioButtons( "license", "Software License", [ "GPL", "LGPL", "other" ], "other", true );
      addTextArea( "comment", "Plugin Description", null, true );
    }
    
    /**
     * A screen with controls to let the user add another resource, service,
     * or view.  This screen will setup a new state-sequence (returning to
     * another instance of this same screen) based on what the user decides
     * to add.  This screen does not actually collect any data itself.
     */
    private function AddMore(dm)
      extends pkg.wizard.WizardPanel( dm, "Add More?", "You can choose to add additional resources, services, or views.", true  )
    {
      var buttonPanel = new swing.JPanel( new java.awt.GridLayout( 0, 1 ) );
      var buttonGroup = new swing.ButtonGroup();
      
      function addChoice( name, seq )
      {
        var rb;
        
        rb = new swing.JRadioButton(name);
        rb.addActionListener( function(evt) {
          setStateSequence(seq);
        } );
        
        buttonPanel.add(rb);
        buttonGroup.add(rb);
      }
      
      //addChoice( "Resource", [] );
      addChoice( "Service",  serviceStateSequence );
      addChoice( "View",     viewStateSequence );
      
      add(buttonPanel);
      
      /**
       * Always returns <code>true</code>, since this screen does not actually
       * collect any data.
       * 
       * @return <code>false</code> if user must enter more data, otherwise if
       *    <code>true</code> if the user can proceed to the next screen
       */
      public function isValid()
      {
        return true;
      }
    }
    
    /**
     * A service to prompt for a service name, which is stored as the property
     * "serviceName".
     */
    private function PromptForServiceName(dm)
      extends pkg.wizard.GenericWizardPanel( dm, "Enter Service Name", "Enter the name for the service.  The name may contain whitespace, or any characters other than \"/\".  This name can be used by other parts of the system, such as other plugins, to refer to this service.", false, false )
    {
      addTextField( "serviceName", "Service Name", null, true );
      addTextArea( "serviceComment", "Service Description", null, true );
      
      public function leavePanel()
      {
        var basename = "services." + dm.get("serviceName");
        dm.put( basename + ".comment", dm.get("serviceComment") );
      }
    }
    
    /**
     * Screen to let the user add more methods to the service.
     */
    private function AddMoreMethods(dm)
      extends pkg.wizard.WizardPanel( dm, "Add More?", "You can choose to add additional methods to the service.", false  )
    {
      var basename = "services." + dm.get("serviceName");
      
      var panel = new swing.JPanel( new java.awt.BorderLayout() );
      
      var list = new swing.JList( new swing.DefaultListModel() );
      for( var mitr=dm.subkeys(basename + ".methods"); mitr.hasNext(); )
        list.getModel().addElement( mitr.next() );
      list.addListSelectionListener( function(evt) {
        // XXX finish me
      } );
      var sp = new swing.JScrollPane(list);
      sp.setBorder( new swing.border.TitledBorder("Methods") );
      panel.add(sp);
      
      var button = new swing.JButton("Add Method");
      button.addActionListener( function(evt) {
        setStateSequence(serviceAddMethodStateSequence);
      } );
      panel.add( java.awt.BorderLayout.SOUTH, button );
      
      add(panel);
      
      /**
       * Always returns <code>true</code>, since this screen does not actually
       * collect any data.
       * 
       * @return <code>false</code> if user must enter more data, otherwise if
       *    <code>true</code> if the user can proceed to the next screen
       */
      public function isValid()
      {
        return true;
      }
    }
    
    /**
     * Screen to collect info about a method to add to the service.
     */
    private function PromptForMethodName(dm)
      extends pkg.wizard.GenericWizardPanel( dm, "Enter Method Name", "Enter the name for the method.  The name should be a valid identifier, and not contain whitespace, etc.  The naming convention is to use mixed case method names starting with a lower case, for example doSomething, or doSomethingElse.", false, false )
    {
      addTextField( "methodName", "Method Name", null, true );
      addTextField( "methodReturns", "Return Value", "", false );
      addTextArea( "methodComment", "Method Description", null, true );
      
      public function leavePanel()
      {
        var basename = "services." + dm.get("serviceName") + ".methods." + dm.get("methodName");
        dm.put( basename + ".comment", dm.get("methodComment") );
        if( dm.get("methodReturns") != null )
          dm.put( basename + ".returns", dm.get("methodReturns") );
      }
    }
    
    /**
     * Screen to let the user add more parameters to the method.
     */
    private function AddMoreParams(dm)
      extends pkg.wizard.WizardPanel( dm, "Add More?", "You can choose to add additional parameters to the view.", false  )
    {
      var basename = "services." + dm.get("serviceName") + ".methods." + dm.get("methodName");
      
      var panel = new swing.JPanel( new java.awt.BorderLayout() );
      
      var list = new swing.JList( new swing.DefaultListModel() );
      for( var pitr=dm.subkeys(basename + ".params"); pitr.hasNext(); )
        list.getModel().addElement( pitr.next() );
      list.addListSelectionListener( function(evt) {
        // XXX finish me
      } );
      var sp = new swing.JScrollPane(list);
      sp.setBorder( new swing.border.TitledBorder("Parameters") );
      panel.add(sp);
      
      var button = new swing.JButton("Add Parameter");
      button.addActionListener( function(evt) {
        setStateSequence(serviceAddMethodParamStateSequence);
      } );
      panel.add( java.awt.BorderLayout.SOUTH, button );
      
      add(panel);
      
      /**
       * Always returns <code>true</code>, since this screen does not actually
       * collect any data.
       * 
       * @return <code>false</code> if user must enter more data, otherwise if
       *    <code>true</code> if the user can proceed to the next screen
       */
      public function isValid()
      {
        return true;
      }
    }
    
    /**
     * Screen to collect info about a method to add to the service.
     */
    private function PromptForParamName(dm)
      extends pkg.wizard.GenericWizardPanel( dm, "Enter Parameter Name", "Enter the name for the parameter.  The name should be a valid identifier, and not contain whitespace, etc.  The naming convention is to use mixed case parameter names starting with a lower case, for example someParam, or someOtherParam.", false, false )
    {
      addTextField( "paramName", "Parameter Name", null, true );
      addTextArea( "paramComment", "Parameter Description", null, true );
      
      public function leavePanel()
      {
        var basename = "services." + dm.get("serviceName") + ".methods." + dm.get("methodName");
        dm.put( basename + ".params." + dm.get("paramName") + ".comment", dm.get("paramComment") );
      }
    }
    
    /**
     * Screen to collect info about a view.
     */
    private function PromptForViewName(dm)
      extends pkg.wizard.GenericWizardPanel( dm, "Enter View Name", "Enter the name for the view.  The name should be a valid identifier, and not contain whitespace, etc.  The naming convention is to use mixed case class names starting with a upper case, for example SomeView, or SomeOtherView.  You can also, optionally, specify an icon name to show in the view's tab", false, false )
    {
      addTextField( "viewName", "View Name", null, true );
      addTextArea( "viewComment", "View Description", null, true );
      addTextField( "viewIcon", "View Icon", null, false );
      
      public function leavePanel()
      {
        var basename = "views." + dm.get("viewName");
        dm.put( basename + ".comment", dm.get("viewComment") );
        if( dm.get("viewIcon") != null )
          dm.put( basename + ".icon", dm.get("viewIcon") );
      }
    }
    
    var serviceStateSequence = [
      function(dm) { return new PromptForServiceName(dm); },
      function(dm) { return new AddMoreMethods(dm); },
      function(dm) { return new AddMore(dm); }
    ];
    
    var serviceAddMethodStateSequence = [
      function(dm) { return new PromptForMethodName(dm); },
      function(dm) { return new AddMoreParams(dm); },
      function(dm) { return new AddMoreMethods(dm); },
      function(dm) { return new AddMore(dm); }
    ];
    
    var serviceAddMethodParamStateSequence = [
      function(dm) { return new PromptForParamName(dm); },
      function(dm) { return new AddMoreParams(dm); },
      function(dm) { return new AddMoreMethods(dm); },
      function(dm) { return new AddMore(dm); }
    ];
    
    var viewStateSequence = [
      function(dm) { return new PromptForViewName(dm); },
      function(dm) { return new AddMore(dm); }
    ];
    
    setStateSequence( [
      function(dm) { return new GeneralInfo(dm); },
      function(dm) { return new AddMore(dm); }
    ] );
  }();
  
  
}
